#pragma once

#include "Instruction.hpp"

#include <asmjit/asmjit.h>
#include <optional>

namespace Dotx64Dbg
{

    inline uint32_t convertAsmJitMnemonic(Mnemonic mnemonic)
    {
        switch (mnemonic)
        {
        case Mnemonic::Movsd:
            return asmjit::x86::Inst::kIdMovsd;
        case Mnemonic::Aaa:
            return asmjit::x86::Inst::kIdAaa;
        case Mnemonic::Aad:
            return asmjit::x86::Inst::kIdAad;
        case Mnemonic::Aam:
            return asmjit::x86::Inst::kIdAam;
        case Mnemonic::Aas:
            return asmjit::x86::Inst::kIdAas;
        case Mnemonic::Adc:
            return asmjit::x86::Inst::kIdAdc;
        case Mnemonic::Adcx:
            return asmjit::x86::Inst::kIdAdcx;
        case Mnemonic::Add:
            return asmjit::x86::Inst::kIdAdd;
        case Mnemonic::Addpd:
            return asmjit::x86::Inst::kIdAddpd;
        case Mnemonic::Addps:
            return asmjit::x86::Inst::kIdAddps;
        case Mnemonic::Addsd:
            return asmjit::x86::Inst::kIdAddsd;
        case Mnemonic::Addss:
            return asmjit::x86::Inst::kIdAddss;
        case Mnemonic::Addsubpd:
            return asmjit::x86::Inst::kIdAddsubpd;
        case Mnemonic::Addsubps:
            return asmjit::x86::Inst::kIdAddsubps;
        case Mnemonic::Adox:
            return asmjit::x86::Inst::kIdAdox;
        case Mnemonic::Aesdec:
            return asmjit::x86::Inst::kIdAesdec;
        case Mnemonic::Aesdeclast:
            return asmjit::x86::Inst::kIdAesdeclast;
        case Mnemonic::Aesenc:
            return asmjit::x86::Inst::kIdAesenc;
        case Mnemonic::Aesenclast:
            return asmjit::x86::Inst::kIdAesenclast;
        case Mnemonic::Aesimc:
            return asmjit::x86::Inst::kIdAesimc;
        case Mnemonic::Aeskeygenassist:
            return asmjit::x86::Inst::kIdAeskeygenassist;
        case Mnemonic::And:
            return asmjit::x86::Inst::kIdAnd;
        case Mnemonic::Andn:
            return asmjit::x86::Inst::kIdAndn;
        case Mnemonic::Andnpd:
            return asmjit::x86::Inst::kIdAndnpd;
        case Mnemonic::Andnps:
            return asmjit::x86::Inst::kIdAndnps;
        case Mnemonic::Andpd:
            return asmjit::x86::Inst::kIdAndpd;
        case Mnemonic::Andps:
            return asmjit::x86::Inst::kIdAndps;
        case Mnemonic::Arpl:
            return asmjit::x86::Inst::kIdArpl;
        case Mnemonic::Bextr:
            return asmjit::x86::Inst::kIdBextr;
        case Mnemonic::Blcfill:
            return asmjit::x86::Inst::kIdBlcfill;
        case Mnemonic::Blci:
            return asmjit::x86::Inst::kIdBlci;
        case Mnemonic::Blcic:
            return asmjit::x86::Inst::kIdBlcic;
        case Mnemonic::Blcmsk:
            return asmjit::x86::Inst::kIdBlcmsk;
        case Mnemonic::Blcs:
            return asmjit::x86::Inst::kIdBlcs;
        case Mnemonic::Blendpd:
            return asmjit::x86::Inst::kIdBlendpd;
        case Mnemonic::Blendps:
            return asmjit::x86::Inst::kIdBlendps;
        case Mnemonic::Blendvpd:
            return asmjit::x86::Inst::kIdBlendvpd;
        case Mnemonic::Blendvps:
            return asmjit::x86::Inst::kIdBlendvps;
        case Mnemonic::Blsfill:
            return asmjit::x86::Inst::kIdBlsfill;
        case Mnemonic::Blsi:
            return asmjit::x86::Inst::kIdBlsi;
        case Mnemonic::Blsic:
            return asmjit::x86::Inst::kIdBlsic;
        case Mnemonic::Blsmsk:
            return asmjit::x86::Inst::kIdBlsmsk;
        case Mnemonic::Blsr:
            return asmjit::x86::Inst::kIdBlsr;
        case Mnemonic::Bndcl:
            return asmjit::x86::Inst::kIdBndcl;
        case Mnemonic::Bndcn:
            return asmjit::x86::Inst::kIdBndcn;
        case Mnemonic::Bndcu:
            return asmjit::x86::Inst::kIdBndcu;
        case Mnemonic::Bndldx:
            return asmjit::x86::Inst::kIdBndldx;
        case Mnemonic::Bndmk:
            return asmjit::x86::Inst::kIdBndmk;
        case Mnemonic::Bndmov:
            return asmjit::x86::Inst::kIdBndmov;
        case Mnemonic::Bndstx:
            return asmjit::x86::Inst::kIdBndstx;
        case Mnemonic::Bound:
            return asmjit::x86::Inst::kIdBound;
        case Mnemonic::Bsf:
            return asmjit::x86::Inst::kIdBsf;
        case Mnemonic::Bsr:
            return asmjit::x86::Inst::kIdBsr;
        case Mnemonic::Bswap:
            return asmjit::x86::Inst::kIdBswap;
        case Mnemonic::Bt:
            return asmjit::x86::Inst::kIdBt;
        case Mnemonic::Btc:
            return asmjit::x86::Inst::kIdBtc;
        case Mnemonic::Btr:
            return asmjit::x86::Inst::kIdBtr;
        case Mnemonic::Bts:
            return asmjit::x86::Inst::kIdBts;
        case Mnemonic::Bzhi:
            return asmjit::x86::Inst::kIdBzhi;
        case Mnemonic::Call:
            return asmjit::x86::Inst::kIdCall;
        case Mnemonic::Cbw:
            return asmjit::x86::Inst::kIdCbw;
        case Mnemonic::Cdq:
            return asmjit::x86::Inst::kIdCdq;
        case Mnemonic::Cdqe:
            return asmjit::x86::Inst::kIdCdqe;
        case Mnemonic::Clac:
            return asmjit::x86::Inst::kIdClac;
        case Mnemonic::Clc:
            return asmjit::x86::Inst::kIdClc;
        case Mnemonic::Cld:
            return asmjit::x86::Inst::kIdCld;
        case Mnemonic::Clflush:
            return asmjit::x86::Inst::kIdClflush;
        case Mnemonic::Clflushopt:
            return asmjit::x86::Inst::kIdClflushopt;
        case Mnemonic::Cli:
            return asmjit::x86::Inst::kIdCli;
        case Mnemonic::Clts:
            return asmjit::x86::Inst::kIdClts;
        case Mnemonic::Clwb:
            return asmjit::x86::Inst::kIdClwb;
        case Mnemonic::Clzero:
            return asmjit::x86::Inst::kIdClzero;
        case Mnemonic::Cmc:
            return asmjit::x86::Inst::kIdCmc;
        case Mnemonic::Cmovb:
            return asmjit::x86::Inst::kIdCmovb;
        case Mnemonic::Cmovbe:
            return asmjit::x86::Inst::kIdCmovbe;
        case Mnemonic::Cmovl:
            return asmjit::x86::Inst::kIdCmovl;
        case Mnemonic::Cmovle:
            return asmjit::x86::Inst::kIdCmovle;
        case Mnemonic::Cmovnb:
            return asmjit::x86::Inst::kIdCmovnb;
        case Mnemonic::Cmovnbe:
            return asmjit::x86::Inst::kIdCmovnbe;
        case Mnemonic::Cmovnl:
            return asmjit::x86::Inst::kIdCmovnl;
        case Mnemonic::Cmovnle:
            return asmjit::x86::Inst::kIdCmovnle;
        case Mnemonic::Cmovno:
            return asmjit::x86::Inst::kIdCmovno;
        case Mnemonic::Cmovnp:
            return asmjit::x86::Inst::kIdCmovnp;
        case Mnemonic::Cmovns:
            return asmjit::x86::Inst::kIdCmovns;
        case Mnemonic::Cmovnz:
            return asmjit::x86::Inst::kIdCmovnz;
        case Mnemonic::Cmovo:
            return asmjit::x86::Inst::kIdCmovo;
        case Mnemonic::Cmovp:
            return asmjit::x86::Inst::kIdCmovp;
        case Mnemonic::Cmovs:
            return asmjit::x86::Inst::kIdCmovs;
        case Mnemonic::Cmovz:
            return asmjit::x86::Inst::kIdCmovz;
        case Mnemonic::Cmp:
            return asmjit::x86::Inst::kIdCmp;
        case Mnemonic::Cmppd:
            return asmjit::x86::Inst::kIdCmppd;
        case Mnemonic::Cmpps:
            return asmjit::x86::Inst::kIdCmpps;
        case Mnemonic::Cmpsd:
            return asmjit::x86::Inst::kIdCmpsd;
        case Mnemonic::Cmpss:
            return asmjit::x86::Inst::kIdCmpss;
        case Mnemonic::Cmpxchg:
            return asmjit::x86::Inst::kIdCmpxchg;
        case Mnemonic::Cmpxchg16b:
            return asmjit::x86::Inst::kIdCmpxchg16b;
        case Mnemonic::Cmpxchg8b:
            return asmjit::x86::Inst::kIdCmpxchg8b;
        case Mnemonic::Comisd:
            return asmjit::x86::Inst::kIdComisd;
        case Mnemonic::Comiss:
            return asmjit::x86::Inst::kIdComiss;
        case Mnemonic::Cpuid:
            return asmjit::x86::Inst::kIdCpuid;
        case Mnemonic::Cqo:
            return asmjit::x86::Inst::kIdCqo;
        case Mnemonic::Crc32:
            return asmjit::x86::Inst::kIdCrc32;
        case Mnemonic::Cvtdq2pd:
            return asmjit::x86::Inst::kIdCvtdq2pd;
        case Mnemonic::Cvtdq2ps:
            return asmjit::x86::Inst::kIdCvtdq2ps;
        case Mnemonic::Cvtpd2dq:
            return asmjit::x86::Inst::kIdCvtpd2dq;
        case Mnemonic::Cvtpd2pi:
            return asmjit::x86::Inst::kIdCvtpd2pi;
        case Mnemonic::Cvtpd2ps:
            return asmjit::x86::Inst::kIdCvtpd2ps;
        case Mnemonic::Cvtpi2pd:
            return asmjit::x86::Inst::kIdCvtpi2pd;
        case Mnemonic::Cvtpi2ps:
            return asmjit::x86::Inst::kIdCvtpi2ps;
        case Mnemonic::Cvtps2dq:
            return asmjit::x86::Inst::kIdCvtps2dq;
        case Mnemonic::Cvtps2pd:
            return asmjit::x86::Inst::kIdCvtps2pd;
        case Mnemonic::Cvtps2pi:
            return asmjit::x86::Inst::kIdCvtps2pi;
        case Mnemonic::Cvtsd2si:
            return asmjit::x86::Inst::kIdCvtsd2si;
        case Mnemonic::Cvtsd2ss:
            return asmjit::x86::Inst::kIdCvtsd2ss;
        case Mnemonic::Cvtsi2sd:
            return asmjit::x86::Inst::kIdCvtsi2sd;
        case Mnemonic::Cvtsi2ss:
            return asmjit::x86::Inst::kIdCvtsi2ss;
        case Mnemonic::Cvtss2sd:
            return asmjit::x86::Inst::kIdCvtss2sd;
        case Mnemonic::Cvtss2si:
            return asmjit::x86::Inst::kIdCvtss2si;
        case Mnemonic::Cvttpd2dq:
            return asmjit::x86::Inst::kIdCvttpd2dq;
        case Mnemonic::Cvttpd2pi:
            return asmjit::x86::Inst::kIdCvttpd2pi;
        case Mnemonic::Cvttps2dq:
            return asmjit::x86::Inst::kIdCvttps2dq;
        case Mnemonic::Cvttps2pi:
            return asmjit::x86::Inst::kIdCvttps2pi;
        case Mnemonic::Cvttsd2si:
            return asmjit::x86::Inst::kIdCvttsd2si;
        case Mnemonic::Cvttss2si:
            return asmjit::x86::Inst::kIdCvttss2si;
        case Mnemonic::Cwd:
            return asmjit::x86::Inst::kIdCwd;
        case Mnemonic::Cwde:
            return asmjit::x86::Inst::kIdCwde;
        case Mnemonic::Daa:
            return asmjit::x86::Inst::kIdDaa;
        case Mnemonic::Das:
            return asmjit::x86::Inst::kIdDas;
        case Mnemonic::Dec:
            return asmjit::x86::Inst::kIdDec;
        case Mnemonic::Div:
            return asmjit::x86::Inst::kIdDiv;
        case Mnemonic::Divpd:
            return asmjit::x86::Inst::kIdDivpd;
        case Mnemonic::Divps:
            return asmjit::x86::Inst::kIdDivps;
        case Mnemonic::Divsd:
            return asmjit::x86::Inst::kIdDivsd;
        case Mnemonic::Divss:
            return asmjit::x86::Inst::kIdDivss;
        case Mnemonic::Dppd:
            return asmjit::x86::Inst::kIdDppd;
        case Mnemonic::Dpps:
            return asmjit::x86::Inst::kIdDpps;
        case Mnemonic::Emms:
            return asmjit::x86::Inst::kIdEmms;
        case Mnemonic::Enter:
            return asmjit::x86::Inst::kIdEnter;
        case Mnemonic::Extractps:
            return asmjit::x86::Inst::kIdExtractps;
        case Mnemonic::Extrq:
            return asmjit::x86::Inst::kIdExtrq;
        case Mnemonic::F2xm1:
            return asmjit::x86::Inst::kIdF2xm1;
        case Mnemonic::Fabs:
            return asmjit::x86::Inst::kIdFabs;
        case Mnemonic::Fadd:
            return asmjit::x86::Inst::kIdFadd;
        case Mnemonic::Faddp:
            return asmjit::x86::Inst::kIdFaddp;
        case Mnemonic::Fbld:
            return asmjit::x86::Inst::kIdFbld;
        case Mnemonic::Fbstp:
            return asmjit::x86::Inst::kIdFbstp;
        case Mnemonic::Fchs:
            return asmjit::x86::Inst::kIdFchs;
        case Mnemonic::Fcmovb:
            return asmjit::x86::Inst::kIdFcmovb;
        case Mnemonic::Fcmovbe:
            return asmjit::x86::Inst::kIdFcmovbe;
        case Mnemonic::Fcmove:
            return asmjit::x86::Inst::kIdFcmove;
        case Mnemonic::Fcmovnb:
            return asmjit::x86::Inst::kIdFcmovnb;
        case Mnemonic::Fcmovnbe:
            return asmjit::x86::Inst::kIdFcmovnbe;
        case Mnemonic::Fcmovne:
            return asmjit::x86::Inst::kIdFcmovne;
        case Mnemonic::Fcmovnu:
            return asmjit::x86::Inst::kIdFcmovnu;
        case Mnemonic::Fcmovu:
            return asmjit::x86::Inst::kIdFcmovu;
        case Mnemonic::Fcom:
            return asmjit::x86::Inst::kIdFcom;
        case Mnemonic::Fcomi:
            return asmjit::x86::Inst::kIdFcomi;
        case Mnemonic::Fcomip:
            return asmjit::x86::Inst::kIdFcomip;
        case Mnemonic::Fcomp:
            return asmjit::x86::Inst::kIdFcomp;
        case Mnemonic::Fcompp:
            return asmjit::x86::Inst::kIdFcompp;
        case Mnemonic::Fcos:
            return asmjit::x86::Inst::kIdFcos;
        case Mnemonic::Fdecstp:
            return asmjit::x86::Inst::kIdFdecstp;
        case Mnemonic::Fdiv:
            return asmjit::x86::Inst::kIdFdiv;
        case Mnemonic::Fdivp:
            return asmjit::x86::Inst::kIdFdivp;
        case Mnemonic::Fdivr:
            return asmjit::x86::Inst::kIdFdivr;
        case Mnemonic::Fdivrp:
            return asmjit::x86::Inst::kIdFdivrp;
        case Mnemonic::Femms:
            return asmjit::x86::Inst::kIdFemms;
        case Mnemonic::Ffree:
            return asmjit::x86::Inst::kIdFfree;
        case Mnemonic::Fiadd:
            return asmjit::x86::Inst::kIdFiadd;
        case Mnemonic::Ficom:
            return asmjit::x86::Inst::kIdFicom;
        case Mnemonic::Ficomp:
            return asmjit::x86::Inst::kIdFicomp;
        case Mnemonic::Fidiv:
            return asmjit::x86::Inst::kIdFidiv;
        case Mnemonic::Fidivr:
            return asmjit::x86::Inst::kIdFidivr;
        case Mnemonic::Fild:
            return asmjit::x86::Inst::kIdFild;
        case Mnemonic::Fimul:
            return asmjit::x86::Inst::kIdFimul;
        case Mnemonic::Fincstp:
            return asmjit::x86::Inst::kIdFincstp;
        case Mnemonic::Fist:
            return asmjit::x86::Inst::kIdFist;
        case Mnemonic::Fistp:
            return asmjit::x86::Inst::kIdFistp;
        case Mnemonic::Fisttp:
            return asmjit::x86::Inst::kIdFisttp;
        case Mnemonic::Fisub:
            return asmjit::x86::Inst::kIdFisub;
        case Mnemonic::Fisubr:
            return asmjit::x86::Inst::kIdFisubr;
        case Mnemonic::Fld:
            return asmjit::x86::Inst::kIdFld;
        case Mnemonic::Fld1:
            return asmjit::x86::Inst::kIdFld1;
        case Mnemonic::Fldcw:
            return asmjit::x86::Inst::kIdFldcw;
        case Mnemonic::Fldenv:
            return asmjit::x86::Inst::kIdFldenv;
        case Mnemonic::Fldl2e:
            return asmjit::x86::Inst::kIdFldl2e;
        case Mnemonic::Fldl2t:
            return asmjit::x86::Inst::kIdFldl2t;
        case Mnemonic::Fldlg2:
            return asmjit::x86::Inst::kIdFldlg2;
        case Mnemonic::Fldln2:
            return asmjit::x86::Inst::kIdFldln2;
        case Mnemonic::Fldpi:
            return asmjit::x86::Inst::kIdFldpi;
        case Mnemonic::Fldz:
            return asmjit::x86::Inst::kIdFldz;
        case Mnemonic::Fmul:
            return asmjit::x86::Inst::kIdFmul;
        case Mnemonic::Fmulp:
            return asmjit::x86::Inst::kIdFmulp;
        case Mnemonic::Fnclex:
            return asmjit::x86::Inst::kIdFnclex;
        case Mnemonic::Fninit:
            return asmjit::x86::Inst::kIdFninit;
        case Mnemonic::Fnop:
            return asmjit::x86::Inst::kIdFnop;
        case Mnemonic::Fnsave:
            return asmjit::x86::Inst::kIdFnsave;
        case Mnemonic::Fnstcw:
            return asmjit::x86::Inst::kIdFnstcw;
        case Mnemonic::Fnstenv:
            return asmjit::x86::Inst::kIdFnstenv;
        case Mnemonic::Fnstsw:
            return asmjit::x86::Inst::kIdFnstsw;
        case Mnemonic::Fpatan:
            return asmjit::x86::Inst::kIdFpatan;
        case Mnemonic::Fprem:
            return asmjit::x86::Inst::kIdFprem;
        case Mnemonic::Fprem1:
            return asmjit::x86::Inst::kIdFprem1;
        case Mnemonic::Fptan:
            return asmjit::x86::Inst::kIdFptan;
        case Mnemonic::Frndint:
            return asmjit::x86::Inst::kIdFrndint;
        case Mnemonic::Frstor:
            return asmjit::x86::Inst::kIdFrstor;
        case Mnemonic::Fscale:
            return asmjit::x86::Inst::kIdFscale;
        case Mnemonic::Fsin:
            return asmjit::x86::Inst::kIdFsin;
        case Mnemonic::Fsincos:
            return asmjit::x86::Inst::kIdFsincos;
        case Mnemonic::Fsqrt:
            return asmjit::x86::Inst::kIdFsqrt;
        case Mnemonic::Fst:
            return asmjit::x86::Inst::kIdFst;
        case Mnemonic::Fstp:
            return asmjit::x86::Inst::kIdFstp;
        case Mnemonic::Fsub:
            return asmjit::x86::Inst::kIdFsub;
        case Mnemonic::Fsubp:
            return asmjit::x86::Inst::kIdFsubp;
        case Mnemonic::Fsubr:
            return asmjit::x86::Inst::kIdFsubr;
        case Mnemonic::Fsubrp:
            return asmjit::x86::Inst::kIdFsubrp;
        case Mnemonic::Ftst:
            return asmjit::x86::Inst::kIdFtst;
        case Mnemonic::Fucom:
            return asmjit::x86::Inst::kIdFucom;
        case Mnemonic::Fucomi:
            return asmjit::x86::Inst::kIdFucomi;
        case Mnemonic::Fucomip:
            return asmjit::x86::Inst::kIdFucomip;
        case Mnemonic::Fucomp:
            return asmjit::x86::Inst::kIdFucomp;
        case Mnemonic::Fucompp:
            return asmjit::x86::Inst::kIdFucompp;
        case Mnemonic::Fwait:
            return asmjit::x86::Inst::kIdFwait;
        case Mnemonic::Fxam:
            return asmjit::x86::Inst::kIdFxam;
        case Mnemonic::Fxch:
            return asmjit::x86::Inst::kIdFxch;
        case Mnemonic::Fxrstor:
            return asmjit::x86::Inst::kIdFxrstor;
        case Mnemonic::Fxrstor64:
            return asmjit::x86::Inst::kIdFxrstor64;
        case Mnemonic::Fxsave:
            return asmjit::x86::Inst::kIdFxsave;
        case Mnemonic::Fxsave64:
            return asmjit::x86::Inst::kIdFxsave64;
        case Mnemonic::Fxtract:
            return asmjit::x86::Inst::kIdFxtract;
        case Mnemonic::Fyl2x:
            return asmjit::x86::Inst::kIdFyl2x;
        case Mnemonic::Fyl2xp1:
            return asmjit::x86::Inst::kIdFyl2xp1;
        case Mnemonic::Haddpd:
            return asmjit::x86::Inst::kIdHaddpd;
        case Mnemonic::Haddps:
            return asmjit::x86::Inst::kIdHaddps;
        case Mnemonic::Hlt:
            return asmjit::x86::Inst::kIdHlt;
        case Mnemonic::Hsubpd:
            return asmjit::x86::Inst::kIdHsubpd;
        case Mnemonic::Hsubps:
            return asmjit::x86::Inst::kIdHsubps;
        case Mnemonic::Idiv:
            return asmjit::x86::Inst::kIdIdiv;
        case Mnemonic::Imul:
            return asmjit::x86::Inst::kIdImul;
        case Mnemonic::In:
            return asmjit::x86::Inst::kIdIn;
        case Mnemonic::Inc:
            return asmjit::x86::Inst::kIdInc;
        case Mnemonic::Insertps:
            return asmjit::x86::Inst::kIdInsertps;
        case Mnemonic::Insertq:
            return asmjit::x86::Inst::kIdInsertq;
        case Mnemonic::Int:
            return asmjit::x86::Inst::kIdInt;
        case Mnemonic::Int3:
            return asmjit::x86::Inst::kIdInt3;
        case Mnemonic::Into:
            return asmjit::x86::Inst::kIdInto;
        case Mnemonic::Invd:
            return asmjit::x86::Inst::kIdInvd;
        case Mnemonic::Invlpg:
            return asmjit::x86::Inst::kIdInvlpg;
        case Mnemonic::Invpcid:
            return asmjit::x86::Inst::kIdInvpcid;
        case Mnemonic::Iret:
            return asmjit::x86::Inst::kIdIret;
        case Mnemonic::Iretd:
            return asmjit::x86::Inst::kIdIretd;
        case Mnemonic::Iretq:
            return asmjit::x86::Inst::kIdIretq;
        case Mnemonic::Jb:
            return asmjit::x86::Inst::kIdJb;
        case Mnemonic::Jbe:
            return asmjit::x86::Inst::kIdJbe;
        case Mnemonic::Jecxz:
            return asmjit::x86::Inst::kIdJecxz;
        case Mnemonic::Jl:
            return asmjit::x86::Inst::kIdJl;
        case Mnemonic::Jle:
            return asmjit::x86::Inst::kIdJle;
        case Mnemonic::Jmp:
            return asmjit::x86::Inst::kIdJmp;
        case Mnemonic::Jnb:
            return asmjit::x86::Inst::kIdJnb;
        case Mnemonic::Jnbe:
            return asmjit::x86::Inst::kIdJnbe;
        case Mnemonic::Jnl:
            return asmjit::x86::Inst::kIdJnl;
        case Mnemonic::Jnle:
            return asmjit::x86::Inst::kIdJnle;
        case Mnemonic::Jno:
            return asmjit::x86::Inst::kIdJno;
        case Mnemonic::Jnp:
            return asmjit::x86::Inst::kIdJnp;
        case Mnemonic::Jns:
            return asmjit::x86::Inst::kIdJns;
        case Mnemonic::Jnz:
            return asmjit::x86::Inst::kIdJnz;
        case Mnemonic::Jo:
            return asmjit::x86::Inst::kIdJo;
        case Mnemonic::Jp:
            return asmjit::x86::Inst::kIdJp;
        case Mnemonic::Js:
            return asmjit::x86::Inst::kIdJs;
        case Mnemonic::Jz:
            return asmjit::x86::Inst::kIdJz;
        case Mnemonic::Kaddb:
            return asmjit::x86::Inst::kIdKaddb;
        case Mnemonic::Kaddd:
            return asmjit::x86::Inst::kIdKaddd;
        case Mnemonic::Kaddq:
            return asmjit::x86::Inst::kIdKaddq;
        case Mnemonic::Kaddw:
            return asmjit::x86::Inst::kIdKaddw;
        case Mnemonic::Kandb:
            return asmjit::x86::Inst::kIdKandb;
        case Mnemonic::Kandd:
            return asmjit::x86::Inst::kIdKandd;
        case Mnemonic::Kandnb:
            return asmjit::x86::Inst::kIdKandnb;
        case Mnemonic::Kandnd:
            return asmjit::x86::Inst::kIdKandnd;
        case Mnemonic::Kandnq:
            return asmjit::x86::Inst::kIdKandnq;
        case Mnemonic::Kandnw:
            return asmjit::x86::Inst::kIdKandnw;
        case Mnemonic::Kandq:
            return asmjit::x86::Inst::kIdKandq;
        case Mnemonic::Kandw:
            return asmjit::x86::Inst::kIdKandw;
        case Mnemonic::Kmovb:
            return asmjit::x86::Inst::kIdKmovb;
        case Mnemonic::Kmovd:
            return asmjit::x86::Inst::kIdKmovd;
        case Mnemonic::Kmovq:
            return asmjit::x86::Inst::kIdKmovq;
        case Mnemonic::Kmovw:
            return asmjit::x86::Inst::kIdKmovw;
        case Mnemonic::Knotb:
            return asmjit::x86::Inst::kIdKnotb;
        case Mnemonic::Knotd:
            return asmjit::x86::Inst::kIdKnotd;
        case Mnemonic::Knotq:
            return asmjit::x86::Inst::kIdKnotq;
        case Mnemonic::Knotw:
            return asmjit::x86::Inst::kIdKnotw;
        case Mnemonic::Korb:
            return asmjit::x86::Inst::kIdKorb;
        case Mnemonic::Kord:
            return asmjit::x86::Inst::kIdKord;
        case Mnemonic::Korq:
            return asmjit::x86::Inst::kIdKorq;
        case Mnemonic::Kortestb:
            return asmjit::x86::Inst::kIdKortestb;
        case Mnemonic::Kortestd:
            return asmjit::x86::Inst::kIdKortestd;
        case Mnemonic::Kortestq:
            return asmjit::x86::Inst::kIdKortestq;
        case Mnemonic::Kortestw:
            return asmjit::x86::Inst::kIdKortestw;
        case Mnemonic::Korw:
            return asmjit::x86::Inst::kIdKorw;
        case Mnemonic::Kshiftlb:
            return asmjit::x86::Inst::kIdKshiftlb;
        case Mnemonic::Kshiftld:
            return asmjit::x86::Inst::kIdKshiftld;
        case Mnemonic::Kshiftlq:
            return asmjit::x86::Inst::kIdKshiftlq;
        case Mnemonic::Kshiftlw:
            return asmjit::x86::Inst::kIdKshiftlw;
        case Mnemonic::Kshiftrb:
            return asmjit::x86::Inst::kIdKshiftrb;
        case Mnemonic::Kshiftrd:
            return asmjit::x86::Inst::kIdKshiftrd;
        case Mnemonic::Kshiftrq:
            return asmjit::x86::Inst::kIdKshiftrq;
        case Mnemonic::Kshiftrw:
            return asmjit::x86::Inst::kIdKshiftrw;
        case Mnemonic::Ktestb:
            return asmjit::x86::Inst::kIdKtestb;
        case Mnemonic::Ktestd:
            return asmjit::x86::Inst::kIdKtestd;
        case Mnemonic::Ktestq:
            return asmjit::x86::Inst::kIdKtestq;
        case Mnemonic::Ktestw:
            return asmjit::x86::Inst::kIdKtestw;
        case Mnemonic::Kunpckbw:
            return asmjit::x86::Inst::kIdKunpckbw;
        case Mnemonic::Kunpckdq:
            return asmjit::x86::Inst::kIdKunpckdq;
        case Mnemonic::Kunpckwd:
            return asmjit::x86::Inst::kIdKunpckwd;
        case Mnemonic::Kxnorb:
            return asmjit::x86::Inst::kIdKxnorb;
        case Mnemonic::Kxnord:
            return asmjit::x86::Inst::kIdKxnord;
        case Mnemonic::Kxnorq:
            return asmjit::x86::Inst::kIdKxnorq;
        case Mnemonic::Kxnorw:
            return asmjit::x86::Inst::kIdKxnorw;
        case Mnemonic::Kxorb:
            return asmjit::x86::Inst::kIdKxorb;
        case Mnemonic::Kxord:
            return asmjit::x86::Inst::kIdKxord;
        case Mnemonic::Kxorq:
            return asmjit::x86::Inst::kIdKxorq;
        case Mnemonic::Kxorw:
            return asmjit::x86::Inst::kIdKxorw;
        case Mnemonic::Lahf:
            return asmjit::x86::Inst::kIdLahf;
        case Mnemonic::Lar:
            return asmjit::x86::Inst::kIdLar;
        case Mnemonic::Lddqu:
            return asmjit::x86::Inst::kIdLddqu;
        case Mnemonic::Ldmxcsr:
            return asmjit::x86::Inst::kIdLdmxcsr;
        case Mnemonic::Lds:
            return asmjit::x86::Inst::kIdLds;
        case Mnemonic::Lea:
            return asmjit::x86::Inst::kIdLea;
        case Mnemonic::Leave:
            return asmjit::x86::Inst::kIdLeave;
        case Mnemonic::Les:
            return asmjit::x86::Inst::kIdLes;
        case Mnemonic::Lfence:
            return asmjit::x86::Inst::kIdLfence;
        case Mnemonic::Lfs:
            return asmjit::x86::Inst::kIdLfs;
        case Mnemonic::Lgdt:
            return asmjit::x86::Inst::kIdLgdt;
        case Mnemonic::Lgs:
            return asmjit::x86::Inst::kIdLgs;
        case Mnemonic::Lidt:
            return asmjit::x86::Inst::kIdLidt;
        case Mnemonic::Lldt:
            return asmjit::x86::Inst::kIdLldt;
        case Mnemonic::Lmsw:
            return asmjit::x86::Inst::kIdLmsw;
        case Mnemonic::Loop:
            return asmjit::x86::Inst::kIdLoop;
        case Mnemonic::Loope:
            return asmjit::x86::Inst::kIdLoope;
        case Mnemonic::Loopne:
            return asmjit::x86::Inst::kIdLoopne;
        case Mnemonic::Lsl:
            return asmjit::x86::Inst::kIdLsl;
        case Mnemonic::Lss:
            return asmjit::x86::Inst::kIdLss;
        case Mnemonic::Ltr:
            return asmjit::x86::Inst::kIdLtr;
        case Mnemonic::Lzcnt:
            return asmjit::x86::Inst::kIdLzcnt;
        case Mnemonic::Maskmovdqu:
            return asmjit::x86::Inst::kIdMaskmovdqu;
        case Mnemonic::Maskmovq:
            return asmjit::x86::Inst::kIdMaskmovq;
        case Mnemonic::Maxpd:
            return asmjit::x86::Inst::kIdMaxpd;
        case Mnemonic::Maxps:
            return asmjit::x86::Inst::kIdMaxps;
        case Mnemonic::Maxsd:
            return asmjit::x86::Inst::kIdMaxsd;
        case Mnemonic::Maxss:
            return asmjit::x86::Inst::kIdMaxss;
        case Mnemonic::Mfence:
            return asmjit::x86::Inst::kIdMfence;
        case Mnemonic::Minpd:
            return asmjit::x86::Inst::kIdMinpd;
        case Mnemonic::Minps:
            return asmjit::x86::Inst::kIdMinps;
        case Mnemonic::Minsd:
            return asmjit::x86::Inst::kIdMinsd;
        case Mnemonic::Minss:
            return asmjit::x86::Inst::kIdMinss;
        case Mnemonic::Monitor:
            return asmjit::x86::Inst::kIdMonitor;
        case Mnemonic::Monitorx:
            return asmjit::x86::Inst::kIdMonitorx;
        case Mnemonic::Mov:
            return asmjit::x86::Inst::kIdMov;
        case Mnemonic::Movapd:
            return asmjit::x86::Inst::kIdMovapd;
        case Mnemonic::Movaps:
            return asmjit::x86::Inst::kIdMovaps;
        case Mnemonic::Movbe:
            return asmjit::x86::Inst::kIdMovbe;
        case Mnemonic::Movd:
            return asmjit::x86::Inst::kIdMovd;
        case Mnemonic::Movddup:
            return asmjit::x86::Inst::kIdMovddup;
        case Mnemonic::Movdq2q:
            return asmjit::x86::Inst::kIdMovdq2q;
        case Mnemonic::Movdqa:
            return asmjit::x86::Inst::kIdMovdqa;
        case Mnemonic::Movdqu:
            return asmjit::x86::Inst::kIdMovdqu;
        case Mnemonic::Movhlps:
            return asmjit::x86::Inst::kIdMovhlps;
        case Mnemonic::Movhpd:
            return asmjit::x86::Inst::kIdMovhpd;
        case Mnemonic::Movhps:
            return asmjit::x86::Inst::kIdMovhps;
        case Mnemonic::Movlhps:
            return asmjit::x86::Inst::kIdMovlhps;
        case Mnemonic::Movlpd:
            return asmjit::x86::Inst::kIdMovlpd;
        case Mnemonic::Movlps:
            return asmjit::x86::Inst::kIdMovlps;
        case Mnemonic::Movmskpd:
            return asmjit::x86::Inst::kIdMovmskpd;
        case Mnemonic::Movmskps:
            return asmjit::x86::Inst::kIdMovmskps;
        case Mnemonic::Movntdq:
            return asmjit::x86::Inst::kIdMovntdq;
        case Mnemonic::Movntdqa:
            return asmjit::x86::Inst::kIdMovntdqa;
        case Mnemonic::Movnti:
            return asmjit::x86::Inst::kIdMovnti;
        case Mnemonic::Movntpd:
            return asmjit::x86::Inst::kIdMovntpd;
        case Mnemonic::Movntps:
            return asmjit::x86::Inst::kIdMovntps;
        case Mnemonic::Movntq:
            return asmjit::x86::Inst::kIdMovntq;
        case Mnemonic::Movntsd:
            return asmjit::x86::Inst::kIdMovntsd;
        case Mnemonic::Movntss:
            return asmjit::x86::Inst::kIdMovntss;
        case Mnemonic::Movq:
            return asmjit::x86::Inst::kIdMovq;
        case Mnemonic::Movq2dq:
            return asmjit::x86::Inst::kIdMovq2dq;
        case Mnemonic::Movshdup:
            return asmjit::x86::Inst::kIdMovshdup;
        case Mnemonic::Movsldup:
            return asmjit::x86::Inst::kIdMovsldup;
        case Mnemonic::Movss:
            return asmjit::x86::Inst::kIdMovss;
        case Mnemonic::Movsx:
            return asmjit::x86::Inst::kIdMovsx;
        case Mnemonic::Movsxd:
            return asmjit::x86::Inst::kIdMovsxd;
        case Mnemonic::Movupd:
            return asmjit::x86::Inst::kIdMovupd;
        case Mnemonic::Movups:
            return asmjit::x86::Inst::kIdMovups;
        case Mnemonic::Movzx:
            return asmjit::x86::Inst::kIdMovzx;
        case Mnemonic::Mpsadbw:
            return asmjit::x86::Inst::kIdMpsadbw;
        case Mnemonic::Mul:
            return asmjit::x86::Inst::kIdMul;
        case Mnemonic::Mulpd:
            return asmjit::x86::Inst::kIdMulpd;
        case Mnemonic::Mulps:
            return asmjit::x86::Inst::kIdMulps;
        case Mnemonic::Mulsd:
            return asmjit::x86::Inst::kIdMulsd;
        case Mnemonic::Mulss:
            return asmjit::x86::Inst::kIdMulss;
        case Mnemonic::Mulx:
            return asmjit::x86::Inst::kIdMulx;
        case Mnemonic::Mwait:
            return asmjit::x86::Inst::kIdMwait;
        case Mnemonic::Mwaitx:
            return asmjit::x86::Inst::kIdMwaitx;
        case Mnemonic::Neg:
            return asmjit::x86::Inst::kIdNeg;
        case Mnemonic::Nop:
            return asmjit::x86::Inst::kIdNop;
        case Mnemonic::Not:
            return asmjit::x86::Inst::kIdNot;
        case Mnemonic::Or:
            return asmjit::x86::Inst::kIdOr;
        case Mnemonic::Orpd:
            return asmjit::x86::Inst::kIdOrpd;
        case Mnemonic::Orps:
            return asmjit::x86::Inst::kIdOrps;
        case Mnemonic::Out:
            return asmjit::x86::Inst::kIdOut;
        case Mnemonic::Pabsb:
            return asmjit::x86::Inst::kIdPabsb;
        case Mnemonic::Pabsd:
            return asmjit::x86::Inst::kIdPabsd;
        case Mnemonic::Pabsw:
            return asmjit::x86::Inst::kIdPabsw;
        case Mnemonic::Packssdw:
            return asmjit::x86::Inst::kIdPackssdw;
        case Mnemonic::Packsswb:
            return asmjit::x86::Inst::kIdPacksswb;
        case Mnemonic::Packusdw:
            return asmjit::x86::Inst::kIdPackusdw;
        case Mnemonic::Packuswb:
            return asmjit::x86::Inst::kIdPackuswb;
        case Mnemonic::Paddb:
            return asmjit::x86::Inst::kIdPaddb;
        case Mnemonic::Paddd:
            return asmjit::x86::Inst::kIdPaddd;
        case Mnemonic::Paddq:
            return asmjit::x86::Inst::kIdPaddq;
        case Mnemonic::Paddsb:
            return asmjit::x86::Inst::kIdPaddsb;
        case Mnemonic::Paddsw:
            return asmjit::x86::Inst::kIdPaddsw;
        case Mnemonic::Paddusb:
            return asmjit::x86::Inst::kIdPaddusb;
        case Mnemonic::Paddusw:
            return asmjit::x86::Inst::kIdPaddusw;
        case Mnemonic::Paddw:
            return asmjit::x86::Inst::kIdPaddw;
        case Mnemonic::Palignr:
            return asmjit::x86::Inst::kIdPalignr;
        case Mnemonic::Pand:
            return asmjit::x86::Inst::kIdPand;
        case Mnemonic::Pandn:
            return asmjit::x86::Inst::kIdPandn;
        case Mnemonic::Pause:
            return asmjit::x86::Inst::kIdPause;
        case Mnemonic::Pavgb:
            return asmjit::x86::Inst::kIdPavgb;
        case Mnemonic::Pavgusb:
            return asmjit::x86::Inst::kIdPavgusb;
        case Mnemonic::Pavgw:
            return asmjit::x86::Inst::kIdPavgw;
        case Mnemonic::Pblendvb:
            return asmjit::x86::Inst::kIdPblendvb;
        case Mnemonic::Pblendw:
            return asmjit::x86::Inst::kIdPblendw;
        case Mnemonic::Pclmulqdq:
            return asmjit::x86::Inst::kIdPclmulqdq;
        case Mnemonic::Pcmpeqb:
            return asmjit::x86::Inst::kIdPcmpeqb;
        case Mnemonic::Pcmpeqd:
            return asmjit::x86::Inst::kIdPcmpeqd;
        case Mnemonic::Pcmpeqq:
            return asmjit::x86::Inst::kIdPcmpeqq;
        case Mnemonic::Pcmpeqw:
            return asmjit::x86::Inst::kIdPcmpeqw;
        case Mnemonic::Pcmpestri:
            return asmjit::x86::Inst::kIdPcmpestri;
        case Mnemonic::Pcmpestrm:
            return asmjit::x86::Inst::kIdPcmpestrm;
        case Mnemonic::Pcmpgtb:
            return asmjit::x86::Inst::kIdPcmpgtb;
        case Mnemonic::Pcmpgtd:
            return asmjit::x86::Inst::kIdPcmpgtd;
        case Mnemonic::Pcmpgtq:
            return asmjit::x86::Inst::kIdPcmpgtq;
        case Mnemonic::Pcmpgtw:
            return asmjit::x86::Inst::kIdPcmpgtw;
        case Mnemonic::Pcmpistri:
            return asmjit::x86::Inst::kIdPcmpistri;
        case Mnemonic::Pcmpistrm:
            return asmjit::x86::Inst::kIdPcmpistrm;
        case Mnemonic::Pdep:
            return asmjit::x86::Inst::kIdPdep;
        case Mnemonic::Pext:
            return asmjit::x86::Inst::kIdPext;
        case Mnemonic::Pextrb:
            return asmjit::x86::Inst::kIdPextrb;
        case Mnemonic::Pextrd:
            return asmjit::x86::Inst::kIdPextrd;
        case Mnemonic::Pextrq:
            return asmjit::x86::Inst::kIdPextrq;
        case Mnemonic::Pextrw:
            return asmjit::x86::Inst::kIdPextrw;
        case Mnemonic::Pf2id:
            return asmjit::x86::Inst::kIdPf2id;
        case Mnemonic::Pf2iw:
            return asmjit::x86::Inst::kIdPf2iw;
        case Mnemonic::Pfacc:
            return asmjit::x86::Inst::kIdPfacc;
        case Mnemonic::Pfadd:
            return asmjit::x86::Inst::kIdPfadd;
        case Mnemonic::Pfcmpeq:
            return asmjit::x86::Inst::kIdPfcmpeq;
        case Mnemonic::Pfcmpge:
            return asmjit::x86::Inst::kIdPfcmpge;
        case Mnemonic::Pfcmpgt:
            return asmjit::x86::Inst::kIdPfcmpgt;
        case Mnemonic::Pfmax:
            return asmjit::x86::Inst::kIdPfmax;
        case Mnemonic::Pfmin:
            return asmjit::x86::Inst::kIdPfmin;
        case Mnemonic::Pfmul:
            return asmjit::x86::Inst::kIdPfmul;
        case Mnemonic::Pfnacc:
            return asmjit::x86::Inst::kIdPfnacc;
        case Mnemonic::Pfpnacc:
            return asmjit::x86::Inst::kIdPfpnacc;
        case Mnemonic::Pfrcp:
            return asmjit::x86::Inst::kIdPfrcp;
        case Mnemonic::Pfrcpit2:
            return asmjit::x86::Inst::kIdPfrcpit2;
        case Mnemonic::Pfrsqit1:
            return asmjit::x86::Inst::kIdPfrsqit1;
        case Mnemonic::Pfsub:
            return asmjit::x86::Inst::kIdPfsub;
        case Mnemonic::Pfsubr:
            return asmjit::x86::Inst::kIdPfsubr;
        case Mnemonic::Phaddd:
            return asmjit::x86::Inst::kIdPhaddd;
        case Mnemonic::Phaddsw:
            return asmjit::x86::Inst::kIdPhaddsw;
        case Mnemonic::Phaddw:
            return asmjit::x86::Inst::kIdPhaddw;
        case Mnemonic::Phminposuw:
            return asmjit::x86::Inst::kIdPhminposuw;
        case Mnemonic::Phsubd:
            return asmjit::x86::Inst::kIdPhsubd;
        case Mnemonic::Phsubsw:
            return asmjit::x86::Inst::kIdPhsubsw;
        case Mnemonic::Phsubw:
            return asmjit::x86::Inst::kIdPhsubw;
        case Mnemonic::Pi2fd:
            return asmjit::x86::Inst::kIdPi2fd;
        case Mnemonic::Pi2fw:
            return asmjit::x86::Inst::kIdPi2fw;
        case Mnemonic::Pinsrb:
            return asmjit::x86::Inst::kIdPinsrb;
        case Mnemonic::Pinsrd:
            return asmjit::x86::Inst::kIdPinsrd;
        case Mnemonic::Pinsrq:
            return asmjit::x86::Inst::kIdPinsrq;
        case Mnemonic::Pinsrw:
            return asmjit::x86::Inst::kIdPinsrw;
        case Mnemonic::Pmaddubsw:
            return asmjit::x86::Inst::kIdPmaddubsw;
        case Mnemonic::Pmaddwd:
            return asmjit::x86::Inst::kIdPmaddwd;
        case Mnemonic::Pmaxsb:
            return asmjit::x86::Inst::kIdPmaxsb;
        case Mnemonic::Pmaxsd:
            return asmjit::x86::Inst::kIdPmaxsd;
        case Mnemonic::Pmaxsw:
            return asmjit::x86::Inst::kIdPmaxsw;
        case Mnemonic::Pmaxub:
            return asmjit::x86::Inst::kIdPmaxub;
        case Mnemonic::Pmaxud:
            return asmjit::x86::Inst::kIdPmaxud;
        case Mnemonic::Pmaxuw:
            return asmjit::x86::Inst::kIdPmaxuw;
        case Mnemonic::Pminsb:
            return asmjit::x86::Inst::kIdPminsb;
        case Mnemonic::Pminsd:
            return asmjit::x86::Inst::kIdPminsd;
        case Mnemonic::Pminsw:
            return asmjit::x86::Inst::kIdPminsw;
        case Mnemonic::Pminub:
            return asmjit::x86::Inst::kIdPminub;
        case Mnemonic::Pminud:
            return asmjit::x86::Inst::kIdPminud;
        case Mnemonic::Pminuw:
            return asmjit::x86::Inst::kIdPminuw;
        case Mnemonic::Pmovmskb:
            return asmjit::x86::Inst::kIdPmovmskb;
        case Mnemonic::Pmovsxbd:
            return asmjit::x86::Inst::kIdPmovsxbd;
        case Mnemonic::Pmovsxbq:
            return asmjit::x86::Inst::kIdPmovsxbq;
        case Mnemonic::Pmovsxbw:
            return asmjit::x86::Inst::kIdPmovsxbw;
        case Mnemonic::Pmovsxdq:
            return asmjit::x86::Inst::kIdPmovsxdq;
        case Mnemonic::Pmovsxwd:
            return asmjit::x86::Inst::kIdPmovsxwd;
        case Mnemonic::Pmovsxwq:
            return asmjit::x86::Inst::kIdPmovsxwq;
        case Mnemonic::Pmovzxbd:
            return asmjit::x86::Inst::kIdPmovzxbd;
        case Mnemonic::Pmovzxbq:
            return asmjit::x86::Inst::kIdPmovzxbq;
        case Mnemonic::Pmovzxbw:
            return asmjit::x86::Inst::kIdPmovzxbw;
        case Mnemonic::Pmovzxdq:
            return asmjit::x86::Inst::kIdPmovzxdq;
        case Mnemonic::Pmovzxwd:
            return asmjit::x86::Inst::kIdPmovzxwd;
        case Mnemonic::Pmovzxwq:
            return asmjit::x86::Inst::kIdPmovzxwq;
        case Mnemonic::Pmuldq:
            return asmjit::x86::Inst::kIdPmuldq;
        case Mnemonic::Pmulhrsw:
            return asmjit::x86::Inst::kIdPmulhrsw;
        case Mnemonic::Pmulhrw:
            return asmjit::x86::Inst::kIdPmulhrw;
        case Mnemonic::Pmulhuw:
            return asmjit::x86::Inst::kIdPmulhuw;
        case Mnemonic::Pmulhw:
            return asmjit::x86::Inst::kIdPmulhw;
        case Mnemonic::Pmulld:
            return asmjit::x86::Inst::kIdPmulld;
        case Mnemonic::Pmullw:
            return asmjit::x86::Inst::kIdPmullw;
        case Mnemonic::Pmuludq:
            return asmjit::x86::Inst::kIdPmuludq;
        case Mnemonic::Pop:
            return asmjit::x86::Inst::kIdPop;
        case Mnemonic::Popa:
            return asmjit::x86::Inst::kIdPopa;
        case Mnemonic::Popad:
            return asmjit::x86::Inst::kIdPopad;
        case Mnemonic::Popcnt:
            return asmjit::x86::Inst::kIdPopcnt;
        case Mnemonic::Popf:
            return asmjit::x86::Inst::kIdPopf;
        case Mnemonic::Popfd:
            return asmjit::x86::Inst::kIdPopfd;
        case Mnemonic::Popfq:
            return asmjit::x86::Inst::kIdPopfq;
        case Mnemonic::Por:
            return asmjit::x86::Inst::kIdPor;
        case Mnemonic::Prefetch:
            return asmjit::x86::Inst::kIdPrefetch;
        case Mnemonic::Prefetchnta:
            return asmjit::x86::Inst::kIdPrefetchnta;
        case Mnemonic::Prefetcht0:
            return asmjit::x86::Inst::kIdPrefetcht0;
        case Mnemonic::Prefetcht1:
            return asmjit::x86::Inst::kIdPrefetcht1;
        case Mnemonic::Prefetcht2:
            return asmjit::x86::Inst::kIdPrefetcht2;
        case Mnemonic::Prefetchw:
            return asmjit::x86::Inst::kIdPrefetchw;
        case Mnemonic::Prefetchwt1:
            return asmjit::x86::Inst::kIdPrefetchwt1;
        case Mnemonic::Psadbw:
            return asmjit::x86::Inst::kIdPsadbw;
        case Mnemonic::Pshufb:
            return asmjit::x86::Inst::kIdPshufb;
        case Mnemonic::Pshufd:
            return asmjit::x86::Inst::kIdPshufd;
        case Mnemonic::Pshufhw:
            return asmjit::x86::Inst::kIdPshufhw;
        case Mnemonic::Pshuflw:
            return asmjit::x86::Inst::kIdPshuflw;
        case Mnemonic::Pshufw:
            return asmjit::x86::Inst::kIdPshufw;
        case Mnemonic::Psignb:
            return asmjit::x86::Inst::kIdPsignb;
        case Mnemonic::Psignd:
            return asmjit::x86::Inst::kIdPsignd;
        case Mnemonic::Psignw:
            return asmjit::x86::Inst::kIdPsignw;
        case Mnemonic::Pslld:
            return asmjit::x86::Inst::kIdPslld;
        case Mnemonic::Pslldq:
            return asmjit::x86::Inst::kIdPslldq;
        case Mnemonic::Psllq:
            return asmjit::x86::Inst::kIdPsllq;
        case Mnemonic::Psllw:
            return asmjit::x86::Inst::kIdPsllw;
        case Mnemonic::Psrad:
            return asmjit::x86::Inst::kIdPsrad;
        case Mnemonic::Psraw:
            return asmjit::x86::Inst::kIdPsraw;
        case Mnemonic::Psrld:
            return asmjit::x86::Inst::kIdPsrld;
        case Mnemonic::Psrldq:
            return asmjit::x86::Inst::kIdPsrldq;
        case Mnemonic::Psrlq:
            return asmjit::x86::Inst::kIdPsrlq;
        case Mnemonic::Psrlw:
            return asmjit::x86::Inst::kIdPsrlw;
        case Mnemonic::Psubb:
            return asmjit::x86::Inst::kIdPsubb;
        case Mnemonic::Psubd:
            return asmjit::x86::Inst::kIdPsubd;
        case Mnemonic::Psubq:
            return asmjit::x86::Inst::kIdPsubq;
        case Mnemonic::Psubsb:
            return asmjit::x86::Inst::kIdPsubsb;
        case Mnemonic::Psubsw:
            return asmjit::x86::Inst::kIdPsubsw;
        case Mnemonic::Psubusb:
            return asmjit::x86::Inst::kIdPsubusb;
        case Mnemonic::Psubusw:
            return asmjit::x86::Inst::kIdPsubusw;
        case Mnemonic::Psubw:
            return asmjit::x86::Inst::kIdPsubw;
        case Mnemonic::Pswapd:
            return asmjit::x86::Inst::kIdPswapd;
        case Mnemonic::Ptest:
            return asmjit::x86::Inst::kIdPtest;
        case Mnemonic::Punpckhbw:
            return asmjit::x86::Inst::kIdPunpckhbw;
        case Mnemonic::Punpckhdq:
            return asmjit::x86::Inst::kIdPunpckhdq;
        case Mnemonic::Punpckhqdq:
            return asmjit::x86::Inst::kIdPunpckhqdq;
        case Mnemonic::Punpckhwd:
            return asmjit::x86::Inst::kIdPunpckhwd;
        case Mnemonic::Punpcklbw:
            return asmjit::x86::Inst::kIdPunpcklbw;
        case Mnemonic::Punpckldq:
            return asmjit::x86::Inst::kIdPunpckldq;
        case Mnemonic::Punpcklqdq:
            return asmjit::x86::Inst::kIdPunpcklqdq;
        case Mnemonic::Punpcklwd:
            return asmjit::x86::Inst::kIdPunpcklwd;
        case Mnemonic::Push:
            return asmjit::x86::Inst::kIdPush;
        case Mnemonic::Pusha:
            return asmjit::x86::Inst::kIdPusha;
        case Mnemonic::Pushad:
            return asmjit::x86::Inst::kIdPushad;
        case Mnemonic::Pushf:
            return asmjit::x86::Inst::kIdPushf;
        case Mnemonic::Pushfd:
            return asmjit::x86::Inst::kIdPushfd;
        case Mnemonic::Pushfq:
            return asmjit::x86::Inst::kIdPushfq;
        case Mnemonic::Pxor:
            return asmjit::x86::Inst::kIdPxor;
        case Mnemonic::Rcl:
            return asmjit::x86::Inst::kIdRcl;
        case Mnemonic::Rcpps:
            return asmjit::x86::Inst::kIdRcpps;
        case Mnemonic::Rcpss:
            return asmjit::x86::Inst::kIdRcpss;
        case Mnemonic::Rcr:
            return asmjit::x86::Inst::kIdRcr;
        case Mnemonic::Rdfsbase:
            return asmjit::x86::Inst::kIdRdfsbase;
        case Mnemonic::Rdgsbase:
            return asmjit::x86::Inst::kIdRdgsbase;
        case Mnemonic::Rdmsr:
            return asmjit::x86::Inst::kIdRdmsr;
        case Mnemonic::Rdpmc:
            return asmjit::x86::Inst::kIdRdpmc;
        case Mnemonic::Rdrand:
            return asmjit::x86::Inst::kIdRdrand;
        case Mnemonic::Rdseed:
            return asmjit::x86::Inst::kIdRdseed;
        case Mnemonic::Rdtsc:
            return asmjit::x86::Inst::kIdRdtsc;
        case Mnemonic::Rdtscp:
            return asmjit::x86::Inst::kIdRdtscp;
        case Mnemonic::Ret:
            return asmjit::x86::Inst::kIdRet;
        case Mnemonic::Rol:
            return asmjit::x86::Inst::kIdRol;
        case Mnemonic::Ror:
            return asmjit::x86::Inst::kIdRor;
        case Mnemonic::Rorx:
            return asmjit::x86::Inst::kIdRorx;
        case Mnemonic::Roundpd:
            return asmjit::x86::Inst::kIdRoundpd;
        case Mnemonic::Roundps:
            return asmjit::x86::Inst::kIdRoundps;
        case Mnemonic::Roundsd:
            return asmjit::x86::Inst::kIdRoundsd;
        case Mnemonic::Roundss:
            return asmjit::x86::Inst::kIdRoundss;
        case Mnemonic::Rsm:
            return asmjit::x86::Inst::kIdRsm;
        case Mnemonic::Rsqrtps:
            return asmjit::x86::Inst::kIdRsqrtps;
        case Mnemonic::Rsqrtss:
            return asmjit::x86::Inst::kIdRsqrtss;
        case Mnemonic::Sahf:
            return asmjit::x86::Inst::kIdSahf;
        case Mnemonic::Sar:
            return asmjit::x86::Inst::kIdSar;
        case Mnemonic::Sarx:
            return asmjit::x86::Inst::kIdSarx;
        case Mnemonic::Sbb:
            return asmjit::x86::Inst::kIdSbb;
        case Mnemonic::Setb:
            return asmjit::x86::Inst::kIdSetb;
        case Mnemonic::Setbe:
            return asmjit::x86::Inst::kIdSetbe;
        case Mnemonic::Setl:
            return asmjit::x86::Inst::kIdSetl;
        case Mnemonic::Setle:
            return asmjit::x86::Inst::kIdSetle;
        case Mnemonic::Setnb:
            return asmjit::x86::Inst::kIdSetnb;
        case Mnemonic::Setnbe:
            return asmjit::x86::Inst::kIdSetnbe;
        case Mnemonic::Setnl:
            return asmjit::x86::Inst::kIdSetnl;
        case Mnemonic::Setnle:
            return asmjit::x86::Inst::kIdSetnle;
        case Mnemonic::Setno:
            return asmjit::x86::Inst::kIdSetno;
        case Mnemonic::Setnp:
            return asmjit::x86::Inst::kIdSetnp;
        case Mnemonic::Setns:
            return asmjit::x86::Inst::kIdSetns;
        case Mnemonic::Setnz:
            return asmjit::x86::Inst::kIdSetnz;
        case Mnemonic::Seto:
            return asmjit::x86::Inst::kIdSeto;
        case Mnemonic::Setp:
            return asmjit::x86::Inst::kIdSetp;
        case Mnemonic::Sets:
            return asmjit::x86::Inst::kIdSets;
        case Mnemonic::Setz:
            return asmjit::x86::Inst::kIdSetz;
        case Mnemonic::Sfence:
            return asmjit::x86::Inst::kIdSfence;
        case Mnemonic::Sgdt:
            return asmjit::x86::Inst::kIdSgdt;
        case Mnemonic::Sha1msg1:
            return asmjit::x86::Inst::kIdSha1msg1;
        case Mnemonic::Sha1msg2:
            return asmjit::x86::Inst::kIdSha1msg2;
        case Mnemonic::Sha1nexte:
            return asmjit::x86::Inst::kIdSha1nexte;
        case Mnemonic::Sha1rnds4:
            return asmjit::x86::Inst::kIdSha1rnds4;
        case Mnemonic::Sha256msg1:
            return asmjit::x86::Inst::kIdSha256msg1;
        case Mnemonic::Sha256msg2:
            return asmjit::x86::Inst::kIdSha256msg2;
        case Mnemonic::Sha256rnds2:
            return asmjit::x86::Inst::kIdSha256rnds2;
        case Mnemonic::Shl:
            return asmjit::x86::Inst::kIdShl;
        case Mnemonic::Shld:
            return asmjit::x86::Inst::kIdShld;
        case Mnemonic::Shlx:
            return asmjit::x86::Inst::kIdShlx;
        case Mnemonic::Shr:
            return asmjit::x86::Inst::kIdShr;
        case Mnemonic::Shrd:
            return asmjit::x86::Inst::kIdShrd;
        case Mnemonic::Shrx:
            return asmjit::x86::Inst::kIdShrx;
        case Mnemonic::Shufpd:
            return asmjit::x86::Inst::kIdShufpd;
        case Mnemonic::Shufps:
            return asmjit::x86::Inst::kIdShufps;
        case Mnemonic::Sidt:
            return asmjit::x86::Inst::kIdSidt;
        case Mnemonic::Sldt:
            return asmjit::x86::Inst::kIdSldt;
        case Mnemonic::Smsw:
            return asmjit::x86::Inst::kIdSmsw;
        case Mnemonic::Sqrtpd:
            return asmjit::x86::Inst::kIdSqrtpd;
        case Mnemonic::Sqrtps:
            return asmjit::x86::Inst::kIdSqrtps;
        case Mnemonic::Sqrtsd:
            return asmjit::x86::Inst::kIdSqrtsd;
        case Mnemonic::Sqrtss:
            return asmjit::x86::Inst::kIdSqrtss;
        case Mnemonic::Stac:
            return asmjit::x86::Inst::kIdStac;
        case Mnemonic::Stc:
            return asmjit::x86::Inst::kIdStc;
        case Mnemonic::Std:
            return asmjit::x86::Inst::kIdStd;
        case Mnemonic::Sti:
            return asmjit::x86::Inst::kIdSti;
        case Mnemonic::Stmxcsr:
            return asmjit::x86::Inst::kIdStmxcsr;
        case Mnemonic::Str:
            return asmjit::x86::Inst::kIdStr;
        case Mnemonic::Sub:
            return asmjit::x86::Inst::kIdSub;
        case Mnemonic::Subpd:
            return asmjit::x86::Inst::kIdSubpd;
        case Mnemonic::Subps:
            return asmjit::x86::Inst::kIdSubps;
        case Mnemonic::Subsd:
            return asmjit::x86::Inst::kIdSubsd;
        case Mnemonic::Subss:
            return asmjit::x86::Inst::kIdSubss;
        case Mnemonic::Swapgs:
            return asmjit::x86::Inst::kIdSwapgs;
        case Mnemonic::Syscall:
            return asmjit::x86::Inst::kIdSyscall;
        case Mnemonic::Sysenter:
            return asmjit::x86::Inst::kIdSysenter;
        case Mnemonic::Sysexit:
            return asmjit::x86::Inst::kIdSysexit;
        case Mnemonic::Sysret:
            return asmjit::x86::Inst::kIdSysret;
        case Mnemonic::T1mskc:
            return asmjit::x86::Inst::kIdT1mskc;
        case Mnemonic::Test:
            return asmjit::x86::Inst::kIdTest;
        case Mnemonic::Tzcnt:
            return asmjit::x86::Inst::kIdTzcnt;
        case Mnemonic::Tzmsk:
            return asmjit::x86::Inst::kIdTzmsk;
        case Mnemonic::Ucomisd:
            return asmjit::x86::Inst::kIdUcomisd;
        case Mnemonic::Ucomiss:
            return asmjit::x86::Inst::kIdUcomiss;
        case Mnemonic::Ud2:
            return asmjit::x86::Inst::kIdUd2;
        case Mnemonic::Unpckhpd:
            return asmjit::x86::Inst::kIdUnpckhpd;
        case Mnemonic::Unpckhps:
            return asmjit::x86::Inst::kIdUnpckhps;
        case Mnemonic::Unpcklpd:
            return asmjit::x86::Inst::kIdUnpcklpd;
        case Mnemonic::Unpcklps:
            return asmjit::x86::Inst::kIdUnpcklps;
        case Mnemonic::V4fmaddps:
            return asmjit::x86::Inst::kIdV4fmaddps;
        case Mnemonic::V4fmaddss:
            return asmjit::x86::Inst::kIdV4fmaddss;
        case Mnemonic::V4fnmaddps:
            return asmjit::x86::Inst::kIdV4fnmaddps;
        case Mnemonic::V4fnmaddss:
            return asmjit::x86::Inst::kIdV4fnmaddss;
        case Mnemonic::Vaddpd:
            return asmjit::x86::Inst::kIdVaddpd;
        case Mnemonic::Vaddps:
            return asmjit::x86::Inst::kIdVaddps;
        case Mnemonic::Vaddsd:
            return asmjit::x86::Inst::kIdVaddsd;
        case Mnemonic::Vaddss:
            return asmjit::x86::Inst::kIdVaddss;
        case Mnemonic::Vaddsubpd:
            return asmjit::x86::Inst::kIdVaddsubpd;
        case Mnemonic::Vaddsubps:
            return asmjit::x86::Inst::kIdVaddsubps;
        case Mnemonic::Vaesdec:
            return asmjit::x86::Inst::kIdVaesdec;
        case Mnemonic::Vaesdeclast:
            return asmjit::x86::Inst::kIdVaesdeclast;
        case Mnemonic::Vaesenc:
            return asmjit::x86::Inst::kIdVaesenc;
        case Mnemonic::Vaesenclast:
            return asmjit::x86::Inst::kIdVaesenclast;
        case Mnemonic::Vaesimc:
            return asmjit::x86::Inst::kIdVaesimc;
        case Mnemonic::Vaeskeygenassist:
            return asmjit::x86::Inst::kIdVaeskeygenassist;
        case Mnemonic::Valignd:
            return asmjit::x86::Inst::kIdValignd;
        case Mnemonic::Valignq:
            return asmjit::x86::Inst::kIdValignq;
        case Mnemonic::Vandnpd:
            return asmjit::x86::Inst::kIdVandnpd;
        case Mnemonic::Vandnps:
            return asmjit::x86::Inst::kIdVandnps;
        case Mnemonic::Vandpd:
            return asmjit::x86::Inst::kIdVandpd;
        case Mnemonic::Vandps:
            return asmjit::x86::Inst::kIdVandps;
        case Mnemonic::Vblendmpd:
            return asmjit::x86::Inst::kIdVblendmpd;
        case Mnemonic::Vblendmps:
            return asmjit::x86::Inst::kIdVblendmps;
        case Mnemonic::Vblendpd:
            return asmjit::x86::Inst::kIdVblendpd;
        case Mnemonic::Vblendps:
            return asmjit::x86::Inst::kIdVblendps;
        case Mnemonic::Vblendvpd:
            return asmjit::x86::Inst::kIdVblendvpd;
        case Mnemonic::Vblendvps:
            return asmjit::x86::Inst::kIdVblendvps;
        case Mnemonic::Vbroadcastf128:
            return asmjit::x86::Inst::kIdVbroadcastf128;
        case Mnemonic::Vbroadcastf32x2:
            return asmjit::x86::Inst::kIdVbroadcastf32x2;
        case Mnemonic::Vbroadcastf32x4:
            return asmjit::x86::Inst::kIdVbroadcastf32x4;
        case Mnemonic::Vbroadcastf32x8:
            return asmjit::x86::Inst::kIdVbroadcastf32x8;
        case Mnemonic::Vbroadcastf64x2:
            return asmjit::x86::Inst::kIdVbroadcastf64x2;
        case Mnemonic::Vbroadcastf64x4:
            return asmjit::x86::Inst::kIdVbroadcastf64x4;
        case Mnemonic::Vbroadcasti128:
            return asmjit::x86::Inst::kIdVbroadcasti128;
        case Mnemonic::Vbroadcasti32x2:
            return asmjit::x86::Inst::kIdVbroadcasti32x2;
        case Mnemonic::Vbroadcasti32x4:
            return asmjit::x86::Inst::kIdVbroadcasti32x4;
        case Mnemonic::Vbroadcasti32x8:
            return asmjit::x86::Inst::kIdVbroadcasti32x8;
        case Mnemonic::Vbroadcasti64x2:
            return asmjit::x86::Inst::kIdVbroadcasti64x2;
        case Mnemonic::Vbroadcasti64x4:
            return asmjit::x86::Inst::kIdVbroadcasti64x4;
        case Mnemonic::Vbroadcastsd:
            return asmjit::x86::Inst::kIdVbroadcastsd;
        case Mnemonic::Vbroadcastss:
            return asmjit::x86::Inst::kIdVbroadcastss;
        case Mnemonic::Vcmppd:
            return asmjit::x86::Inst::kIdVcmppd;
        case Mnemonic::Vcmpps:
            return asmjit::x86::Inst::kIdVcmpps;
        case Mnemonic::Vcmpsd:
            return asmjit::x86::Inst::kIdVcmpsd;
        case Mnemonic::Vcmpss:
            return asmjit::x86::Inst::kIdVcmpss;
        case Mnemonic::Vcomisd:
            return asmjit::x86::Inst::kIdVcomisd;
        case Mnemonic::Vcomiss:
            return asmjit::x86::Inst::kIdVcomiss;
        case Mnemonic::Vcompresspd:
            return asmjit::x86::Inst::kIdVcompresspd;
        case Mnemonic::Vcompressps:
            return asmjit::x86::Inst::kIdVcompressps;
        case Mnemonic::Vcvtdq2pd:
            return asmjit::x86::Inst::kIdVcvtdq2pd;
        case Mnemonic::Vcvtdq2ps:
            return asmjit::x86::Inst::kIdVcvtdq2ps;
        case Mnemonic::Vcvtpd2dq:
            return asmjit::x86::Inst::kIdVcvtpd2dq;
        case Mnemonic::Vcvtpd2ps:
            return asmjit::x86::Inst::kIdVcvtpd2ps;
        case Mnemonic::Vcvtpd2qq:
            return asmjit::x86::Inst::kIdVcvtpd2qq;
        case Mnemonic::Vcvtpd2udq:
            return asmjit::x86::Inst::kIdVcvtpd2udq;
        case Mnemonic::Vcvtpd2uqq:
            return asmjit::x86::Inst::kIdVcvtpd2uqq;
        case Mnemonic::Vcvtph2ps:
            return asmjit::x86::Inst::kIdVcvtph2ps;
        case Mnemonic::Vcvtps2dq:
            return asmjit::x86::Inst::kIdVcvtps2dq;
        case Mnemonic::Vcvtps2pd:
            return asmjit::x86::Inst::kIdVcvtps2pd;
        case Mnemonic::Vcvtps2ph:
            return asmjit::x86::Inst::kIdVcvtps2ph;
        case Mnemonic::Vcvtps2qq:
            return asmjit::x86::Inst::kIdVcvtps2qq;
        case Mnemonic::Vcvtps2udq:
            return asmjit::x86::Inst::kIdVcvtps2udq;
        case Mnemonic::Vcvtps2uqq:
            return asmjit::x86::Inst::kIdVcvtps2uqq;
        case Mnemonic::Vcvtqq2pd:
            return asmjit::x86::Inst::kIdVcvtqq2pd;
        case Mnemonic::Vcvtqq2ps:
            return asmjit::x86::Inst::kIdVcvtqq2ps;
        case Mnemonic::Vcvtsd2si:
            return asmjit::x86::Inst::kIdVcvtsd2si;
        case Mnemonic::Vcvtsd2ss:
            return asmjit::x86::Inst::kIdVcvtsd2ss;
        case Mnemonic::Vcvtsd2usi:
            return asmjit::x86::Inst::kIdVcvtsd2usi;
        case Mnemonic::Vcvtsi2sd:
            return asmjit::x86::Inst::kIdVcvtsi2sd;
        case Mnemonic::Vcvtsi2ss:
            return asmjit::x86::Inst::kIdVcvtsi2ss;
        case Mnemonic::Vcvtss2sd:
            return asmjit::x86::Inst::kIdVcvtss2sd;
        case Mnemonic::Vcvtss2si:
            return asmjit::x86::Inst::kIdVcvtss2si;
        case Mnemonic::Vcvtss2usi:
            return asmjit::x86::Inst::kIdVcvtss2usi;
        case Mnemonic::Vcvttpd2dq:
            return asmjit::x86::Inst::kIdVcvttpd2dq;
        case Mnemonic::Vcvttpd2qq:
            return asmjit::x86::Inst::kIdVcvttpd2qq;
        case Mnemonic::Vcvttpd2udq:
            return asmjit::x86::Inst::kIdVcvttpd2udq;
        case Mnemonic::Vcvttpd2uqq:
            return asmjit::x86::Inst::kIdVcvttpd2uqq;
        case Mnemonic::Vcvttps2dq:
            return asmjit::x86::Inst::kIdVcvttps2dq;
        case Mnemonic::Vcvttps2qq:
            return asmjit::x86::Inst::kIdVcvttps2qq;
        case Mnemonic::Vcvttps2udq:
            return asmjit::x86::Inst::kIdVcvttps2udq;
        case Mnemonic::Vcvttps2uqq:
            return asmjit::x86::Inst::kIdVcvttps2uqq;
        case Mnemonic::Vcvttsd2si:
            return asmjit::x86::Inst::kIdVcvttsd2si;
        case Mnemonic::Vcvttsd2usi:
            return asmjit::x86::Inst::kIdVcvttsd2usi;
        case Mnemonic::Vcvttss2si:
            return asmjit::x86::Inst::kIdVcvttss2si;
        case Mnemonic::Vcvttss2usi:
            return asmjit::x86::Inst::kIdVcvttss2usi;
        case Mnemonic::Vcvtudq2pd:
            return asmjit::x86::Inst::kIdVcvtudq2pd;
        case Mnemonic::Vcvtudq2ps:
            return asmjit::x86::Inst::kIdVcvtudq2ps;
        case Mnemonic::Vcvtuqq2pd:
            return asmjit::x86::Inst::kIdVcvtuqq2pd;
        case Mnemonic::Vcvtuqq2ps:
            return asmjit::x86::Inst::kIdVcvtuqq2ps;
        case Mnemonic::Vcvtusi2sd:
            return asmjit::x86::Inst::kIdVcvtusi2sd;
        case Mnemonic::Vcvtusi2ss:
            return asmjit::x86::Inst::kIdVcvtusi2ss;
        case Mnemonic::Vdbpsadbw:
            return asmjit::x86::Inst::kIdVdbpsadbw;
        case Mnemonic::Vdivpd:
            return asmjit::x86::Inst::kIdVdivpd;
        case Mnemonic::Vdivps:
            return asmjit::x86::Inst::kIdVdivps;
        case Mnemonic::Vdivsd:
            return asmjit::x86::Inst::kIdVdivsd;
        case Mnemonic::Vdivss:
            return asmjit::x86::Inst::kIdVdivss;
        case Mnemonic::Vdppd:
            return asmjit::x86::Inst::kIdVdppd;
        case Mnemonic::Vdpps:
            return asmjit::x86::Inst::kIdVdpps;
        case Mnemonic::Verr:
            return asmjit::x86::Inst::kIdVerr;
        case Mnemonic::Verw:
            return asmjit::x86::Inst::kIdVerw;
        case Mnemonic::Vexp2pd:
            return asmjit::x86::Inst::kIdVexp2pd;
        case Mnemonic::Vexp2ps:
            return asmjit::x86::Inst::kIdVexp2ps;
        case Mnemonic::Vexpandpd:
            return asmjit::x86::Inst::kIdVexpandpd;
        case Mnemonic::Vexpandps:
            return asmjit::x86::Inst::kIdVexpandps;
        case Mnemonic::Vextractf128:
            return asmjit::x86::Inst::kIdVextractf128;
        case Mnemonic::Vextractf32x4:
            return asmjit::x86::Inst::kIdVextractf32x4;
        case Mnemonic::Vextractf32x8:
            return asmjit::x86::Inst::kIdVextractf32x8;
        case Mnemonic::Vextractf64x2:
            return asmjit::x86::Inst::kIdVextractf64x2;
        case Mnemonic::Vextractf64x4:
            return asmjit::x86::Inst::kIdVextractf64x4;
        case Mnemonic::Vextracti128:
            return asmjit::x86::Inst::kIdVextracti128;
        case Mnemonic::Vextracti32x4:
            return asmjit::x86::Inst::kIdVextracti32x4;
        case Mnemonic::Vextracti32x8:
            return asmjit::x86::Inst::kIdVextracti32x8;
        case Mnemonic::Vextracti64x2:
            return asmjit::x86::Inst::kIdVextracti64x2;
        case Mnemonic::Vextracti64x4:
            return asmjit::x86::Inst::kIdVextracti64x4;
        case Mnemonic::Vextractps:
            return asmjit::x86::Inst::kIdVextractps;
        case Mnemonic::Vfixupimmpd:
            return asmjit::x86::Inst::kIdVfixupimmpd;
        case Mnemonic::Vfixupimmps:
            return asmjit::x86::Inst::kIdVfixupimmps;
        case Mnemonic::Vfixupimmsd:
            return asmjit::x86::Inst::kIdVfixupimmsd;
        case Mnemonic::Vfixupimmss:
            return asmjit::x86::Inst::kIdVfixupimmss;
        case Mnemonic::Vfmadd132pd:
            return asmjit::x86::Inst::kIdVfmadd132pd;
        case Mnemonic::Vfmadd132ps:
            return asmjit::x86::Inst::kIdVfmadd132ps;
        case Mnemonic::Vfmadd132sd:
            return asmjit::x86::Inst::kIdVfmadd132sd;
        case Mnemonic::Vfmadd132ss:
            return asmjit::x86::Inst::kIdVfmadd132ss;
        case Mnemonic::Vfmadd213pd:
            return asmjit::x86::Inst::kIdVfmadd213pd;
        case Mnemonic::Vfmadd213ps:
            return asmjit::x86::Inst::kIdVfmadd213ps;
        case Mnemonic::Vfmadd213sd:
            return asmjit::x86::Inst::kIdVfmadd213sd;
        case Mnemonic::Vfmadd213ss:
            return asmjit::x86::Inst::kIdVfmadd213ss;
        case Mnemonic::Vfmadd231pd:
            return asmjit::x86::Inst::kIdVfmadd231pd;
        case Mnemonic::Vfmadd231ps:
            return asmjit::x86::Inst::kIdVfmadd231ps;
        case Mnemonic::Vfmadd231sd:
            return asmjit::x86::Inst::kIdVfmadd231sd;
        case Mnemonic::Vfmadd231ss:
            return asmjit::x86::Inst::kIdVfmadd231ss;
        case Mnemonic::Vfmaddpd:
            return asmjit::x86::Inst::kIdVfmaddpd;
        case Mnemonic::Vfmaddps:
            return asmjit::x86::Inst::kIdVfmaddps;
        case Mnemonic::Vfmaddsd:
            return asmjit::x86::Inst::kIdVfmaddsd;
        case Mnemonic::Vfmaddss:
            return asmjit::x86::Inst::kIdVfmaddss;
        case Mnemonic::Vfmaddsub132pd:
            return asmjit::x86::Inst::kIdVfmaddsub132pd;
        case Mnemonic::Vfmaddsub132ps:
            return asmjit::x86::Inst::kIdVfmaddsub132ps;
        case Mnemonic::Vfmaddsub213pd:
            return asmjit::x86::Inst::kIdVfmaddsub213pd;
        case Mnemonic::Vfmaddsub213ps:
            return asmjit::x86::Inst::kIdVfmaddsub213ps;
        case Mnemonic::Vfmaddsub231pd:
            return asmjit::x86::Inst::kIdVfmaddsub231pd;
        case Mnemonic::Vfmaddsub231ps:
            return asmjit::x86::Inst::kIdVfmaddsub231ps;
        case Mnemonic::Vfmaddsubpd:
            return asmjit::x86::Inst::kIdVfmaddsubpd;
        case Mnemonic::Vfmaddsubps:
            return asmjit::x86::Inst::kIdVfmaddsubps;
        case Mnemonic::Vfmsub132pd:
            return asmjit::x86::Inst::kIdVfmsub132pd;
        case Mnemonic::Vfmsub132ps:
            return asmjit::x86::Inst::kIdVfmsub132ps;
        case Mnemonic::Vfmsub132sd:
            return asmjit::x86::Inst::kIdVfmsub132sd;
        case Mnemonic::Vfmsub132ss:
            return asmjit::x86::Inst::kIdVfmsub132ss;
        case Mnemonic::Vfmsub213pd:
            return asmjit::x86::Inst::kIdVfmsub213pd;
        case Mnemonic::Vfmsub213ps:
            return asmjit::x86::Inst::kIdVfmsub213ps;
        case Mnemonic::Vfmsub213sd:
            return asmjit::x86::Inst::kIdVfmsub213sd;
        case Mnemonic::Vfmsub213ss:
            return asmjit::x86::Inst::kIdVfmsub213ss;
        case Mnemonic::Vfmsub231pd:
            return asmjit::x86::Inst::kIdVfmsub231pd;
        case Mnemonic::Vfmsub231ps:
            return asmjit::x86::Inst::kIdVfmsub231ps;
        case Mnemonic::Vfmsub231sd:
            return asmjit::x86::Inst::kIdVfmsub231sd;
        case Mnemonic::Vfmsub231ss:
            return asmjit::x86::Inst::kIdVfmsub231ss;
        case Mnemonic::Vfmsubadd132pd:
            return asmjit::x86::Inst::kIdVfmsubadd132pd;
        case Mnemonic::Vfmsubadd132ps:
            return asmjit::x86::Inst::kIdVfmsubadd132ps;
        case Mnemonic::Vfmsubadd213pd:
            return asmjit::x86::Inst::kIdVfmsubadd213pd;
        case Mnemonic::Vfmsubadd213ps:
            return asmjit::x86::Inst::kIdVfmsubadd213ps;
        case Mnemonic::Vfmsubadd231pd:
            return asmjit::x86::Inst::kIdVfmsubadd231pd;
        case Mnemonic::Vfmsubadd231ps:
            return asmjit::x86::Inst::kIdVfmsubadd231ps;
        case Mnemonic::Vfmsubaddpd:
            return asmjit::x86::Inst::kIdVfmsubaddpd;
        case Mnemonic::Vfmsubaddps:
            return asmjit::x86::Inst::kIdVfmsubaddps;
        case Mnemonic::Vfmsubpd:
            return asmjit::x86::Inst::kIdVfmsubpd;
        case Mnemonic::Vfmsubps:
            return asmjit::x86::Inst::kIdVfmsubps;
        case Mnemonic::Vfmsubsd:
            return asmjit::x86::Inst::kIdVfmsubsd;
        case Mnemonic::Vfmsubss:
            return asmjit::x86::Inst::kIdVfmsubss;
        case Mnemonic::Vfnmadd132pd:
            return asmjit::x86::Inst::kIdVfnmadd132pd;
        case Mnemonic::Vfnmadd132ps:
            return asmjit::x86::Inst::kIdVfnmadd132ps;
        case Mnemonic::Vfnmadd132sd:
            return asmjit::x86::Inst::kIdVfnmadd132sd;
        case Mnemonic::Vfnmadd132ss:
            return asmjit::x86::Inst::kIdVfnmadd132ss;
        case Mnemonic::Vfnmadd213pd:
            return asmjit::x86::Inst::kIdVfnmadd213pd;
        case Mnemonic::Vfnmadd213ps:
            return asmjit::x86::Inst::kIdVfnmadd213ps;
        case Mnemonic::Vfnmadd213sd:
            return asmjit::x86::Inst::kIdVfnmadd213sd;
        case Mnemonic::Vfnmadd213ss:
            return asmjit::x86::Inst::kIdVfnmadd213ss;
        case Mnemonic::Vfnmadd231pd:
            return asmjit::x86::Inst::kIdVfnmadd231pd;
        case Mnemonic::Vfnmadd231ps:
            return asmjit::x86::Inst::kIdVfnmadd231ps;
        case Mnemonic::Vfnmadd231sd:
            return asmjit::x86::Inst::kIdVfnmadd231sd;
        case Mnemonic::Vfnmadd231ss:
            return asmjit::x86::Inst::kIdVfnmadd231ss;
        case Mnemonic::Vfnmaddpd:
            return asmjit::x86::Inst::kIdVfnmaddpd;
        case Mnemonic::Vfnmaddps:
            return asmjit::x86::Inst::kIdVfnmaddps;
        case Mnemonic::Vfnmaddsd:
            return asmjit::x86::Inst::kIdVfnmaddsd;
        case Mnemonic::Vfnmaddss:
            return asmjit::x86::Inst::kIdVfnmaddss;
        case Mnemonic::Vfnmsub132pd:
            return asmjit::x86::Inst::kIdVfnmsub132pd;
        case Mnemonic::Vfnmsub132ps:
            return asmjit::x86::Inst::kIdVfnmsub132ps;
        case Mnemonic::Vfnmsub132sd:
            return asmjit::x86::Inst::kIdVfnmsub132sd;
        case Mnemonic::Vfnmsub132ss:
            return asmjit::x86::Inst::kIdVfnmsub132ss;
        case Mnemonic::Vfnmsub213pd:
            return asmjit::x86::Inst::kIdVfnmsub213pd;
        case Mnemonic::Vfnmsub213ps:
            return asmjit::x86::Inst::kIdVfnmsub213ps;
        case Mnemonic::Vfnmsub213sd:
            return asmjit::x86::Inst::kIdVfnmsub213sd;
        case Mnemonic::Vfnmsub213ss:
            return asmjit::x86::Inst::kIdVfnmsub213ss;
        case Mnemonic::Vfnmsub231pd:
            return asmjit::x86::Inst::kIdVfnmsub231pd;
        case Mnemonic::Vfnmsub231ps:
            return asmjit::x86::Inst::kIdVfnmsub231ps;
        case Mnemonic::Vfnmsub231sd:
            return asmjit::x86::Inst::kIdVfnmsub231sd;
        case Mnemonic::Vfnmsub231ss:
            return asmjit::x86::Inst::kIdVfnmsub231ss;
        case Mnemonic::Vfnmsubpd:
            return asmjit::x86::Inst::kIdVfnmsubpd;
        case Mnemonic::Vfnmsubps:
            return asmjit::x86::Inst::kIdVfnmsubps;
        case Mnemonic::Vfnmsubsd:
            return asmjit::x86::Inst::kIdVfnmsubsd;
        case Mnemonic::Vfnmsubss:
            return asmjit::x86::Inst::kIdVfnmsubss;
        case Mnemonic::Vfpclasspd:
            return asmjit::x86::Inst::kIdVfpclasspd;
        case Mnemonic::Vfpclassps:
            return asmjit::x86::Inst::kIdVfpclassps;
        case Mnemonic::Vfpclasssd:
            return asmjit::x86::Inst::kIdVfpclasssd;
        case Mnemonic::Vfpclassss:
            return asmjit::x86::Inst::kIdVfpclassss;
        case Mnemonic::Vfrczpd:
            return asmjit::x86::Inst::kIdVfrczpd;
        case Mnemonic::Vfrczps:
            return asmjit::x86::Inst::kIdVfrczps;
        case Mnemonic::Vfrczsd:
            return asmjit::x86::Inst::kIdVfrczsd;
        case Mnemonic::Vfrczss:
            return asmjit::x86::Inst::kIdVfrczss;
        case Mnemonic::Vgatherdpd:
            return asmjit::x86::Inst::kIdVgatherdpd;
        case Mnemonic::Vgatherdps:
            return asmjit::x86::Inst::kIdVgatherdps;
        case Mnemonic::Vgatherpf0dpd:
            return asmjit::x86::Inst::kIdVgatherpf0dpd;
        case Mnemonic::Vgatherpf0dps:
            return asmjit::x86::Inst::kIdVgatherpf0dps;
        case Mnemonic::Vgatherpf0qpd:
            return asmjit::x86::Inst::kIdVgatherpf0qpd;
        case Mnemonic::Vgatherpf0qps:
            return asmjit::x86::Inst::kIdVgatherpf0qps;
        case Mnemonic::Vgatherpf1dpd:
            return asmjit::x86::Inst::kIdVgatherpf1dpd;
        case Mnemonic::Vgatherpf1dps:
            return asmjit::x86::Inst::kIdVgatherpf1dps;
        case Mnemonic::Vgatherpf1qpd:
            return asmjit::x86::Inst::kIdVgatherpf1qpd;
        case Mnemonic::Vgatherpf1qps:
            return asmjit::x86::Inst::kIdVgatherpf1qps;
        case Mnemonic::Vgatherqpd:
            return asmjit::x86::Inst::kIdVgatherqpd;
        case Mnemonic::Vgatherqps:
            return asmjit::x86::Inst::kIdVgatherqps;
        case Mnemonic::Vgetexppd:
            return asmjit::x86::Inst::kIdVgetexppd;
        case Mnemonic::Vgetexpps:
            return asmjit::x86::Inst::kIdVgetexpps;
        case Mnemonic::Vgetexpsd:
            return asmjit::x86::Inst::kIdVgetexpsd;
        case Mnemonic::Vgetexpss:
            return asmjit::x86::Inst::kIdVgetexpss;
        case Mnemonic::Vgetmantpd:
            return asmjit::x86::Inst::kIdVgetmantpd;
        case Mnemonic::Vgetmantps:
            return asmjit::x86::Inst::kIdVgetmantps;
        case Mnemonic::Vgetmantsd:
            return asmjit::x86::Inst::kIdVgetmantsd;
        case Mnemonic::Vgetmantss:
            return asmjit::x86::Inst::kIdVgetmantss;
        case Mnemonic::Vhaddpd:
            return asmjit::x86::Inst::kIdVhaddpd;
        case Mnemonic::Vhaddps:
            return asmjit::x86::Inst::kIdVhaddps;
        case Mnemonic::Vhsubpd:
            return asmjit::x86::Inst::kIdVhsubpd;
        case Mnemonic::Vhsubps:
            return asmjit::x86::Inst::kIdVhsubps;
        case Mnemonic::Vinsertf128:
            return asmjit::x86::Inst::kIdVinsertf128;
        case Mnemonic::Vinsertf32x4:
            return asmjit::x86::Inst::kIdVinsertf32x4;
        case Mnemonic::Vinsertf32x8:
            return asmjit::x86::Inst::kIdVinsertf32x8;
        case Mnemonic::Vinsertf64x2:
            return asmjit::x86::Inst::kIdVinsertf64x2;
        case Mnemonic::Vinsertf64x4:
            return asmjit::x86::Inst::kIdVinsertf64x4;
        case Mnemonic::Vinserti128:
            return asmjit::x86::Inst::kIdVinserti128;
        case Mnemonic::Vinserti32x4:
            return asmjit::x86::Inst::kIdVinserti32x4;
        case Mnemonic::Vinserti32x8:
            return asmjit::x86::Inst::kIdVinserti32x8;
        case Mnemonic::Vinserti64x2:
            return asmjit::x86::Inst::kIdVinserti64x2;
        case Mnemonic::Vinserti64x4:
            return asmjit::x86::Inst::kIdVinserti64x4;
        case Mnemonic::Vinsertps:
            return asmjit::x86::Inst::kIdVinsertps;
        case Mnemonic::Vlddqu:
            return asmjit::x86::Inst::kIdVlddqu;
        case Mnemonic::Vldmxcsr:
            return asmjit::x86::Inst::kIdVldmxcsr;
        case Mnemonic::Vmaskmovdqu:
            return asmjit::x86::Inst::kIdVmaskmovdqu;
        case Mnemonic::Vmaskmovpd:
            return asmjit::x86::Inst::kIdVmaskmovpd;
        case Mnemonic::Vmaskmovps:
            return asmjit::x86::Inst::kIdVmaskmovps;
        case Mnemonic::Vmaxpd:
            return asmjit::x86::Inst::kIdVmaxpd;
        case Mnemonic::Vmaxps:
            return asmjit::x86::Inst::kIdVmaxps;
        case Mnemonic::Vmaxsd:
            return asmjit::x86::Inst::kIdVmaxsd;
        case Mnemonic::Vmaxss:
            return asmjit::x86::Inst::kIdVmaxss;
        case Mnemonic::Vminpd:
            return asmjit::x86::Inst::kIdVminpd;
        case Mnemonic::Vminps:
            return asmjit::x86::Inst::kIdVminps;
        case Mnemonic::Vminsd:
            return asmjit::x86::Inst::kIdVminsd;
        case Mnemonic::Vminss:
            return asmjit::x86::Inst::kIdVminss;
        case Mnemonic::Vmovapd:
            return asmjit::x86::Inst::kIdVmovapd;
        case Mnemonic::Vmovaps:
            return asmjit::x86::Inst::kIdVmovaps;
        case Mnemonic::Vmovd:
            return asmjit::x86::Inst::kIdVmovd;
        case Mnemonic::Vmovddup:
            return asmjit::x86::Inst::kIdVmovddup;
        case Mnemonic::Vmovdqa:
            return asmjit::x86::Inst::kIdVmovdqa;
        case Mnemonic::Vmovdqa32:
            return asmjit::x86::Inst::kIdVmovdqa32;
        case Mnemonic::Vmovdqa64:
            return asmjit::x86::Inst::kIdVmovdqa64;
        case Mnemonic::Vmovdqu:
            return asmjit::x86::Inst::kIdVmovdqu;
        case Mnemonic::Vmovdqu16:
            return asmjit::x86::Inst::kIdVmovdqu16;
        case Mnemonic::Vmovdqu32:
            return asmjit::x86::Inst::kIdVmovdqu32;
        case Mnemonic::Vmovdqu64:
            return asmjit::x86::Inst::kIdVmovdqu64;
        case Mnemonic::Vmovdqu8:
            return asmjit::x86::Inst::kIdVmovdqu8;
        case Mnemonic::Vmovhlps:
            return asmjit::x86::Inst::kIdVmovhlps;
        case Mnemonic::Vmovhpd:
            return asmjit::x86::Inst::kIdVmovhpd;
        case Mnemonic::Vmovhps:
            return asmjit::x86::Inst::kIdVmovhps;
        case Mnemonic::Vmovlhps:
            return asmjit::x86::Inst::kIdVmovlhps;
        case Mnemonic::Vmovlpd:
            return asmjit::x86::Inst::kIdVmovlpd;
        case Mnemonic::Vmovlps:
            return asmjit::x86::Inst::kIdVmovlps;
        case Mnemonic::Vmovmskpd:
            return asmjit::x86::Inst::kIdVmovmskpd;
        case Mnemonic::Vmovmskps:
            return asmjit::x86::Inst::kIdVmovmskps;
        case Mnemonic::Vmovntdq:
            return asmjit::x86::Inst::kIdVmovntdq;
        case Mnemonic::Vmovntdqa:
            return asmjit::x86::Inst::kIdVmovntdqa;
        case Mnemonic::Vmovntpd:
            return asmjit::x86::Inst::kIdVmovntpd;
        case Mnemonic::Vmovntps:
            return asmjit::x86::Inst::kIdVmovntps;
        case Mnemonic::Vmovq:
            return asmjit::x86::Inst::kIdVmovq;
        case Mnemonic::Vmovsd:
            return asmjit::x86::Inst::kIdVmovsd;
        case Mnemonic::Vmovshdup:
            return asmjit::x86::Inst::kIdVmovshdup;
        case Mnemonic::Vmovsldup:
            return asmjit::x86::Inst::kIdVmovsldup;
        case Mnemonic::Vmovss:
            return asmjit::x86::Inst::kIdVmovss;
        case Mnemonic::Vmovupd:
            return asmjit::x86::Inst::kIdVmovupd;
        case Mnemonic::Vmovups:
            return asmjit::x86::Inst::kIdVmovups;
        case Mnemonic::Vmpsadbw:
            return asmjit::x86::Inst::kIdVmpsadbw;
        case Mnemonic::Vmulpd:
            return asmjit::x86::Inst::kIdVmulpd;
        case Mnemonic::Vmulps:
            return asmjit::x86::Inst::kIdVmulps;
        case Mnemonic::Vmulsd:
            return asmjit::x86::Inst::kIdVmulsd;
        case Mnemonic::Vmulss:
            return asmjit::x86::Inst::kIdVmulss;
        case Mnemonic::Vorpd:
            return asmjit::x86::Inst::kIdVorpd;
        case Mnemonic::Vorps:
            return asmjit::x86::Inst::kIdVorps;
        case Mnemonic::Vp4dpwssd:
            return asmjit::x86::Inst::kIdVp4dpwssd;
        case Mnemonic::Vp4dpwssds:
            return asmjit::x86::Inst::kIdVp4dpwssds;
        case Mnemonic::Vpabsb:
            return asmjit::x86::Inst::kIdVpabsb;
        case Mnemonic::Vpabsd:
            return asmjit::x86::Inst::kIdVpabsd;
        case Mnemonic::Vpabsq:
            return asmjit::x86::Inst::kIdVpabsq;
        case Mnemonic::Vpabsw:
            return asmjit::x86::Inst::kIdVpabsw;
        case Mnemonic::Vpackssdw:
            return asmjit::x86::Inst::kIdVpackssdw;
        case Mnemonic::Vpacksswb:
            return asmjit::x86::Inst::kIdVpacksswb;
        case Mnemonic::Vpackusdw:
            return asmjit::x86::Inst::kIdVpackusdw;
        case Mnemonic::Vpackuswb:
            return asmjit::x86::Inst::kIdVpackuswb;
        case Mnemonic::Vpaddb:
            return asmjit::x86::Inst::kIdVpaddb;
        case Mnemonic::Vpaddd:
            return asmjit::x86::Inst::kIdVpaddd;
        case Mnemonic::Vpaddq:
            return asmjit::x86::Inst::kIdVpaddq;
        case Mnemonic::Vpaddsb:
            return asmjit::x86::Inst::kIdVpaddsb;
        case Mnemonic::Vpaddsw:
            return asmjit::x86::Inst::kIdVpaddsw;
        case Mnemonic::Vpaddusb:
            return asmjit::x86::Inst::kIdVpaddusb;
        case Mnemonic::Vpaddusw:
            return asmjit::x86::Inst::kIdVpaddusw;
        case Mnemonic::Vpaddw:
            return asmjit::x86::Inst::kIdVpaddw;
        case Mnemonic::Vpalignr:
            return asmjit::x86::Inst::kIdVpalignr;
        case Mnemonic::Vpand:
            return asmjit::x86::Inst::kIdVpand;
        case Mnemonic::Vpandd:
            return asmjit::x86::Inst::kIdVpandd;
        case Mnemonic::Vpandn:
            return asmjit::x86::Inst::kIdVpandn;
        case Mnemonic::Vpandnd:
            return asmjit::x86::Inst::kIdVpandnd;
        case Mnemonic::Vpandnq:
            return asmjit::x86::Inst::kIdVpandnq;
        case Mnemonic::Vpandq:
            return asmjit::x86::Inst::kIdVpandq;
        case Mnemonic::Vpavgb:
            return asmjit::x86::Inst::kIdVpavgb;
        case Mnemonic::Vpavgw:
            return asmjit::x86::Inst::kIdVpavgw;
        case Mnemonic::Vpblendd:
            return asmjit::x86::Inst::kIdVpblendd;
        case Mnemonic::Vpblendvb:
            return asmjit::x86::Inst::kIdVpblendvb;
        case Mnemonic::Vpblendw:
            return asmjit::x86::Inst::kIdVpblendw;
        case Mnemonic::Vpbroadcastb:
            return asmjit::x86::Inst::kIdVpbroadcastb;
        case Mnemonic::Vpbroadcastd:
            return asmjit::x86::Inst::kIdVpbroadcastd;
        case Mnemonic::Vpbroadcastmb2q:
            return asmjit::x86::Inst::kIdVpbroadcastmb2q;
        case Mnemonic::Vpbroadcastq:
            return asmjit::x86::Inst::kIdVpbroadcastq;
        case Mnemonic::Vpbroadcastw:
            return asmjit::x86::Inst::kIdVpbroadcastw;
        case Mnemonic::Vpclmulqdq:
            return asmjit::x86::Inst::kIdVpclmulqdq;
        case Mnemonic::Vpcmov:
            return asmjit::x86::Inst::kIdVpcmov;
        case Mnemonic::Vpcmpb:
            return asmjit::x86::Inst::kIdVpcmpb;
        case Mnemonic::Vpcmpd:
            return asmjit::x86::Inst::kIdVpcmpd;
        case Mnemonic::Vpcmpeqb:
            return asmjit::x86::Inst::kIdVpcmpeqb;
        case Mnemonic::Vpcmpeqd:
            return asmjit::x86::Inst::kIdVpcmpeqd;
        case Mnemonic::Vpcmpeqq:
            return asmjit::x86::Inst::kIdVpcmpeqq;
        case Mnemonic::Vpcmpeqw:
            return asmjit::x86::Inst::kIdVpcmpeqw;
        case Mnemonic::Vpcmpestri:
            return asmjit::x86::Inst::kIdVpcmpestri;
        case Mnemonic::Vpcmpestrm:
            return asmjit::x86::Inst::kIdVpcmpestrm;
        case Mnemonic::Vpcmpgtb:
            return asmjit::x86::Inst::kIdVpcmpgtb;
        case Mnemonic::Vpcmpgtd:
            return asmjit::x86::Inst::kIdVpcmpgtd;
        case Mnemonic::Vpcmpgtq:
            return asmjit::x86::Inst::kIdVpcmpgtq;
        case Mnemonic::Vpcmpgtw:
            return asmjit::x86::Inst::kIdVpcmpgtw;
        case Mnemonic::Vpcmpistri:
            return asmjit::x86::Inst::kIdVpcmpistri;
        case Mnemonic::Vpcmpistrm:
            return asmjit::x86::Inst::kIdVpcmpistrm;
        case Mnemonic::Vpcmpq:
            return asmjit::x86::Inst::kIdVpcmpq;
        case Mnemonic::Vpcmpub:
            return asmjit::x86::Inst::kIdVpcmpub;
        case Mnemonic::Vpcmpud:
            return asmjit::x86::Inst::kIdVpcmpud;
        case Mnemonic::Vpcmpuq:
            return asmjit::x86::Inst::kIdVpcmpuq;
        case Mnemonic::Vpcmpuw:
            return asmjit::x86::Inst::kIdVpcmpuw;
        case Mnemonic::Vpcmpw:
            return asmjit::x86::Inst::kIdVpcmpw;
        case Mnemonic::Vpcomb:
            return asmjit::x86::Inst::kIdVpcomb;
        case Mnemonic::Vpcomd:
            return asmjit::x86::Inst::kIdVpcomd;
        case Mnemonic::Vpcompressb:
            return asmjit::x86::Inst::kIdVpcompressb;
        case Mnemonic::Vpcompressd:
            return asmjit::x86::Inst::kIdVpcompressd;
        case Mnemonic::Vpcompressq:
            return asmjit::x86::Inst::kIdVpcompressq;
        case Mnemonic::Vpcompressw:
            return asmjit::x86::Inst::kIdVpcompressw;
        case Mnemonic::Vpcomq:
            return asmjit::x86::Inst::kIdVpcomq;
        case Mnemonic::Vpcomub:
            return asmjit::x86::Inst::kIdVpcomub;
        case Mnemonic::Vpcomud:
            return asmjit::x86::Inst::kIdVpcomud;
        case Mnemonic::Vpcomuq:
            return asmjit::x86::Inst::kIdVpcomuq;
        case Mnemonic::Vpcomuw:
            return asmjit::x86::Inst::kIdVpcomuw;
        case Mnemonic::Vpcomw:
            return asmjit::x86::Inst::kIdVpcomw;
        case Mnemonic::Vpconflictd:
            return asmjit::x86::Inst::kIdVpconflictd;
        case Mnemonic::Vpconflictq:
            return asmjit::x86::Inst::kIdVpconflictq;
        case Mnemonic::Vperm2f128:
            return asmjit::x86::Inst::kIdVperm2f128;
        case Mnemonic::Vperm2i128:
            return asmjit::x86::Inst::kIdVperm2i128;
        case Mnemonic::Vpermb:
            return asmjit::x86::Inst::kIdVpermb;
        case Mnemonic::Vpermd:
            return asmjit::x86::Inst::kIdVpermd;
        case Mnemonic::Vpermi2b:
            return asmjit::x86::Inst::kIdVpermi2b;
        case Mnemonic::Vpermi2d:
            return asmjit::x86::Inst::kIdVpermi2d;
        case Mnemonic::Vpermi2pd:
            return asmjit::x86::Inst::kIdVpermi2pd;
        case Mnemonic::Vpermi2ps:
            return asmjit::x86::Inst::kIdVpermi2ps;
        case Mnemonic::Vpermi2q:
            return asmjit::x86::Inst::kIdVpermi2q;
        case Mnemonic::Vpermi2w:
            return asmjit::x86::Inst::kIdVpermi2w;
        case Mnemonic::Vpermil2pd:
            return asmjit::x86::Inst::kIdVpermil2pd;
        case Mnemonic::Vpermil2ps:
            return asmjit::x86::Inst::kIdVpermil2ps;
        case Mnemonic::Vpermilpd:
            return asmjit::x86::Inst::kIdVpermilpd;
        case Mnemonic::Vpermilps:
            return asmjit::x86::Inst::kIdVpermilps;
        case Mnemonic::Vpermpd:
            return asmjit::x86::Inst::kIdVpermpd;
        case Mnemonic::Vpermps:
            return asmjit::x86::Inst::kIdVpermps;
        case Mnemonic::Vpermq:
            return asmjit::x86::Inst::kIdVpermq;
        case Mnemonic::Vpermt2b:
            return asmjit::x86::Inst::kIdVpermt2b;
        case Mnemonic::Vpermt2d:
            return asmjit::x86::Inst::kIdVpermt2d;
        case Mnemonic::Vpermt2pd:
            return asmjit::x86::Inst::kIdVpermt2pd;
        case Mnemonic::Vpermt2ps:
            return asmjit::x86::Inst::kIdVpermt2ps;
        case Mnemonic::Vpermt2q:
            return asmjit::x86::Inst::kIdVpermt2q;
        case Mnemonic::Vpermt2w:
            return asmjit::x86::Inst::kIdVpermt2w;
        case Mnemonic::Vpermw:
            return asmjit::x86::Inst::kIdVpermw;
        case Mnemonic::Vpexpandb:
            return asmjit::x86::Inst::kIdVpexpandb;
        case Mnemonic::Vpexpandd:
            return asmjit::x86::Inst::kIdVpexpandd;
        case Mnemonic::Vpexpandq:
            return asmjit::x86::Inst::kIdVpexpandq;
        case Mnemonic::Vpexpandw:
            return asmjit::x86::Inst::kIdVpexpandw;
        case Mnemonic::Vpextrb:
            return asmjit::x86::Inst::kIdVpextrb;
        case Mnemonic::Vpextrd:
            return asmjit::x86::Inst::kIdVpextrd;
        case Mnemonic::Vpextrq:
            return asmjit::x86::Inst::kIdVpextrq;
        case Mnemonic::Vpextrw:
            return asmjit::x86::Inst::kIdVpextrw;
        case Mnemonic::Vpgatherdd:
            return asmjit::x86::Inst::kIdVpgatherdd;
        case Mnemonic::Vpgatherdq:
            return asmjit::x86::Inst::kIdVpgatherdq;
        case Mnemonic::Vpgatherqd:
            return asmjit::x86::Inst::kIdVpgatherqd;
        case Mnemonic::Vpgatherqq:
            return asmjit::x86::Inst::kIdVpgatherqq;
        case Mnemonic::Vphaddbd:
            return asmjit::x86::Inst::kIdVphaddbd;
        case Mnemonic::Vphaddbq:
            return asmjit::x86::Inst::kIdVphaddbq;
        case Mnemonic::Vphaddbw:
            return asmjit::x86::Inst::kIdVphaddbw;
        case Mnemonic::Vphaddd:
            return asmjit::x86::Inst::kIdVphaddd;
        case Mnemonic::Vphadddq:
            return asmjit::x86::Inst::kIdVphadddq;
        case Mnemonic::Vphaddsw:
            return asmjit::x86::Inst::kIdVphaddsw;
        case Mnemonic::Vphaddubd:
            return asmjit::x86::Inst::kIdVphaddubd;
        case Mnemonic::Vphaddubq:
            return asmjit::x86::Inst::kIdVphaddubq;
        case Mnemonic::Vphaddubw:
            return asmjit::x86::Inst::kIdVphaddubw;
        case Mnemonic::Vphaddudq:
            return asmjit::x86::Inst::kIdVphaddudq;
        case Mnemonic::Vphadduwd:
            return asmjit::x86::Inst::kIdVphadduwd;
        case Mnemonic::Vphadduwq:
            return asmjit::x86::Inst::kIdVphadduwq;
        case Mnemonic::Vphaddw:
            return asmjit::x86::Inst::kIdVphaddw;
        case Mnemonic::Vphaddwd:
            return asmjit::x86::Inst::kIdVphaddwd;
        case Mnemonic::Vphaddwq:
            return asmjit::x86::Inst::kIdVphaddwq;
        case Mnemonic::Vphminposuw:
            return asmjit::x86::Inst::kIdVphminposuw;
        case Mnemonic::Vphsubbw:
            return asmjit::x86::Inst::kIdVphsubbw;
        case Mnemonic::Vphsubd:
            return asmjit::x86::Inst::kIdVphsubd;
        case Mnemonic::Vphsubdq:
            return asmjit::x86::Inst::kIdVphsubdq;
        case Mnemonic::Vphsubsw:
            return asmjit::x86::Inst::kIdVphsubsw;
        case Mnemonic::Vphsubw:
            return asmjit::x86::Inst::kIdVphsubw;
        case Mnemonic::Vphsubwd:
            return asmjit::x86::Inst::kIdVphsubwd;
        case Mnemonic::Vpinsrb:
            return asmjit::x86::Inst::kIdVpinsrb;
        case Mnemonic::Vpinsrd:
            return asmjit::x86::Inst::kIdVpinsrd;
        case Mnemonic::Vpinsrq:
            return asmjit::x86::Inst::kIdVpinsrq;
        case Mnemonic::Vpinsrw:
            return asmjit::x86::Inst::kIdVpinsrw;
        case Mnemonic::Vplzcntd:
            return asmjit::x86::Inst::kIdVplzcntd;
        case Mnemonic::Vplzcntq:
            return asmjit::x86::Inst::kIdVplzcntq;
        case Mnemonic::Vpmacsdd:
            return asmjit::x86::Inst::kIdVpmacsdd;
        case Mnemonic::Vpmacsdqh:
            return asmjit::x86::Inst::kIdVpmacsdqh;
        case Mnemonic::Vpmacsdql:
            return asmjit::x86::Inst::kIdVpmacsdql;
        case Mnemonic::Vpmacssdd:
            return asmjit::x86::Inst::kIdVpmacssdd;
        case Mnemonic::Vpmacssdqh:
            return asmjit::x86::Inst::kIdVpmacssdqh;
        case Mnemonic::Vpmacssdql:
            return asmjit::x86::Inst::kIdVpmacssdql;
        case Mnemonic::Vpmacsswd:
            return asmjit::x86::Inst::kIdVpmacsswd;
        case Mnemonic::Vpmacssww:
            return asmjit::x86::Inst::kIdVpmacssww;
        case Mnemonic::Vpmacswd:
            return asmjit::x86::Inst::kIdVpmacswd;
        case Mnemonic::Vpmacsww:
            return asmjit::x86::Inst::kIdVpmacsww;
        case Mnemonic::Vpmadcsswd:
            return asmjit::x86::Inst::kIdVpmadcsswd;
        case Mnemonic::Vpmadcswd:
            return asmjit::x86::Inst::kIdVpmadcswd;
        case Mnemonic::Vpmadd52huq:
            return asmjit::x86::Inst::kIdVpmadd52huq;
        case Mnemonic::Vpmadd52luq:
            return asmjit::x86::Inst::kIdVpmadd52luq;
        case Mnemonic::Vpmaddubsw:
            return asmjit::x86::Inst::kIdVpmaddubsw;
        case Mnemonic::Vpmaddwd:
            return asmjit::x86::Inst::kIdVpmaddwd;
        case Mnemonic::Vpmaskmovd:
            return asmjit::x86::Inst::kIdVpmaskmovd;
        case Mnemonic::Vpmaskmovq:
            return asmjit::x86::Inst::kIdVpmaskmovq;
        case Mnemonic::Vpmaxsb:
            return asmjit::x86::Inst::kIdVpmaxsb;
        case Mnemonic::Vpmaxsd:
            return asmjit::x86::Inst::kIdVpmaxsd;
        case Mnemonic::Vpmaxsq:
            return asmjit::x86::Inst::kIdVpmaxsq;
        case Mnemonic::Vpmaxsw:
            return asmjit::x86::Inst::kIdVpmaxsw;
        case Mnemonic::Vpmaxub:
            return asmjit::x86::Inst::kIdVpmaxub;
        case Mnemonic::Vpmaxud:
            return asmjit::x86::Inst::kIdVpmaxud;
        case Mnemonic::Vpmaxuq:
            return asmjit::x86::Inst::kIdVpmaxuq;
        case Mnemonic::Vpmaxuw:
            return asmjit::x86::Inst::kIdVpmaxuw;
        case Mnemonic::Vpminsb:
            return asmjit::x86::Inst::kIdVpminsb;
        case Mnemonic::Vpminsd:
            return asmjit::x86::Inst::kIdVpminsd;
        case Mnemonic::Vpminsq:
            return asmjit::x86::Inst::kIdVpminsq;
        case Mnemonic::Vpminsw:
            return asmjit::x86::Inst::kIdVpminsw;
        case Mnemonic::Vpminub:
            return asmjit::x86::Inst::kIdVpminub;
        case Mnemonic::Vpminud:
            return asmjit::x86::Inst::kIdVpminud;
        case Mnemonic::Vpminuq:
            return asmjit::x86::Inst::kIdVpminuq;
        case Mnemonic::Vpminuw:
            return asmjit::x86::Inst::kIdVpminuw;
        case Mnemonic::Vpmovb2m:
            return asmjit::x86::Inst::kIdVpmovb2m;
        case Mnemonic::Vpmovd2m:
            return asmjit::x86::Inst::kIdVpmovd2m;
        case Mnemonic::Vpmovdb:
            return asmjit::x86::Inst::kIdVpmovdb;
        case Mnemonic::Vpmovdw:
            return asmjit::x86::Inst::kIdVpmovdw;
        case Mnemonic::Vpmovm2b:
            return asmjit::x86::Inst::kIdVpmovm2b;
        case Mnemonic::Vpmovm2d:
            return asmjit::x86::Inst::kIdVpmovm2d;
        case Mnemonic::Vpmovm2q:
            return asmjit::x86::Inst::kIdVpmovm2q;
        case Mnemonic::Vpmovm2w:
            return asmjit::x86::Inst::kIdVpmovm2w;
        case Mnemonic::Vpmovmskb:
            return asmjit::x86::Inst::kIdVpmovmskb;
        case Mnemonic::Vpmovq2m:
            return asmjit::x86::Inst::kIdVpmovq2m;
        case Mnemonic::Vpmovqb:
            return asmjit::x86::Inst::kIdVpmovqb;
        case Mnemonic::Vpmovqd:
            return asmjit::x86::Inst::kIdVpmovqd;
        case Mnemonic::Vpmovqw:
            return asmjit::x86::Inst::kIdVpmovqw;
        case Mnemonic::Vpmovsdb:
            return asmjit::x86::Inst::kIdVpmovsdb;
        case Mnemonic::Vpmovsdw:
            return asmjit::x86::Inst::kIdVpmovsdw;
        case Mnemonic::Vpmovsqb:
            return asmjit::x86::Inst::kIdVpmovsqb;
        case Mnemonic::Vpmovsqd:
            return asmjit::x86::Inst::kIdVpmovsqd;
        case Mnemonic::Vpmovsqw:
            return asmjit::x86::Inst::kIdVpmovsqw;
        case Mnemonic::Vpmovswb:
            return asmjit::x86::Inst::kIdVpmovswb;
        case Mnemonic::Vpmovsxbd:
            return asmjit::x86::Inst::kIdVpmovsxbd;
        case Mnemonic::Vpmovsxbq:
            return asmjit::x86::Inst::kIdVpmovsxbq;
        case Mnemonic::Vpmovsxbw:
            return asmjit::x86::Inst::kIdVpmovsxbw;
        case Mnemonic::Vpmovsxdq:
            return asmjit::x86::Inst::kIdVpmovsxdq;
        case Mnemonic::Vpmovsxwd:
            return asmjit::x86::Inst::kIdVpmovsxwd;
        case Mnemonic::Vpmovsxwq:
            return asmjit::x86::Inst::kIdVpmovsxwq;
        case Mnemonic::Vpmovusdb:
            return asmjit::x86::Inst::kIdVpmovusdb;
        case Mnemonic::Vpmovusdw:
            return asmjit::x86::Inst::kIdVpmovusdw;
        case Mnemonic::Vpmovusqb:
            return asmjit::x86::Inst::kIdVpmovusqb;
        case Mnemonic::Vpmovusqd:
            return asmjit::x86::Inst::kIdVpmovusqd;
        case Mnemonic::Vpmovusqw:
            return asmjit::x86::Inst::kIdVpmovusqw;
        case Mnemonic::Vpmovuswb:
            return asmjit::x86::Inst::kIdVpmovuswb;
        case Mnemonic::Vpmovw2m:
            return asmjit::x86::Inst::kIdVpmovw2m;
        case Mnemonic::Vpmovwb:
            return asmjit::x86::Inst::kIdVpmovwb;
        case Mnemonic::Vpmovzxbd:
            return asmjit::x86::Inst::kIdVpmovzxbd;
        case Mnemonic::Vpmovzxbq:
            return asmjit::x86::Inst::kIdVpmovzxbq;
        case Mnemonic::Vpmovzxbw:
            return asmjit::x86::Inst::kIdVpmovzxbw;
        case Mnemonic::Vpmovzxdq:
            return asmjit::x86::Inst::kIdVpmovzxdq;
        case Mnemonic::Vpmovzxwd:
            return asmjit::x86::Inst::kIdVpmovzxwd;
        case Mnemonic::Vpmovzxwq:
            return asmjit::x86::Inst::kIdVpmovzxwq;
        case Mnemonic::Vpmuldq:
            return asmjit::x86::Inst::kIdVpmuldq;
        case Mnemonic::Vpmulhrsw:
            return asmjit::x86::Inst::kIdVpmulhrsw;
        case Mnemonic::Vpmulhuw:
            return asmjit::x86::Inst::kIdVpmulhuw;
        case Mnemonic::Vpmulhw:
            return asmjit::x86::Inst::kIdVpmulhw;
        case Mnemonic::Vpmulld:
            return asmjit::x86::Inst::kIdVpmulld;
        case Mnemonic::Vpmullq:
            return asmjit::x86::Inst::kIdVpmullq;
        case Mnemonic::Vpmullw:
            return asmjit::x86::Inst::kIdVpmullw;
        case Mnemonic::Vpmultishiftqb:
            return asmjit::x86::Inst::kIdVpmultishiftqb;
        case Mnemonic::Vpmuludq:
            return asmjit::x86::Inst::kIdVpmuludq;
        case Mnemonic::Vpopcntb:
            return asmjit::x86::Inst::kIdVpopcntb;
        case Mnemonic::Vpopcntd:
            return asmjit::x86::Inst::kIdVpopcntd;
        case Mnemonic::Vpopcntq:
            return asmjit::x86::Inst::kIdVpopcntq;
        case Mnemonic::Vpopcntw:
            return asmjit::x86::Inst::kIdVpopcntw;
        case Mnemonic::Vpor:
            return asmjit::x86::Inst::kIdVpor;
        case Mnemonic::Vpord:
            return asmjit::x86::Inst::kIdVpord;
        case Mnemonic::Vporq:
            return asmjit::x86::Inst::kIdVporq;
        case Mnemonic::Vpperm:
            return asmjit::x86::Inst::kIdVpperm;
        case Mnemonic::Vprold:
            return asmjit::x86::Inst::kIdVprold;
        case Mnemonic::Vprolq:
            return asmjit::x86::Inst::kIdVprolq;
        case Mnemonic::Vprolvd:
            return asmjit::x86::Inst::kIdVprolvd;
        case Mnemonic::Vprolvq:
            return asmjit::x86::Inst::kIdVprolvq;
        case Mnemonic::Vprord:
            return asmjit::x86::Inst::kIdVprord;
        case Mnemonic::Vprorq:
            return asmjit::x86::Inst::kIdVprorq;
        case Mnemonic::Vprorvd:
            return asmjit::x86::Inst::kIdVprorvd;
        case Mnemonic::Vprorvq:
            return asmjit::x86::Inst::kIdVprorvq;
        case Mnemonic::Vprotb:
            return asmjit::x86::Inst::kIdVprotb;
        case Mnemonic::Vprotd:
            return asmjit::x86::Inst::kIdVprotd;
        case Mnemonic::Vprotq:
            return asmjit::x86::Inst::kIdVprotq;
        case Mnemonic::Vprotw:
            return asmjit::x86::Inst::kIdVprotw;
        case Mnemonic::Vpsadbw:
            return asmjit::x86::Inst::kIdVpsadbw;
        case Mnemonic::Vpscatterdd:
            return asmjit::x86::Inst::kIdVpscatterdd;
        case Mnemonic::Vpscatterdq:
            return asmjit::x86::Inst::kIdVpscatterdq;
        case Mnemonic::Vpscatterqd:
            return asmjit::x86::Inst::kIdVpscatterqd;
        case Mnemonic::Vpscatterqq:
            return asmjit::x86::Inst::kIdVpscatterqq;
        case Mnemonic::Vpshab:
            return asmjit::x86::Inst::kIdVpshab;
        case Mnemonic::Vpshad:
            return asmjit::x86::Inst::kIdVpshad;
        case Mnemonic::Vpshaq:
            return asmjit::x86::Inst::kIdVpshaq;
        case Mnemonic::Vpshaw:
            return asmjit::x86::Inst::kIdVpshaw;
        case Mnemonic::Vpshlb:
            return asmjit::x86::Inst::kIdVpshlb;
        case Mnemonic::Vpshld:
            return asmjit::x86::Inst::kIdVpshld;
        case Mnemonic::Vpshldd:
            return asmjit::x86::Inst::kIdVpshldd;
        case Mnemonic::Vpshldq:
            return asmjit::x86::Inst::kIdVpshldq;
        case Mnemonic::Vpshldvd:
            return asmjit::x86::Inst::kIdVpshldvd;
        case Mnemonic::Vpshldvq:
            return asmjit::x86::Inst::kIdVpshldvq;
        case Mnemonic::Vpshldvw:
            return asmjit::x86::Inst::kIdVpshldvw;
        case Mnemonic::Vpshldw:
            return asmjit::x86::Inst::kIdVpshldw;
        case Mnemonic::Vpshlq:
            return asmjit::x86::Inst::kIdVpshlq;
        case Mnemonic::Vpshlw:
            return asmjit::x86::Inst::kIdVpshlw;
        case Mnemonic::Vpshrdd:
            return asmjit::x86::Inst::kIdVpshrdd;
        case Mnemonic::Vpshrdq:
            return asmjit::x86::Inst::kIdVpshrdq;
        case Mnemonic::Vpshrdvd:
            return asmjit::x86::Inst::kIdVpshrdvd;
        case Mnemonic::Vpshrdvq:
            return asmjit::x86::Inst::kIdVpshrdvq;
        case Mnemonic::Vpshrdvw:
            return asmjit::x86::Inst::kIdVpshrdvw;
        case Mnemonic::Vpshrdw:
            return asmjit::x86::Inst::kIdVpshrdw;
        case Mnemonic::Vpshufb:
            return asmjit::x86::Inst::kIdVpshufb;
        case Mnemonic::Vpshufbitqmb:
            return asmjit::x86::Inst::kIdVpshufbitqmb;
        case Mnemonic::Vpshufd:
            return asmjit::x86::Inst::kIdVpshufd;
        case Mnemonic::Vpshufhw:
            return asmjit::x86::Inst::kIdVpshufhw;
        case Mnemonic::Vpshuflw:
            return asmjit::x86::Inst::kIdVpshuflw;
        case Mnemonic::Vpsignb:
            return asmjit::x86::Inst::kIdVpsignb;
        case Mnemonic::Vpsignd:
            return asmjit::x86::Inst::kIdVpsignd;
        case Mnemonic::Vpsignw:
            return asmjit::x86::Inst::kIdVpsignw;
        case Mnemonic::Vpslld:
            return asmjit::x86::Inst::kIdVpslld;
        case Mnemonic::Vpslldq:
            return asmjit::x86::Inst::kIdVpslldq;
        case Mnemonic::Vpsllq:
            return asmjit::x86::Inst::kIdVpsllq;
        case Mnemonic::Vpsllvd:
            return asmjit::x86::Inst::kIdVpsllvd;
        case Mnemonic::Vpsllvq:
            return asmjit::x86::Inst::kIdVpsllvq;
        case Mnemonic::Vpsllvw:
            return asmjit::x86::Inst::kIdVpsllvw;
        case Mnemonic::Vpsllw:
            return asmjit::x86::Inst::kIdVpsllw;
        case Mnemonic::Vpsrad:
            return asmjit::x86::Inst::kIdVpsrad;
        case Mnemonic::Vpsraq:
            return asmjit::x86::Inst::kIdVpsraq;
        case Mnemonic::Vpsravd:
            return asmjit::x86::Inst::kIdVpsravd;
        case Mnemonic::Vpsravq:
            return asmjit::x86::Inst::kIdVpsravq;
        case Mnemonic::Vpsravw:
            return asmjit::x86::Inst::kIdVpsravw;
        case Mnemonic::Vpsraw:
            return asmjit::x86::Inst::kIdVpsraw;
        case Mnemonic::Vpsrld:
            return asmjit::x86::Inst::kIdVpsrld;
        case Mnemonic::Vpsrldq:
            return asmjit::x86::Inst::kIdVpsrldq;
        case Mnemonic::Vpsrlq:
            return asmjit::x86::Inst::kIdVpsrlq;
        case Mnemonic::Vpsrlvd:
            return asmjit::x86::Inst::kIdVpsrlvd;
        case Mnemonic::Vpsrlvq:
            return asmjit::x86::Inst::kIdVpsrlvq;
        case Mnemonic::Vpsrlvw:
            return asmjit::x86::Inst::kIdVpsrlvw;
        case Mnemonic::Vpsrlw:
            return asmjit::x86::Inst::kIdVpsrlw;
        case Mnemonic::Vpsubb:
            return asmjit::x86::Inst::kIdVpsubb;
        case Mnemonic::Vpsubd:
            return asmjit::x86::Inst::kIdVpsubd;
        case Mnemonic::Vpsubq:
            return asmjit::x86::Inst::kIdVpsubq;
        case Mnemonic::Vpsubsb:
            return asmjit::x86::Inst::kIdVpsubsb;
        case Mnemonic::Vpsubsw:
            return asmjit::x86::Inst::kIdVpsubsw;
        case Mnemonic::Vpsubusb:
            return asmjit::x86::Inst::kIdVpsubusb;
        case Mnemonic::Vpsubusw:
            return asmjit::x86::Inst::kIdVpsubusw;
        case Mnemonic::Vpsubw:
            return asmjit::x86::Inst::kIdVpsubw;
        case Mnemonic::Vpternlogd:
            return asmjit::x86::Inst::kIdVpternlogd;
        case Mnemonic::Vpternlogq:
            return asmjit::x86::Inst::kIdVpternlogq;
        case Mnemonic::Vptest:
            return asmjit::x86::Inst::kIdVptest;
        case Mnemonic::Vptestmb:
            return asmjit::x86::Inst::kIdVptestmb;
        case Mnemonic::Vptestmd:
            return asmjit::x86::Inst::kIdVptestmd;
        case Mnemonic::Vptestmq:
            return asmjit::x86::Inst::kIdVptestmq;
        case Mnemonic::Vptestmw:
            return asmjit::x86::Inst::kIdVptestmw;
        case Mnemonic::Vptestnmb:
            return asmjit::x86::Inst::kIdVptestnmb;
        case Mnemonic::Vptestnmd:
            return asmjit::x86::Inst::kIdVptestnmd;
        case Mnemonic::Vptestnmq:
            return asmjit::x86::Inst::kIdVptestnmq;
        case Mnemonic::Vptestnmw:
            return asmjit::x86::Inst::kIdVptestnmw;
        case Mnemonic::Vpunpckhbw:
            return asmjit::x86::Inst::kIdVpunpckhbw;
        case Mnemonic::Vpunpckhdq:
            return asmjit::x86::Inst::kIdVpunpckhdq;
        case Mnemonic::Vpunpckhqdq:
            return asmjit::x86::Inst::kIdVpunpckhqdq;
        case Mnemonic::Vpunpckhwd:
            return asmjit::x86::Inst::kIdVpunpckhwd;
        case Mnemonic::Vpunpcklbw:
            return asmjit::x86::Inst::kIdVpunpcklbw;
        case Mnemonic::Vpunpckldq:
            return asmjit::x86::Inst::kIdVpunpckldq;
        case Mnemonic::Vpunpcklqdq:
            return asmjit::x86::Inst::kIdVpunpcklqdq;
        case Mnemonic::Vpunpcklwd:
            return asmjit::x86::Inst::kIdVpunpcklwd;
        case Mnemonic::Vpxor:
            return asmjit::x86::Inst::kIdVpxor;
        case Mnemonic::Vpxord:
            return asmjit::x86::Inst::kIdVpxord;
        case Mnemonic::Vpxorq:
            return asmjit::x86::Inst::kIdVpxorq;
        case Mnemonic::Vrangepd:
            return asmjit::x86::Inst::kIdVrangepd;
        case Mnemonic::Vrangeps:
            return asmjit::x86::Inst::kIdVrangeps;
        case Mnemonic::Vrangesd:
            return asmjit::x86::Inst::kIdVrangesd;
        case Mnemonic::Vrangess:
            return asmjit::x86::Inst::kIdVrangess;
        case Mnemonic::Vrcp14pd:
            return asmjit::x86::Inst::kIdVrcp14pd;
        case Mnemonic::Vrcp14ps:
            return asmjit::x86::Inst::kIdVrcp14ps;
        case Mnemonic::Vrcp14sd:
            return asmjit::x86::Inst::kIdVrcp14sd;
        case Mnemonic::Vrcp14ss:
            return asmjit::x86::Inst::kIdVrcp14ss;
        case Mnemonic::Vrcp28pd:
            return asmjit::x86::Inst::kIdVrcp28pd;
        case Mnemonic::Vrcp28ps:
            return asmjit::x86::Inst::kIdVrcp28ps;
        case Mnemonic::Vrcp28sd:
            return asmjit::x86::Inst::kIdVrcp28sd;
        case Mnemonic::Vrcp28ss:
            return asmjit::x86::Inst::kIdVrcp28ss;
        case Mnemonic::Vrcpps:
            return asmjit::x86::Inst::kIdVrcpps;
        case Mnemonic::Vrcpss:
            return asmjit::x86::Inst::kIdVrcpss;
        case Mnemonic::Vreducepd:
            return asmjit::x86::Inst::kIdVreducepd;
        case Mnemonic::Vreduceps:
            return asmjit::x86::Inst::kIdVreduceps;
        case Mnemonic::Vreducesd:
            return asmjit::x86::Inst::kIdVreducesd;
        case Mnemonic::Vreducess:
            return asmjit::x86::Inst::kIdVreducess;
        case Mnemonic::Vrndscalepd:
            return asmjit::x86::Inst::kIdVrndscalepd;
        case Mnemonic::Vrndscaleps:
            return asmjit::x86::Inst::kIdVrndscaleps;
        case Mnemonic::Vrndscalesd:
            return asmjit::x86::Inst::kIdVrndscalesd;
        case Mnemonic::Vrndscaless:
            return asmjit::x86::Inst::kIdVrndscaless;
        case Mnemonic::Vroundpd:
            return asmjit::x86::Inst::kIdVroundpd;
        case Mnemonic::Vroundps:
            return asmjit::x86::Inst::kIdVroundps;
        case Mnemonic::Vroundsd:
            return asmjit::x86::Inst::kIdVroundsd;
        case Mnemonic::Vroundss:
            return asmjit::x86::Inst::kIdVroundss;
        case Mnemonic::Vrsqrt14pd:
            return asmjit::x86::Inst::kIdVrsqrt14pd;
        case Mnemonic::Vrsqrt14ps:
            return asmjit::x86::Inst::kIdVrsqrt14ps;
        case Mnemonic::Vrsqrt14sd:
            return asmjit::x86::Inst::kIdVrsqrt14sd;
        case Mnemonic::Vrsqrt14ss:
            return asmjit::x86::Inst::kIdVrsqrt14ss;
        case Mnemonic::Vrsqrt28pd:
            return asmjit::x86::Inst::kIdVrsqrt28pd;
        case Mnemonic::Vrsqrt28ps:
            return asmjit::x86::Inst::kIdVrsqrt28ps;
        case Mnemonic::Vrsqrt28sd:
            return asmjit::x86::Inst::kIdVrsqrt28sd;
        case Mnemonic::Vrsqrt28ss:
            return asmjit::x86::Inst::kIdVrsqrt28ss;
        case Mnemonic::Vrsqrtps:
            return asmjit::x86::Inst::kIdVrsqrtps;
        case Mnemonic::Vrsqrtss:
            return asmjit::x86::Inst::kIdVrsqrtss;
        case Mnemonic::Vscalefpd:
            return asmjit::x86::Inst::kIdVscalefpd;
        case Mnemonic::Vscalefps:
            return asmjit::x86::Inst::kIdVscalefps;
        case Mnemonic::Vscalefsd:
            return asmjit::x86::Inst::kIdVscalefsd;
        case Mnemonic::Vscalefss:
            return asmjit::x86::Inst::kIdVscalefss;
        case Mnemonic::Vscatterdpd:
            return asmjit::x86::Inst::kIdVscatterdpd;
        case Mnemonic::Vscatterdps:
            return asmjit::x86::Inst::kIdVscatterdps;
        case Mnemonic::Vscatterpf0dpd:
            return asmjit::x86::Inst::kIdVscatterpf0dpd;
        case Mnemonic::Vscatterpf0dps:
            return asmjit::x86::Inst::kIdVscatterpf0dps;
        case Mnemonic::Vscatterpf0qpd:
            return asmjit::x86::Inst::kIdVscatterpf0qpd;
        case Mnemonic::Vscatterpf0qps:
            return asmjit::x86::Inst::kIdVscatterpf0qps;
        case Mnemonic::Vscatterpf1dpd:
            return asmjit::x86::Inst::kIdVscatterpf1dpd;
        case Mnemonic::Vscatterpf1dps:
            return asmjit::x86::Inst::kIdVscatterpf1dps;
        case Mnemonic::Vscatterpf1qpd:
            return asmjit::x86::Inst::kIdVscatterpf1qpd;
        case Mnemonic::Vscatterpf1qps:
            return asmjit::x86::Inst::kIdVscatterpf1qps;
        case Mnemonic::Vscatterqpd:
            return asmjit::x86::Inst::kIdVscatterqpd;
        case Mnemonic::Vscatterqps:
            return asmjit::x86::Inst::kIdVscatterqps;
        case Mnemonic::Vshuff32x4:
            return asmjit::x86::Inst::kIdVshuff32x4;
        case Mnemonic::Vshuff64x2:
            return asmjit::x86::Inst::kIdVshuff64x2;
        case Mnemonic::Vshufi32x4:
            return asmjit::x86::Inst::kIdVshufi32x4;
        case Mnemonic::Vshufi64x2:
            return asmjit::x86::Inst::kIdVshufi64x2;
        case Mnemonic::Vshufpd:
            return asmjit::x86::Inst::kIdVshufpd;
        case Mnemonic::Vshufps:
            return asmjit::x86::Inst::kIdVshufps;
        case Mnemonic::Vsqrtpd:
            return asmjit::x86::Inst::kIdVsqrtpd;
        case Mnemonic::Vsqrtps:
            return asmjit::x86::Inst::kIdVsqrtps;
        case Mnemonic::Vsqrtsd:
            return asmjit::x86::Inst::kIdVsqrtsd;
        case Mnemonic::Vsqrtss:
            return asmjit::x86::Inst::kIdVsqrtss;
        case Mnemonic::Vstmxcsr:
            return asmjit::x86::Inst::kIdVstmxcsr;
        case Mnemonic::Vsubpd:
            return asmjit::x86::Inst::kIdVsubpd;
        case Mnemonic::Vsubps:
            return asmjit::x86::Inst::kIdVsubps;
        case Mnemonic::Vsubsd:
            return asmjit::x86::Inst::kIdVsubsd;
        case Mnemonic::Vsubss:
            return asmjit::x86::Inst::kIdVsubss;
        case Mnemonic::Vtestpd:
            return asmjit::x86::Inst::kIdVtestpd;
        case Mnemonic::Vtestps:
            return asmjit::x86::Inst::kIdVtestps;
        case Mnemonic::Vucomisd:
            return asmjit::x86::Inst::kIdVucomisd;
        case Mnemonic::Vucomiss:
            return asmjit::x86::Inst::kIdVucomiss;
        case Mnemonic::Vunpckhpd:
            return asmjit::x86::Inst::kIdVunpckhpd;
        case Mnemonic::Vunpckhps:
            return asmjit::x86::Inst::kIdVunpckhps;
        case Mnemonic::Vunpcklpd:
            return asmjit::x86::Inst::kIdVunpcklpd;
        case Mnemonic::Vunpcklps:
            return asmjit::x86::Inst::kIdVunpcklps;
        case Mnemonic::Vxorpd:
            return asmjit::x86::Inst::kIdVxorpd;
        case Mnemonic::Vxorps:
            return asmjit::x86::Inst::kIdVxorps;
        case Mnemonic::Vzeroall:
            return asmjit::x86::Inst::kIdVzeroall;
        case Mnemonic::Vzeroupper:
            return asmjit::x86::Inst::kIdVzeroupper;
        case Mnemonic::Wbinvd:
            return asmjit::x86::Inst::kIdWbinvd;
        case Mnemonic::Wrfsbase:
            return asmjit::x86::Inst::kIdWrfsbase;
        case Mnemonic::Wrgsbase:
            return asmjit::x86::Inst::kIdWrgsbase;
        case Mnemonic::Wrmsr:
            return asmjit::x86::Inst::kIdWrmsr;
        case Mnemonic::Xabort:
            return asmjit::x86::Inst::kIdXabort;
        case Mnemonic::Xadd:
            return asmjit::x86::Inst::kIdXadd;
        case Mnemonic::Xbegin:
            return asmjit::x86::Inst::kIdXbegin;
        case Mnemonic::Xchg:
            return asmjit::x86::Inst::kIdXchg;
        case Mnemonic::Xend:
            return asmjit::x86::Inst::kIdXend;
        case Mnemonic::Xgetbv:
            return asmjit::x86::Inst::kIdXgetbv;
        case Mnemonic::Xlat:
            return asmjit::x86::Inst::kIdXlatb;
        case Mnemonic::Xor:
            return asmjit::x86::Inst::kIdXor;
        case Mnemonic::Xorpd:
            return asmjit::x86::Inst::kIdXorpd;
        case Mnemonic::Xorps:
            return asmjit::x86::Inst::kIdXorps;
        case Mnemonic::Xrstor:
            return asmjit::x86::Inst::kIdXrstor;
        case Mnemonic::Xrstor64:
            return asmjit::x86::Inst::kIdXrstor64;
        case Mnemonic::Xrstors:
            return asmjit::x86::Inst::kIdXrstors;
        case Mnemonic::Xrstors64:
            return asmjit::x86::Inst::kIdXrstors64;
        case Mnemonic::Xsave:
            return asmjit::x86::Inst::kIdXsave;
        case Mnemonic::Xsave64:
            return asmjit::x86::Inst::kIdXsave64;
        case Mnemonic::Xsavec:
            return asmjit::x86::Inst::kIdXsavec;
        case Mnemonic::Xsavec64:
            return asmjit::x86::Inst::kIdXsavec64;
        case Mnemonic::Xsaveopt:
            return asmjit::x86::Inst::kIdXsaveopt;
        case Mnemonic::Xsaveopt64:
            return asmjit::x86::Inst::kIdXsaveopt64;
        case Mnemonic::Xsaves:
            return asmjit::x86::Inst::kIdXsaves;
        case Mnemonic::Xsaves64:
            return asmjit::x86::Inst::kIdXsaves64;
        case Mnemonic::Xsetbv:
            return asmjit::x86::Inst::kIdXsetbv;
        case Mnemonic::Xtest:
            return asmjit::x86::Inst::kIdXtest;
        case Mnemonic::Jrcxz:
            return asmjit::x86::Inst::kIdJecxz;
        case Mnemonic::Jcxz:
            return asmjit::x86::Inst::kIdJecxz;
        case Mnemonic::Stgi:
            return asmjit::x86::Inst::kIdStgi;
        default:
            __debugbreak();
        }
        return 0;
    }

    inline asmjit::x86::Reg convertAsmJitRegister(const Registers::Id reg)
    {
        switch (reg)
        {
        case Registers::Id::None:
            return {};
            // Segment registers
        case Registers::Id::Es:
            return asmjit::x86::es;
        case Registers::Id::Cs:
            return asmjit::x86::cs;
        case Registers::Id::Ss:
            return asmjit::x86::ss;
        case Registers::Id::Ds:
            return asmjit::x86::ds;
        case Registers::Id::Fs:
            return asmjit::x86::fs;
        case Registers::Id::Gs:
            return asmjit::x86::gs;
            // General purpose registers  8-bit
        case Registers::Id::Al:
            return asmjit::x86::al;
        case Registers::Id::Cl:
            return asmjit::x86::cl;
        case Registers::Id::Dl:
            return asmjit::x86::dl;
        case Registers::Id::Bl:
            return asmjit::x86::bl;
        case Registers::Id::Ah:
            return asmjit::x86::ah;
        case Registers::Id::Ch:
            return asmjit::x86::ch;
        case Registers::Id::Dh:
            return asmjit::x86::dh;
        case Registers::Id::Bh:
            return asmjit::x86::bh;
        case Registers::Id::Spl:
            return asmjit::x86::spl;
        case Registers::Id::Bpl:
            return asmjit::x86::bpl;
        case Registers::Id::Sil:
            return asmjit::x86::sil;
        case Registers::Id::Dil:
            return asmjit::x86::dil;
        case Registers::Id::R8b:
            return asmjit::x86::r8b;
        case Registers::Id::R9b:
            return asmjit::x86::r9b;
        case Registers::Id::R10b:
            return asmjit::x86::r10b;
        case Registers::Id::R11b:
            return asmjit::x86::r11b;
        case Registers::Id::R12b:
            return asmjit::x86::r12b;
        case Registers::Id::R13b:
            return asmjit::x86::r13b;
        case Registers::Id::R14b:
            return asmjit::x86::r14b;
        case Registers::Id::R15b:
            return asmjit::x86::r15b;
            // General purpose registers 16-bit
        case Registers::Id::Ax:
            return asmjit::x86::ax;
        case Registers::Id::Cx:
            return asmjit::x86::cx;
        case Registers::Id::Dx:
            return asmjit::x86::dx;
        case Registers::Id::Bx:
            return asmjit::x86::bx;
        case Registers::Id::Sp:
            return asmjit::x86::sp;
        case Registers::Id::Bp:
            return asmjit::x86::bp;
        case Registers::Id::Si:
            return asmjit::x86::si;
        case Registers::Id::Di:
            return asmjit::x86::di;
        case Registers::Id::R8w:
            return asmjit::x86::r8w;
        case Registers::Id::R9w:
            return asmjit::x86::r9w;
        case Registers::Id::R10w:
            return asmjit::x86::r10w;
        case Registers::Id::R11w:
            return asmjit::x86::r11w;
        case Registers::Id::R12w:
            return asmjit::x86::r12w;
        case Registers::Id::R13w:
            return asmjit::x86::r13w;
        case Registers::Id::R14w:
            return asmjit::x86::r14w;
        case Registers::Id::R15w:
            return asmjit::x86::r15w;
            // General purpose registers 32-bit
        case Registers::Id::Eax:
            return asmjit::x86::eax;
        case Registers::Id::Ecx:
            return asmjit::x86::ecx;
        case Registers::Id::Edx:
            return asmjit::x86::edx;
        case Registers::Id::Ebx:
            return asmjit::x86::ebx;
        case Registers::Id::Esp:
            return asmjit::x86::esp;
        case Registers::Id::Ebp:
            return asmjit::x86::ebp;
        case Registers::Id::Esi:
            return asmjit::x86::esi;
        case Registers::Id::Edi:
            return asmjit::x86::edi;
        case Registers::Id::R8d:
            return asmjit::x86::r8d;
        case Registers::Id::R9d:
            return asmjit::x86::r9d;
        case Registers::Id::R10d:
            return asmjit::x86::r10d;
        case Registers::Id::R11d:
            return asmjit::x86::r11d;
        case Registers::Id::R12d:
            return asmjit::x86::r12d;
        case Registers::Id::R13d:
            return asmjit::x86::r13d;
        case Registers::Id::R14d:
            return asmjit::x86::r14d;
        case Registers::Id::R15d:
            return asmjit::x86::r15d;
            // General purpose registers 64-bit
        case Registers::Id::Rax:
            return asmjit::x86::rax;
        case Registers::Id::Rcx:
            return asmjit::x86::rcx;
        case Registers::Id::Rdx:
            return asmjit::x86::rdx;
        case Registers::Id::Rbx:
            return asmjit::x86::rbx;
        case Registers::Id::Rsp:
            return asmjit::x86::rsp;
        case Registers::Id::Rbp:
            return asmjit::x86::rbp;
        case Registers::Id::Rsi:
            return asmjit::x86::rsi;
        case Registers::Id::Rdi:
            return asmjit::x86::rdi;
        case Registers::Id::R8:
            return asmjit::x86::r8;
        case Registers::Id::R9:
            return asmjit::x86::r9;
        case Registers::Id::R10:
            return asmjit::x86::r10;
        case Registers::Id::R11:
            return asmjit::x86::r11;
        case Registers::Id::R12:
            return asmjit::x86::r12;
        case Registers::Id::R13:
            return asmjit::x86::r13;
        case Registers::Id::R14:
            return asmjit::x86::r14;
        case Registers::Id::R15:
            return asmjit::x86::r15;
            // Floating point legacy registers
        case Registers::Id::St0:
            return asmjit::x86::st0;
        case Registers::Id::St1:
            return asmjit::x86::st1;
        case Registers::Id::St2:
            return asmjit::x86::st2;
        case Registers::Id::St3:
            return asmjit::x86::st3;
        case Registers::Id::St4:
            return asmjit::x86::st4;
        case Registers::Id::St5:
            return asmjit::x86::st5;
        case Registers::Id::St6:
            return asmjit::x86::st6;
        case Registers::Id::St7:
            return asmjit::x86::st7;
            // Floating point multimedia registers
        case Registers::Id::Mm0:
            return asmjit::x86::mm0;
        case Registers::Id::Mm1:
            return asmjit::x86::mm1;
        case Registers::Id::Mm2:
            return asmjit::x86::mm2;
        case Registers::Id::Mm3:
            return asmjit::x86::mm3;
        case Registers::Id::Mm4:
            return asmjit::x86::mm4;
        case Registers::Id::Mm5:
            return asmjit::x86::mm5;
        case Registers::Id::Mm6:
            return asmjit::x86::mm6;
        case Registers::Id::Mm7:
            return asmjit::x86::mm7;
            // Floating point vector registers 128-bit
        case Registers::Id::Xmm0:
            return asmjit::x86::xmm0;
        case Registers::Id::Xmm1:
            return asmjit::x86::xmm1;
        case Registers::Id::Xmm2:
            return asmjit::x86::xmm2;
        case Registers::Id::Xmm3:
            return asmjit::x86::xmm3;
        case Registers::Id::Xmm4:
            return asmjit::x86::xmm4;
        case Registers::Id::Xmm5:
            return asmjit::x86::xmm5;
        case Registers::Id::Xmm6:
            return asmjit::x86::xmm6;
        case Registers::Id::Xmm7:
            return asmjit::x86::xmm7;
        case Registers::Id::Xmm8:
            return asmjit::x86::xmm8;
        case Registers::Id::Xmm9:
            return asmjit::x86::xmm9;
        case Registers::Id::Xmm10:
            return asmjit::x86::xmm10;
        case Registers::Id::Xmm11:
            return asmjit::x86::xmm11;
        case Registers::Id::Xmm12:
            return asmjit::x86::xmm12;
        case Registers::Id::Xmm13:
            return asmjit::x86::xmm13;
        case Registers::Id::Xmm14:
            return asmjit::x86::xmm14;
        case Registers::Id::Xmm15:
            return asmjit::x86::xmm15;
        case Registers::Id::Xmm16:
            return asmjit::x86::xmm16;
        case Registers::Id::Xmm17:
            return asmjit::x86::xmm17;
        case Registers::Id::Xmm18:
            return asmjit::x86::xmm18;
        case Registers::Id::Xmm19:
            return asmjit::x86::xmm19;
        case Registers::Id::Xmm20:
            return asmjit::x86::xmm20;
        case Registers::Id::Xmm21:
            return asmjit::x86::xmm21;
        case Registers::Id::Xmm22:
            return asmjit::x86::xmm22;
        case Registers::Id::Xmm23:
            return asmjit::x86::xmm23;
        case Registers::Id::Xmm24:
            return asmjit::x86::xmm24;
        case Registers::Id::Xmm25:
            return asmjit::x86::xmm25;
        case Registers::Id::Xmm26:
            return asmjit::x86::xmm26;
        case Registers::Id::Xmm27:
            return asmjit::x86::xmm27;
        case Registers::Id::Xmm28:
            return asmjit::x86::xmm28;
        case Registers::Id::Xmm29:
            return asmjit::x86::xmm29;
        case Registers::Id::Xmm30:
            return asmjit::x86::xmm30;
        case Registers::Id::Xmm31:
            return asmjit::x86::xmm31;
            // Floating point vector registers 256-bit
        case Registers::Id::Ymm0:
            return asmjit::x86::ymm0;
        case Registers::Id::Ymm1:
            return asmjit::x86::ymm1;
        case Registers::Id::Ymm2:
            return asmjit::x86::ymm2;
        case Registers::Id::Ymm3:
            return asmjit::x86::ymm3;
        case Registers::Id::Ymm4:
            return asmjit::x86::ymm4;
        case Registers::Id::Ymm5:
            return asmjit::x86::ymm5;
        case Registers::Id::Ymm6:
            return asmjit::x86::ymm6;
        case Registers::Id::Ymm7:
            return asmjit::x86::ymm7;
        case Registers::Id::Ymm8:
            return asmjit::x86::ymm8;
        case Registers::Id::Ymm9:
            return asmjit::x86::ymm9;
        case Registers::Id::Ymm10:
            return asmjit::x86::ymm10;
        case Registers::Id::Ymm11:
            return asmjit::x86::ymm11;
        case Registers::Id::Ymm12:
            return asmjit::x86::ymm12;
        case Registers::Id::Ymm13:
            return asmjit::x86::ymm13;
        case Registers::Id::Ymm14:
            return asmjit::x86::ymm14;
        case Registers::Id::Ymm15:
            return asmjit::x86::ymm15;
        case Registers::Id::Ymm16:
            return asmjit::x86::ymm16;
        case Registers::Id::Ymm17:
            return asmjit::x86::ymm17;
        case Registers::Id::Ymm18:
            return asmjit::x86::ymm18;
        case Registers::Id::Ymm19:
            return asmjit::x86::ymm19;
        case Registers::Id::Ymm20:
            return asmjit::x86::ymm20;
        case Registers::Id::Ymm21:
            return asmjit::x86::ymm21;
        case Registers::Id::Ymm22:
            return asmjit::x86::ymm22;
        case Registers::Id::Ymm23:
            return asmjit::x86::ymm23;
        case Registers::Id::Ymm24:
            return asmjit::x86::ymm24;
        case Registers::Id::Ymm25:
            return asmjit::x86::ymm25;
        case Registers::Id::Ymm26:
            return asmjit::x86::ymm26;
        case Registers::Id::Ymm27:
            return asmjit::x86::ymm27;
        case Registers::Id::Ymm28:
            return asmjit::x86::ymm28;
        case Registers::Id::Ymm29:
            return asmjit::x86::ymm29;
        case Registers::Id::Ymm30:
            return asmjit::x86::ymm30;
        case Registers::Id::Ymm31:
            return asmjit::x86::ymm31;
            // Floating point vector registers 512-bit
        case Registers::Id::Zmm0:
            return asmjit::x86::zmm0;
        case Registers::Id::Zmm1:
            return asmjit::x86::zmm1;
        case Registers::Id::Zmm2:
            return asmjit::x86::zmm2;
        case Registers::Id::Zmm3:
            return asmjit::x86::zmm3;
        case Registers::Id::Zmm4:
            return asmjit::x86::zmm4;
        case Registers::Id::Zmm5:
            return asmjit::x86::zmm5;
        case Registers::Id::Zmm6:
            return asmjit::x86::zmm6;
        case Registers::Id::Zmm7:
            return asmjit::x86::zmm7;
        case Registers::Id::Zmm8:
            return asmjit::x86::zmm8;
        case Registers::Id::Zmm9:
            return asmjit::x86::zmm9;
        case Registers::Id::Zmm10:
            return asmjit::x86::zmm10;
        case Registers::Id::Zmm11:
            return asmjit::x86::zmm11;
        case Registers::Id::Zmm12:
            return asmjit::x86::zmm12;
        case Registers::Id::Zmm13:
            return asmjit::x86::zmm13;
        case Registers::Id::Zmm14:
            return asmjit::x86::zmm14;
        case Registers::Id::Zmm15:
            return asmjit::x86::zmm15;
        case Registers::Id::Zmm16:
            return asmjit::x86::zmm16;
        case Registers::Id::Zmm17:
            return asmjit::x86::zmm17;
        case Registers::Id::Zmm18:
            return asmjit::x86::zmm18;
        case Registers::Id::Zmm19:
            return asmjit::x86::zmm19;
        case Registers::Id::Zmm20:
            return asmjit::x86::zmm20;
        case Registers::Id::Zmm21:
            return asmjit::x86::zmm21;
        case Registers::Id::Zmm22:
            return asmjit::x86::zmm22;
        case Registers::Id::Zmm23:
            return asmjit::x86::zmm23;
        case Registers::Id::Zmm24:
            return asmjit::x86::zmm24;
        case Registers::Id::Zmm25:
            return asmjit::x86::zmm25;
        case Registers::Id::Zmm26:
            return asmjit::x86::zmm26;
        case Registers::Id::Zmm27:
            return asmjit::x86::zmm27;
        case Registers::Id::Zmm28:
            return asmjit::x86::zmm28;
        case Registers::Id::Zmm29:
            return asmjit::x86::zmm29;
        case Registers::Id::Zmm30:
            return asmjit::x86::zmm30;
        case Registers::Id::Zmm31:
            return asmjit::x86::zmm31;
        case Registers::Id::Cr0:
            return asmjit::x86::cr0;
        case Registers::Id::Cr1:
            return asmjit::x86::cr1;
        case Registers::Id::Cr2:
            return asmjit::x86::cr2;
        case Registers::Id::Cr3:
            return asmjit::x86::cr3;
        case Registers::Id::Cr4:
            return asmjit::x86::cr4;
        case Registers::Id::Cr5:
            return asmjit::x86::cr5;
        case Registers::Id::Cr6:
            return asmjit::x86::cr6;
        case Registers::Id::Cr7:
            return asmjit::x86::cr7;
        case Registers::Id::Cr8:
            return asmjit::x86::cr8;
        case Registers::Id::Cr9:
            return asmjit::x86::cr9;
        case Registers::Id::Cr10:
            return asmjit::x86::cr10;
        case Registers::Id::Cr11:
            return asmjit::x86::cr11;
        case Registers::Id::Cr12:
            return asmjit::x86::cr12;
        case Registers::Id::Cr13:
            return asmjit::x86::cr13;
        case Registers::Id::Cr14:
            return asmjit::x86::cr14;
        case Registers::Id::Cr15:
            return asmjit::x86::cr15;
        case Registers::Id::Idtr:
            // Not mappable.
            return {};
            // NOTE: AsmJit might handle this differently.
        case Registers::Id::Ip:
        case Registers::Id::Eip:
        case Registers::Id::Rip:
            return asmjit::x86::rip;
        case Registers::Id::Tr:
        case Registers::Id::Tr0:
        case Registers::Id::Tr1:
        case Registers::Id::Tr2:
        case Registers::Id::Tr3:
        case Registers::Id::Tr4:
        case Registers::Id::Tr5:
        case Registers::Id::Tr6:
        case Registers::Id::Tr7:
            return {};
        }

        // Bug?
        __debugbreak();
        return asmjit::x86::Reg();
    }

} // namespace Dotx64Dbg
