#pragma once

#include "Register.hpp"
#include "Mnemonic.hpp"

#include <zydis/zydis.h>

namespace Dotx64Dbg
{

    constexpr Registers::Id convertZydisRegister(ZydisRegister reg)
    {
        switch (reg)
        {
        case ZYDIS_REGISTER_NONE:
            return Registers::Id::None;
        case ZYDIS_REGISTER_AL:
            return Registers::Id::Al;
        case ZYDIS_REGISTER_CL:
            return Registers::Id::Cl;
        case ZYDIS_REGISTER_DL:
            return Registers::Id::Dl;
        case ZYDIS_REGISTER_BL:
            return Registers::Id::Bl;
        case ZYDIS_REGISTER_AH:
            return Registers::Id::Ah;
        case ZYDIS_REGISTER_CH:
            return Registers::Id::Ch;
        case ZYDIS_REGISTER_DH:
            return Registers::Id::Dh;
        case ZYDIS_REGISTER_BH:
            return Registers::Id::Bh;
        case ZYDIS_REGISTER_SPL:
            return Registers::Id::Spl;
        case ZYDIS_REGISTER_BPL:
            return Registers::Id::Bpl;
        case ZYDIS_REGISTER_SIL:
            return Registers::Id::Sil;
        case ZYDIS_REGISTER_DIL:
            return Registers::Id::Dil;
        case ZYDIS_REGISTER_R8B:
            return Registers::Id::R8b;
        case ZYDIS_REGISTER_R9B:
            return Registers::Id::R9b;
        case ZYDIS_REGISTER_R10B:
            return Registers::Id::R10b;
        case ZYDIS_REGISTER_R11B:
            return Registers::Id::R11b;
        case ZYDIS_REGISTER_R12B:
            return Registers::Id::R12b;
        case ZYDIS_REGISTER_R13B:
            return Registers::Id::R13b;
        case ZYDIS_REGISTER_R14B:
            return Registers::Id::R14b;
        case ZYDIS_REGISTER_R15B:
            return Registers::Id::R15b;
        case ZYDIS_REGISTER_AX:
            return Registers::Id::Ax;
        case ZYDIS_REGISTER_CX:
            return Registers::Id::Cx;
        case ZYDIS_REGISTER_DX:
            return Registers::Id::Dx;
        case ZYDIS_REGISTER_BX:
            return Registers::Id::Bx;
        case ZYDIS_REGISTER_SP:
            return Registers::Id::Sp;
        case ZYDIS_REGISTER_BP:
            return Registers::Id::Bp;
        case ZYDIS_REGISTER_SI:
            return Registers::Id::Si;
        case ZYDIS_REGISTER_DI:
            return Registers::Id::Di;
        case ZYDIS_REGISTER_R8W:
            return Registers::Id::R8w;
        case ZYDIS_REGISTER_R9W:
            return Registers::Id::R9w;
        case ZYDIS_REGISTER_R10W:
            return Registers::Id::R10w;
        case ZYDIS_REGISTER_R11W:
            return Registers::Id::R11w;
        case ZYDIS_REGISTER_R12W:
            return Registers::Id::R12w;
        case ZYDIS_REGISTER_R13W:
            return Registers::Id::R13w;
        case ZYDIS_REGISTER_R14W:
            return Registers::Id::R14w;
        case ZYDIS_REGISTER_R15W:
            return Registers::Id::R15w;
        case ZYDIS_REGISTER_EAX:
            return Registers::Id::Eax;
        case ZYDIS_REGISTER_ECX:
            return Registers::Id::Ecx;
        case ZYDIS_REGISTER_EDX:
            return Registers::Id::Edx;
        case ZYDIS_REGISTER_EBX:
            return Registers::Id::Ebx;
        case ZYDIS_REGISTER_ESP:
            return Registers::Id::Esp;
        case ZYDIS_REGISTER_EBP:
            return Registers::Id::Ebp;
        case ZYDIS_REGISTER_ESI:
            return Registers::Id::Esi;
        case ZYDIS_REGISTER_EDI:
            return Registers::Id::Edi;
        case ZYDIS_REGISTER_R8D:
            return Registers::Id::R8d;
        case ZYDIS_REGISTER_R9D:
            return Registers::Id::R9d;
        case ZYDIS_REGISTER_R10D:
            return Registers::Id::R10d;
        case ZYDIS_REGISTER_R11D:
            return Registers::Id::R11d;
        case ZYDIS_REGISTER_R12D:
            return Registers::Id::R12d;
        case ZYDIS_REGISTER_R13D:
            return Registers::Id::R13d;
        case ZYDIS_REGISTER_R14D:
            return Registers::Id::R14d;
        case ZYDIS_REGISTER_R15D:
            return Registers::Id::R15d;
        case ZYDIS_REGISTER_RAX:
            return Registers::Id::Rax;
        case ZYDIS_REGISTER_RCX:
            return Registers::Id::Rcx;
        case ZYDIS_REGISTER_RDX:
            return Registers::Id::Rdx;
        case ZYDIS_REGISTER_RBX:
            return Registers::Id::Rbx;
        case ZYDIS_REGISTER_RSP:
            return Registers::Id::Rsp;
        case ZYDIS_REGISTER_RBP:
            return Registers::Id::Rbp;
        case ZYDIS_REGISTER_RSI:
            return Registers::Id::Rsi;
        case ZYDIS_REGISTER_RDI:
            return Registers::Id::Rdi;
        case ZYDIS_REGISTER_R8:
            return Registers::Id::R8;
        case ZYDIS_REGISTER_R9:
            return Registers::Id::R9;
        case ZYDIS_REGISTER_R10:
            return Registers::Id::R10;
        case ZYDIS_REGISTER_R11:
            return Registers::Id::R11;
        case ZYDIS_REGISTER_R12:
            return Registers::Id::R12;
        case ZYDIS_REGISTER_R13:
            return Registers::Id::R13;
        case ZYDIS_REGISTER_R14:
            return Registers::Id::R14;
        case ZYDIS_REGISTER_R15:
            return Registers::Id::R15;
        case ZYDIS_REGISTER_ST0:
            return Registers::Id::St0;
        case ZYDIS_REGISTER_ST1:
            return Registers::Id::St1;
        case ZYDIS_REGISTER_ST2:
            return Registers::Id::St2;
        case ZYDIS_REGISTER_ST3:
            return Registers::Id::St3;
        case ZYDIS_REGISTER_ST4:
            return Registers::Id::St4;
        case ZYDIS_REGISTER_ST5:
            return Registers::Id::St5;
        case ZYDIS_REGISTER_ST6:
            return Registers::Id::St6;
        case ZYDIS_REGISTER_ST7:
            return Registers::Id::St7;
        case ZYDIS_REGISTER_X87CONTROL:
            return Registers::Id::X87Control;
        case ZYDIS_REGISTER_X87STATUS:
            return Registers::Id::X87Status;
        case ZYDIS_REGISTER_X87TAG:
            return Registers::Id::X87Tag;
        case ZYDIS_REGISTER_MM0:
            return Registers::Id::Mm0;
        case ZYDIS_REGISTER_MM1:
            return Registers::Id::Mm1;
        case ZYDIS_REGISTER_MM2:
            return Registers::Id::Mm2;
        case ZYDIS_REGISTER_MM3:
            return Registers::Id::Mm3;
        case ZYDIS_REGISTER_MM4:
            return Registers::Id::Mm4;
        case ZYDIS_REGISTER_MM5:
            return Registers::Id::Mm5;
        case ZYDIS_REGISTER_MM6:
            return Registers::Id::Mm6;
        case ZYDIS_REGISTER_MM7:
            return Registers::Id::Mm7;
        case ZYDIS_REGISTER_XMM0:
            return Registers::Id::Xmm0;
        case ZYDIS_REGISTER_XMM1:
            return Registers::Id::Xmm1;
        case ZYDIS_REGISTER_XMM2:
            return Registers::Id::Xmm2;
        case ZYDIS_REGISTER_XMM3:
            return Registers::Id::Xmm3;
        case ZYDIS_REGISTER_XMM4:
            return Registers::Id::Xmm4;
        case ZYDIS_REGISTER_XMM5:
            return Registers::Id::Xmm5;
        case ZYDIS_REGISTER_XMM6:
            return Registers::Id::Xmm6;
        case ZYDIS_REGISTER_XMM7:
            return Registers::Id::Xmm7;
        case ZYDIS_REGISTER_XMM8:
            return Registers::Id::Xmm8;
        case ZYDIS_REGISTER_XMM9:
            return Registers::Id::Xmm9;
        case ZYDIS_REGISTER_XMM10:
            return Registers::Id::Xmm10;
        case ZYDIS_REGISTER_XMM11:
            return Registers::Id::Xmm11;
        case ZYDIS_REGISTER_XMM12:
            return Registers::Id::Xmm12;
        case ZYDIS_REGISTER_XMM13:
            return Registers::Id::Xmm13;
        case ZYDIS_REGISTER_XMM14:
            return Registers::Id::Xmm14;
        case ZYDIS_REGISTER_XMM15:
            return Registers::Id::Xmm15;
        case ZYDIS_REGISTER_XMM16:
            return Registers::Id::Xmm16;
        case ZYDIS_REGISTER_XMM17:
            return Registers::Id::Xmm17;
        case ZYDIS_REGISTER_XMM18:
            return Registers::Id::Xmm18;
        case ZYDIS_REGISTER_XMM19:
            return Registers::Id::Xmm19;
        case ZYDIS_REGISTER_XMM20:
            return Registers::Id::Xmm20;
        case ZYDIS_REGISTER_XMM21:
            return Registers::Id::Xmm21;
        case ZYDIS_REGISTER_XMM22:
            return Registers::Id::Xmm22;
        case ZYDIS_REGISTER_XMM23:
            return Registers::Id::Xmm23;
        case ZYDIS_REGISTER_XMM24:
            return Registers::Id::Xmm24;
        case ZYDIS_REGISTER_XMM25:
            return Registers::Id::Xmm25;
        case ZYDIS_REGISTER_XMM26:
            return Registers::Id::Xmm26;
        case ZYDIS_REGISTER_XMM27:
            return Registers::Id::Xmm27;
        case ZYDIS_REGISTER_XMM28:
            return Registers::Id::Xmm28;
        case ZYDIS_REGISTER_XMM29:
            return Registers::Id::Xmm29;
        case ZYDIS_REGISTER_XMM30:
            return Registers::Id::Xmm30;
        case ZYDIS_REGISTER_XMM31:
            return Registers::Id::Xmm31;
        case ZYDIS_REGISTER_YMM0:
            return Registers::Id::Ymm0;
        case ZYDIS_REGISTER_YMM1:
            return Registers::Id::Ymm1;
        case ZYDIS_REGISTER_YMM2:
            return Registers::Id::Ymm2;
        case ZYDIS_REGISTER_YMM3:
            return Registers::Id::Ymm3;
        case ZYDIS_REGISTER_YMM4:
            return Registers::Id::Ymm4;
        case ZYDIS_REGISTER_YMM5:
            return Registers::Id::Ymm5;
        case ZYDIS_REGISTER_YMM6:
            return Registers::Id::Ymm6;
        case ZYDIS_REGISTER_YMM7:
            return Registers::Id::Ymm7;
        case ZYDIS_REGISTER_YMM8:
            return Registers::Id::Ymm8;
        case ZYDIS_REGISTER_YMM9:
            return Registers::Id::Ymm9;
        case ZYDIS_REGISTER_YMM10:
            return Registers::Id::Ymm10;
        case ZYDIS_REGISTER_YMM11:
            return Registers::Id::Ymm11;
        case ZYDIS_REGISTER_YMM12:
            return Registers::Id::Ymm12;
        case ZYDIS_REGISTER_YMM13:
            return Registers::Id::Ymm13;
        case ZYDIS_REGISTER_YMM14:
            return Registers::Id::Ymm14;
        case ZYDIS_REGISTER_YMM15:
            return Registers::Id::Ymm15;
        case ZYDIS_REGISTER_YMM16:
            return Registers::Id::Ymm16;
        case ZYDIS_REGISTER_YMM17:
            return Registers::Id::Ymm17;
        case ZYDIS_REGISTER_YMM18:
            return Registers::Id::Ymm18;
        case ZYDIS_REGISTER_YMM19:
            return Registers::Id::Ymm19;
        case ZYDIS_REGISTER_YMM20:
            return Registers::Id::Ymm20;
        case ZYDIS_REGISTER_YMM21:
            return Registers::Id::Ymm21;
        case ZYDIS_REGISTER_YMM22:
            return Registers::Id::Ymm22;
        case ZYDIS_REGISTER_YMM23:
            return Registers::Id::Ymm23;
        case ZYDIS_REGISTER_YMM24:
            return Registers::Id::Ymm24;
        case ZYDIS_REGISTER_YMM25:
            return Registers::Id::Ymm25;
        case ZYDIS_REGISTER_YMM26:
            return Registers::Id::Ymm26;
        case ZYDIS_REGISTER_YMM27:
            return Registers::Id::Ymm27;
        case ZYDIS_REGISTER_YMM28:
            return Registers::Id::Ymm28;
        case ZYDIS_REGISTER_YMM29:
            return Registers::Id::Ymm29;
        case ZYDIS_REGISTER_YMM30:
            return Registers::Id::Ymm30;
        case ZYDIS_REGISTER_YMM31:
            return Registers::Id::Ymm31;
        case ZYDIS_REGISTER_ZMM0:
            return Registers::Id::Zmm0;
        case ZYDIS_REGISTER_ZMM1:
            return Registers::Id::Zmm1;
        case ZYDIS_REGISTER_ZMM2:
            return Registers::Id::Zmm2;
        case ZYDIS_REGISTER_ZMM3:
            return Registers::Id::Zmm3;
        case ZYDIS_REGISTER_ZMM4:
            return Registers::Id::Zmm4;
        case ZYDIS_REGISTER_ZMM5:
            return Registers::Id::Zmm5;
        case ZYDIS_REGISTER_ZMM6:
            return Registers::Id::Zmm6;
        case ZYDIS_REGISTER_ZMM7:
            return Registers::Id::Zmm7;
        case ZYDIS_REGISTER_ZMM8:
            return Registers::Id::Zmm8;
        case ZYDIS_REGISTER_ZMM9:
            return Registers::Id::Zmm9;
        case ZYDIS_REGISTER_ZMM10:
            return Registers::Id::Zmm10;
        case ZYDIS_REGISTER_ZMM11:
            return Registers::Id::Zmm11;
        case ZYDIS_REGISTER_ZMM12:
            return Registers::Id::Zmm12;
        case ZYDIS_REGISTER_ZMM13:
            return Registers::Id::Zmm13;
        case ZYDIS_REGISTER_ZMM14:
            return Registers::Id::Zmm14;
        case ZYDIS_REGISTER_ZMM15:
            return Registers::Id::Zmm15;
        case ZYDIS_REGISTER_ZMM16:
            return Registers::Id::Zmm16;
        case ZYDIS_REGISTER_ZMM17:
            return Registers::Id::Zmm17;
        case ZYDIS_REGISTER_ZMM18:
            return Registers::Id::Zmm18;
        case ZYDIS_REGISTER_ZMM19:
            return Registers::Id::Zmm19;
        case ZYDIS_REGISTER_ZMM20:
            return Registers::Id::Zmm20;
        case ZYDIS_REGISTER_ZMM21:
            return Registers::Id::Zmm21;
        case ZYDIS_REGISTER_ZMM22:
            return Registers::Id::Zmm22;
        case ZYDIS_REGISTER_ZMM23:
            return Registers::Id::Zmm23;
        case ZYDIS_REGISTER_ZMM24:
            return Registers::Id::Zmm24;
        case ZYDIS_REGISTER_ZMM25:
            return Registers::Id::Zmm25;
        case ZYDIS_REGISTER_ZMM26:
            return Registers::Id::Zmm26;
        case ZYDIS_REGISTER_ZMM27:
            return Registers::Id::Zmm27;
        case ZYDIS_REGISTER_ZMM28:
            return Registers::Id::Zmm28;
        case ZYDIS_REGISTER_ZMM29:
            return Registers::Id::Zmm29;
        case ZYDIS_REGISTER_ZMM30:
            return Registers::Id::Zmm30;
        case ZYDIS_REGISTER_ZMM31:
            return Registers::Id::Zmm31;
        case ZYDIS_REGISTER_TMM0:
            return Registers::Id::Tmm0;
        case ZYDIS_REGISTER_TMM1:
            return Registers::Id::Tmm1;
        case ZYDIS_REGISTER_TMM2:
            return Registers::Id::Tmm2;
        case ZYDIS_REGISTER_TMM3:
            return Registers::Id::Tmm3;
        case ZYDIS_REGISTER_TMM4:
            return Registers::Id::Tmm4;
        case ZYDIS_REGISTER_TMM5:
            return Registers::Id::Tmm5;
        case ZYDIS_REGISTER_TMM6:
            return Registers::Id::Tmm6;
        case ZYDIS_REGISTER_TMM7:
            return Registers::Id::Tmm7;
        case ZYDIS_REGISTER_FLAGS:
            return Registers::Id::Flags;
        case ZYDIS_REGISTER_EFLAGS:
            return Registers::Id::EFlags;
        case ZYDIS_REGISTER_RFLAGS:
            return Registers::Id::RFlags;
        case ZYDIS_REGISTER_IP:
            return Registers::Id::Ip;
        case ZYDIS_REGISTER_EIP:
            return Registers::Id::Eip;
        case ZYDIS_REGISTER_RIP:
            return Registers::Id::Rip;
        case ZYDIS_REGISTER_ES:
            return Registers::Id::Es;
        case ZYDIS_REGISTER_CS:
            return Registers::Id::Cs;
        case ZYDIS_REGISTER_SS:
            return Registers::Id::Ss;
        case ZYDIS_REGISTER_DS:
            return Registers::Id::Ds;
        case ZYDIS_REGISTER_FS:
            return Registers::Id::Fs;
        case ZYDIS_REGISTER_GS:
            return Registers::Id::Gs;
        case ZYDIS_REGISTER_GDTR:
            return Registers::Id::Gdtr;
        case ZYDIS_REGISTER_LDTR:
            return Registers::Id::Ldtr;
        case ZYDIS_REGISTER_IDTR:
            return Registers::Id::Idtr;
        case ZYDIS_REGISTER_TR:
            return Registers::Id::Tr;
        case ZYDIS_REGISTER_TR0:
            return Registers::Id::Tr0;
        case ZYDIS_REGISTER_TR1:
            return Registers::Id::Tr1;
        case ZYDIS_REGISTER_TR2:
            return Registers::Id::Tr2;
        case ZYDIS_REGISTER_TR3:
            return Registers::Id::Tr3;
        case ZYDIS_REGISTER_TR4:
            return Registers::Id::Tr4;
        case ZYDIS_REGISTER_TR5:
            return Registers::Id::Tr5;
        case ZYDIS_REGISTER_TR6:
            return Registers::Id::Tr6;
        case ZYDIS_REGISTER_TR7:
            return Registers::Id::Tr7;
        case ZYDIS_REGISTER_CR0:
            return Registers::Id::Cr0;
        case ZYDIS_REGISTER_CR1:
            return Registers::Id::Cr1;
        case ZYDIS_REGISTER_CR2:
            return Registers::Id::Cr2;
        case ZYDIS_REGISTER_CR3:
            return Registers::Id::Cr3;
        case ZYDIS_REGISTER_CR4:
            return Registers::Id::Cr4;
        case ZYDIS_REGISTER_CR5:
            return Registers::Id::Cr5;
        case ZYDIS_REGISTER_CR6:
            return Registers::Id::Cr6;
        case ZYDIS_REGISTER_CR7:
            return Registers::Id::Cr7;
        case ZYDIS_REGISTER_CR8:
            return Registers::Id::Cr8;
        case ZYDIS_REGISTER_CR9:
            return Registers::Id::Cr9;
        case ZYDIS_REGISTER_CR10:
            return Registers::Id::Cr10;
        case ZYDIS_REGISTER_CR11:
            return Registers::Id::Cr11;
        case ZYDIS_REGISTER_CR12:
            return Registers::Id::Cr12;
        case ZYDIS_REGISTER_CR13:
            return Registers::Id::Cr13;
        case ZYDIS_REGISTER_CR14:
            return Registers::Id::Cr14;
        case ZYDIS_REGISTER_CR15:
            return Registers::Id::Cr15;
        case ZYDIS_REGISTER_DR0:
            return Registers::Id::Dr0;
        case ZYDIS_REGISTER_DR1:
            return Registers::Id::Dr1;
        case ZYDIS_REGISTER_DR2:
            return Registers::Id::Dr2;
        case ZYDIS_REGISTER_DR3:
            return Registers::Id::Dr3;
        case ZYDIS_REGISTER_DR4:
            return Registers::Id::Dr4;
        case ZYDIS_REGISTER_DR5:
            return Registers::Id::Dr5;
        case ZYDIS_REGISTER_DR6:
            return Registers::Id::Dr6;
        case ZYDIS_REGISTER_DR7:
            return Registers::Id::Dr7;
        case ZYDIS_REGISTER_DR8:
            return Registers::Id::Dr8;
        case ZYDIS_REGISTER_DR9:
            return Registers::Id::Dr9;
        case ZYDIS_REGISTER_DR10:
            return Registers::Id::Dr10;
        case ZYDIS_REGISTER_DR11:
            return Registers::Id::Dr11;
        case ZYDIS_REGISTER_DR12:
            return Registers::Id::Dr12;
        case ZYDIS_REGISTER_DR13:
            return Registers::Id::Dr13;
        case ZYDIS_REGISTER_DR14:
            return Registers::Id::Dr14;
        case ZYDIS_REGISTER_DR15:
            return Registers::Id::Dr15;
        case ZYDIS_REGISTER_K0:
            return Registers::Id::K0;
        case ZYDIS_REGISTER_K1:
            return Registers::Id::K1;
        case ZYDIS_REGISTER_K2:
            return Registers::Id::K2;
        case ZYDIS_REGISTER_K3:
            return Registers::Id::K3;
        case ZYDIS_REGISTER_K4:
            return Registers::Id::K4;
        case ZYDIS_REGISTER_K5:
            return Registers::Id::K5;
        case ZYDIS_REGISTER_K6:
            return Registers::Id::K6;
        case ZYDIS_REGISTER_K7:
            return Registers::Id::K7;
        case ZYDIS_REGISTER_BND0:
            return Registers::Id::Bnd0;
        case ZYDIS_REGISTER_BND1:
            return Registers::Id::Bnd1;
        case ZYDIS_REGISTER_BND2:
            return Registers::Id::Bnd2;
        case ZYDIS_REGISTER_BND3:
            return Registers::Id::Bnd3;
        case ZYDIS_REGISTER_BNDCFG:
            return Registers::Id::BndCfg;
        case ZYDIS_REGISTER_BNDSTATUS:
            return Registers::Id::BndStatus;
        case ZYDIS_REGISTER_MXCSR:
            return Registers::Id::Mxcsr;
        case ZYDIS_REGISTER_PKRU:
            return Registers::Id::Pkru;
        case ZYDIS_REGISTER_XCR0:
            return Registers::Id::Xcr0;
        }
        return Registers::Id::Invalid;
    }

    inline Mnemonic convertZydisMnemonic(ZydisMnemonic mnemonic)
    {
        switch (mnemonic)
        {
        case ZYDIS_MNEMONIC_INVALID:
            return Mnemonic::Invalid;
        case ZYDIS_MNEMONIC_AAA:
            return Mnemonic::Aaa;
        case ZYDIS_MNEMONIC_AAD:
            return Mnemonic::Aad;
        case ZYDIS_MNEMONIC_AAM:
            return Mnemonic::Aam;
        case ZYDIS_MNEMONIC_AAS:
            return Mnemonic::Aas;
        case ZYDIS_MNEMONIC_ADC:
            return Mnemonic::Adc;
        case ZYDIS_MNEMONIC_ADCX:
            return Mnemonic::Adcx;
        case ZYDIS_MNEMONIC_ADD:
            return Mnemonic::Add;
        case ZYDIS_MNEMONIC_ADDPD:
            return Mnemonic::Addpd;
        case ZYDIS_MNEMONIC_ADDPS:
            return Mnemonic::Addps;
        case ZYDIS_MNEMONIC_ADDSD:
            return Mnemonic::Addsd;
        case ZYDIS_MNEMONIC_ADDSS:
            return Mnemonic::Addss;
        case ZYDIS_MNEMONIC_ADDSUBPD:
            return Mnemonic::Addsubpd;
        case ZYDIS_MNEMONIC_ADDSUBPS:
            return Mnemonic::Addsubps;
        case ZYDIS_MNEMONIC_ADOX:
            return Mnemonic::Adox;
        case ZYDIS_MNEMONIC_AESDEC:
            return Mnemonic::Aesdec;
        case ZYDIS_MNEMONIC_AESDEC128KL:
            return Mnemonic::Aesdec128kl;
        case ZYDIS_MNEMONIC_AESDEC256KL:
            return Mnemonic::Aesdec256kl;
        case ZYDIS_MNEMONIC_AESDECLAST:
            return Mnemonic::Aesdeclast;
        case ZYDIS_MNEMONIC_AESDECWIDE128KL:
            return Mnemonic::Aesdecwide128kl;
        case ZYDIS_MNEMONIC_AESDECWIDE256KL:
            return Mnemonic::Aesdecwide256kl;
        case ZYDIS_MNEMONIC_AESENC:
            return Mnemonic::Aesenc;
        case ZYDIS_MNEMONIC_AESENC128KL:
            return Mnemonic::Aesenc128kl;
        case ZYDIS_MNEMONIC_AESENC256KL:
            return Mnemonic::Aesenc256kl;
        case ZYDIS_MNEMONIC_AESENCLAST:
            return Mnemonic::Aesenclast;
        case ZYDIS_MNEMONIC_AESENCWIDE128KL:
            return Mnemonic::Aesencwide128kl;
        case ZYDIS_MNEMONIC_AESENCWIDE256KL:
            return Mnemonic::Aesencwide256kl;
        case ZYDIS_MNEMONIC_AESIMC:
            return Mnemonic::Aesimc;
        case ZYDIS_MNEMONIC_AESKEYGENASSIST:
            return Mnemonic::Aeskeygenassist;
        case ZYDIS_MNEMONIC_AND:
            return Mnemonic::And;
        case ZYDIS_MNEMONIC_ANDN:
            return Mnemonic::Andn;
        case ZYDIS_MNEMONIC_ANDNPD:
            return Mnemonic::Andnpd;
        case ZYDIS_MNEMONIC_ANDNPS:
            return Mnemonic::Andnps;
        case ZYDIS_MNEMONIC_ANDPD:
            return Mnemonic::Andpd;
        case ZYDIS_MNEMONIC_ANDPS:
            return Mnemonic::Andps;
        case ZYDIS_MNEMONIC_ARPL:
            return Mnemonic::Arpl;
        case ZYDIS_MNEMONIC_BEXTR:
            return Mnemonic::Bextr;
        case ZYDIS_MNEMONIC_BLCFILL:
            return Mnemonic::Blcfill;
        case ZYDIS_MNEMONIC_BLCI:
            return Mnemonic::Blci;
        case ZYDIS_MNEMONIC_BLCIC:
            return Mnemonic::Blcic;
        case ZYDIS_MNEMONIC_BLCMSK:
            return Mnemonic::Blcmsk;
        case ZYDIS_MNEMONIC_BLCS:
            return Mnemonic::Blcs;
        case ZYDIS_MNEMONIC_BLENDPD:
            return Mnemonic::Blendpd;
        case ZYDIS_MNEMONIC_BLENDPS:
            return Mnemonic::Blendps;
        case ZYDIS_MNEMONIC_BLENDVPD:
            return Mnemonic::Blendvpd;
        case ZYDIS_MNEMONIC_BLENDVPS:
            return Mnemonic::Blendvps;
        case ZYDIS_MNEMONIC_BLSFILL:
            return Mnemonic::Blsfill;
        case ZYDIS_MNEMONIC_BLSI:
            return Mnemonic::Blsi;
        case ZYDIS_MNEMONIC_BLSIC:
            return Mnemonic::Blsic;
        case ZYDIS_MNEMONIC_BLSMSK:
            return Mnemonic::Blsmsk;
        case ZYDIS_MNEMONIC_BLSR:
            return Mnemonic::Blsr;
        case ZYDIS_MNEMONIC_BNDCL:
            return Mnemonic::Bndcl;
        case ZYDIS_MNEMONIC_BNDCN:
            return Mnemonic::Bndcn;
        case ZYDIS_MNEMONIC_BNDCU:
            return Mnemonic::Bndcu;
        case ZYDIS_MNEMONIC_BNDLDX:
            return Mnemonic::Bndldx;
        case ZYDIS_MNEMONIC_BNDMK:
            return Mnemonic::Bndmk;
        case ZYDIS_MNEMONIC_BNDMOV:
            return Mnemonic::Bndmov;
        case ZYDIS_MNEMONIC_BNDSTX:
            return Mnemonic::Bndstx;
        case ZYDIS_MNEMONIC_BOUND:
            return Mnemonic::Bound;
        case ZYDIS_MNEMONIC_BSF:
            return Mnemonic::Bsf;
        case ZYDIS_MNEMONIC_BSR:
            return Mnemonic::Bsr;
        case ZYDIS_MNEMONIC_BSWAP:
            return Mnemonic::Bswap;
        case ZYDIS_MNEMONIC_BT:
            return Mnemonic::Bt;
        case ZYDIS_MNEMONIC_BTC:
            return Mnemonic::Btc;
        case ZYDIS_MNEMONIC_BTR:
            return Mnemonic::Btr;
        case ZYDIS_MNEMONIC_BTS:
            return Mnemonic::Bts;
        case ZYDIS_MNEMONIC_BZHI:
            return Mnemonic::Bzhi;
        case ZYDIS_MNEMONIC_CALL:
            return Mnemonic::Call;
        case ZYDIS_MNEMONIC_CBW:
            return Mnemonic::Cbw;
        case ZYDIS_MNEMONIC_CDQ:
            return Mnemonic::Cdq;
        case ZYDIS_MNEMONIC_CDQE:
            return Mnemonic::Cdqe;
        case ZYDIS_MNEMONIC_CLAC:
            return Mnemonic::Clac;
        case ZYDIS_MNEMONIC_CLC:
            return Mnemonic::Clc;
        case ZYDIS_MNEMONIC_CLD:
            return Mnemonic::Cld;
        case ZYDIS_MNEMONIC_CLDEMOTE:
            return Mnemonic::Cldemote;
        case ZYDIS_MNEMONIC_CLEVICT0:
            return Mnemonic::Clevict0;
        case ZYDIS_MNEMONIC_CLEVICT1:
            return Mnemonic::Clevict1;
        case ZYDIS_MNEMONIC_CLFLUSH:
            return Mnemonic::Clflush;
        case ZYDIS_MNEMONIC_CLFLUSHOPT:
            return Mnemonic::Clflushopt;
        case ZYDIS_MNEMONIC_CLGI:
            return Mnemonic::Clgi;
        case ZYDIS_MNEMONIC_CLI:
            return Mnemonic::Cli;
        case ZYDIS_MNEMONIC_CLRSSBSY:
            return Mnemonic::Clrssbsy;
        case ZYDIS_MNEMONIC_CLTS:
            return Mnemonic::Clts;
        case ZYDIS_MNEMONIC_CLUI:
            return Mnemonic::Clui;
        case ZYDIS_MNEMONIC_CLWB:
            return Mnemonic::Clwb;
        case ZYDIS_MNEMONIC_CLZERO:
            return Mnemonic::Clzero;
        case ZYDIS_MNEMONIC_CMC:
            return Mnemonic::Cmc;
        case ZYDIS_MNEMONIC_CMOVB:
            return Mnemonic::Cmovb;
        case ZYDIS_MNEMONIC_CMOVBE:
            return Mnemonic::Cmovbe;
        case ZYDIS_MNEMONIC_CMOVL:
            return Mnemonic::Cmovl;
        case ZYDIS_MNEMONIC_CMOVLE:
            return Mnemonic::Cmovle;
        case ZYDIS_MNEMONIC_CMOVNB:
            return Mnemonic::Cmovnb;
        case ZYDIS_MNEMONIC_CMOVNBE:
            return Mnemonic::Cmovnbe;
        case ZYDIS_MNEMONIC_CMOVNL:
            return Mnemonic::Cmovnl;
        case ZYDIS_MNEMONIC_CMOVNLE:
            return Mnemonic::Cmovnle;
        case ZYDIS_MNEMONIC_CMOVNO:
            return Mnemonic::Cmovno;
        case ZYDIS_MNEMONIC_CMOVNP:
            return Mnemonic::Cmovnp;
        case ZYDIS_MNEMONIC_CMOVNS:
            return Mnemonic::Cmovns;
        case ZYDIS_MNEMONIC_CMOVNZ:
            return Mnemonic::Cmovnz;
        case ZYDIS_MNEMONIC_CMOVO:
            return Mnemonic::Cmovo;
        case ZYDIS_MNEMONIC_CMOVP:
            return Mnemonic::Cmovp;
        case ZYDIS_MNEMONIC_CMOVS:
            return Mnemonic::Cmovs;
        case ZYDIS_MNEMONIC_CMOVZ:
            return Mnemonic::Cmovz;
        case ZYDIS_MNEMONIC_CMP:
            return Mnemonic::Cmp;
        case ZYDIS_MNEMONIC_CMPPD:
            return Mnemonic::Cmppd;
        case ZYDIS_MNEMONIC_CMPPS:
            return Mnemonic::Cmpps;
        case ZYDIS_MNEMONIC_CMPSB:
            return Mnemonic::Cmpsb;
        case ZYDIS_MNEMONIC_CMPSD:
            return Mnemonic::Cmpsd;
        case ZYDIS_MNEMONIC_CMPSQ:
            return Mnemonic::Cmpsq;
        case ZYDIS_MNEMONIC_CMPSS:
            return Mnemonic::Cmpss;
        case ZYDIS_MNEMONIC_CMPSW:
            return Mnemonic::Cmpsw;
        case ZYDIS_MNEMONIC_CMPXCHG:
            return Mnemonic::Cmpxchg;
        case ZYDIS_MNEMONIC_CMPXCHG16B:
            return Mnemonic::Cmpxchg16b;
        case ZYDIS_MNEMONIC_CMPXCHG8B:
            return Mnemonic::Cmpxchg8b;
        case ZYDIS_MNEMONIC_COMISD:
            return Mnemonic::Comisd;
        case ZYDIS_MNEMONIC_COMISS:
            return Mnemonic::Comiss;
        case ZYDIS_MNEMONIC_CPUID:
            return Mnemonic::Cpuid;
        case ZYDIS_MNEMONIC_CQO:
            return Mnemonic::Cqo;
        case ZYDIS_MNEMONIC_CRC32:
            return Mnemonic::Crc32;
        case ZYDIS_MNEMONIC_CVTDQ2PD:
            return Mnemonic::Cvtdq2pd;
        case ZYDIS_MNEMONIC_CVTDQ2PS:
            return Mnemonic::Cvtdq2ps;
        case ZYDIS_MNEMONIC_CVTPD2DQ:
            return Mnemonic::Cvtpd2dq;
        case ZYDIS_MNEMONIC_CVTPD2PI:
            return Mnemonic::Cvtpd2pi;
        case ZYDIS_MNEMONIC_CVTPD2PS:
            return Mnemonic::Cvtpd2ps;
        case ZYDIS_MNEMONIC_CVTPI2PD:
            return Mnemonic::Cvtpi2pd;
        case ZYDIS_MNEMONIC_CVTPI2PS:
            return Mnemonic::Cvtpi2ps;
        case ZYDIS_MNEMONIC_CVTPS2DQ:
            return Mnemonic::Cvtps2dq;
        case ZYDIS_MNEMONIC_CVTPS2PD:
            return Mnemonic::Cvtps2pd;
        case ZYDIS_MNEMONIC_CVTPS2PI:
            return Mnemonic::Cvtps2pi;
        case ZYDIS_MNEMONIC_CVTSD2SI:
            return Mnemonic::Cvtsd2si;
        case ZYDIS_MNEMONIC_CVTSD2SS:
            return Mnemonic::Cvtsd2ss;
        case ZYDIS_MNEMONIC_CVTSI2SD:
            return Mnemonic::Cvtsi2sd;
        case ZYDIS_MNEMONIC_CVTSI2SS:
            return Mnemonic::Cvtsi2ss;
        case ZYDIS_MNEMONIC_CVTSS2SD:
            return Mnemonic::Cvtss2sd;
        case ZYDIS_MNEMONIC_CVTSS2SI:
            return Mnemonic::Cvtss2si;
        case ZYDIS_MNEMONIC_CVTTPD2DQ:
            return Mnemonic::Cvttpd2dq;
        case ZYDIS_MNEMONIC_CVTTPD2PI:
            return Mnemonic::Cvttpd2pi;
        case ZYDIS_MNEMONIC_CVTTPS2DQ:
            return Mnemonic::Cvttps2dq;
        case ZYDIS_MNEMONIC_CVTTPS2PI:
            return Mnemonic::Cvttps2pi;
        case ZYDIS_MNEMONIC_CVTTSD2SI:
            return Mnemonic::Cvttsd2si;
        case ZYDIS_MNEMONIC_CVTTSS2SI:
            return Mnemonic::Cvttss2si;
        case ZYDIS_MNEMONIC_CWD:
            return Mnemonic::Cwd;
        case ZYDIS_MNEMONIC_CWDE:
            return Mnemonic::Cwde;
        case ZYDIS_MNEMONIC_DAA:
            return Mnemonic::Daa;
        case ZYDIS_MNEMONIC_DAS:
            return Mnemonic::Das;
        case ZYDIS_MNEMONIC_DEC:
            return Mnemonic::Dec;
        case ZYDIS_MNEMONIC_DELAY:
            return Mnemonic::Delay;
        case ZYDIS_MNEMONIC_DIV:
            return Mnemonic::Div;
        case ZYDIS_MNEMONIC_DIVPD:
            return Mnemonic::Divpd;
        case ZYDIS_MNEMONIC_DIVPS:
            return Mnemonic::Divps;
        case ZYDIS_MNEMONIC_DIVSD:
            return Mnemonic::Divsd;
        case ZYDIS_MNEMONIC_DIVSS:
            return Mnemonic::Divss;
        case ZYDIS_MNEMONIC_DPPD:
            return Mnemonic::Dppd;
        case ZYDIS_MNEMONIC_DPPS:
            return Mnemonic::Dpps;
        case ZYDIS_MNEMONIC_EMMS:
            return Mnemonic::Emms;
        case ZYDIS_MNEMONIC_ENCLS:
            return Mnemonic::Encls;
        case ZYDIS_MNEMONIC_ENCLU:
            return Mnemonic::Enclu;
        case ZYDIS_MNEMONIC_ENCLV:
            return Mnemonic::Enclv;
        case ZYDIS_MNEMONIC_ENCODEKEY128:
            return Mnemonic::Encodekey128;
        case ZYDIS_MNEMONIC_ENCODEKEY256:
            return Mnemonic::Encodekey256;
        case ZYDIS_MNEMONIC_ENDBR32:
            return Mnemonic::Endbr32;
        case ZYDIS_MNEMONIC_ENDBR64:
            return Mnemonic::Endbr64;
        case ZYDIS_MNEMONIC_ENQCMD:
            return Mnemonic::Enqcmd;
        case ZYDIS_MNEMONIC_ENQCMDS:
            return Mnemonic::Enqcmds;
        case ZYDIS_MNEMONIC_ENTER:
            return Mnemonic::Enter;
        case ZYDIS_MNEMONIC_EXTRACTPS:
            return Mnemonic::Extractps;
        case ZYDIS_MNEMONIC_EXTRQ:
            return Mnemonic::Extrq;
        case ZYDIS_MNEMONIC_F2XM1:
            return Mnemonic::F2xm1;
        case ZYDIS_MNEMONIC_FABS:
            return Mnemonic::Fabs;
        case ZYDIS_MNEMONIC_FADD:
            return Mnemonic::Fadd;
        case ZYDIS_MNEMONIC_FADDP:
            return Mnemonic::Faddp;
        case ZYDIS_MNEMONIC_FBLD:
            return Mnemonic::Fbld;
        case ZYDIS_MNEMONIC_FBSTP:
            return Mnemonic::Fbstp;
        case ZYDIS_MNEMONIC_FCHS:
            return Mnemonic::Fchs;
        case ZYDIS_MNEMONIC_FCMOVB:
            return Mnemonic::Fcmovb;
        case ZYDIS_MNEMONIC_FCMOVBE:
            return Mnemonic::Fcmovbe;
        case ZYDIS_MNEMONIC_FCMOVE:
            return Mnemonic::Fcmove;
        case ZYDIS_MNEMONIC_FCMOVNB:
            return Mnemonic::Fcmovnb;
        case ZYDIS_MNEMONIC_FCMOVNBE:
            return Mnemonic::Fcmovnbe;
        case ZYDIS_MNEMONIC_FCMOVNE:
            return Mnemonic::Fcmovne;
        case ZYDIS_MNEMONIC_FCMOVNU:
            return Mnemonic::Fcmovnu;
        case ZYDIS_MNEMONIC_FCMOVU:
            return Mnemonic::Fcmovu;
        case ZYDIS_MNEMONIC_FCOM:
            return Mnemonic::Fcom;
        case ZYDIS_MNEMONIC_FCOMI:
            return Mnemonic::Fcomi;
        case ZYDIS_MNEMONIC_FCOMIP:
            return Mnemonic::Fcomip;
        case ZYDIS_MNEMONIC_FCOMP:
            return Mnemonic::Fcomp;
        case ZYDIS_MNEMONIC_FCOMPP:
            return Mnemonic::Fcompp;
        case ZYDIS_MNEMONIC_FCOS:
            return Mnemonic::Fcos;
        case ZYDIS_MNEMONIC_FDECSTP:
            return Mnemonic::Fdecstp;
        case ZYDIS_MNEMONIC_FDISI8087_NOP:
            return Mnemonic::Fdisi8087_nop;
        case ZYDIS_MNEMONIC_FDIV:
            return Mnemonic::Fdiv;
        case ZYDIS_MNEMONIC_FDIVP:
            return Mnemonic::Fdivp;
        case ZYDIS_MNEMONIC_FDIVR:
            return Mnemonic::Fdivr;
        case ZYDIS_MNEMONIC_FDIVRP:
            return Mnemonic::Fdivrp;
        case ZYDIS_MNEMONIC_FEMMS:
            return Mnemonic::Femms;
        case ZYDIS_MNEMONIC_FENI8087_NOP:
            return Mnemonic::Feni8087_nop;
        case ZYDIS_MNEMONIC_FFREE:
            return Mnemonic::Ffree;
        case ZYDIS_MNEMONIC_FFREEP:
            return Mnemonic::Ffreep;
        case ZYDIS_MNEMONIC_FIADD:
            return Mnemonic::Fiadd;
        case ZYDIS_MNEMONIC_FICOM:
            return Mnemonic::Ficom;
        case ZYDIS_MNEMONIC_FICOMP:
            return Mnemonic::Ficomp;
        case ZYDIS_MNEMONIC_FIDIV:
            return Mnemonic::Fidiv;
        case ZYDIS_MNEMONIC_FIDIVR:
            return Mnemonic::Fidivr;
        case ZYDIS_MNEMONIC_FILD:
            return Mnemonic::Fild;
        case ZYDIS_MNEMONIC_FIMUL:
            return Mnemonic::Fimul;
        case ZYDIS_MNEMONIC_FINCSTP:
            return Mnemonic::Fincstp;
        case ZYDIS_MNEMONIC_FIST:
            return Mnemonic::Fist;
        case ZYDIS_MNEMONIC_FISTP:
            return Mnemonic::Fistp;
        case ZYDIS_MNEMONIC_FISTTP:
            return Mnemonic::Fisttp;
        case ZYDIS_MNEMONIC_FISUB:
            return Mnemonic::Fisub;
        case ZYDIS_MNEMONIC_FISUBR:
            return Mnemonic::Fisubr;
        case ZYDIS_MNEMONIC_FLD:
            return Mnemonic::Fld;
        case ZYDIS_MNEMONIC_FLD1:
            return Mnemonic::Fld1;
        case ZYDIS_MNEMONIC_FLDCW:
            return Mnemonic::Fldcw;
        case ZYDIS_MNEMONIC_FLDENV:
            return Mnemonic::Fldenv;
        case ZYDIS_MNEMONIC_FLDL2E:
            return Mnemonic::Fldl2e;
        case ZYDIS_MNEMONIC_FLDL2T:
            return Mnemonic::Fldl2t;
        case ZYDIS_MNEMONIC_FLDLG2:
            return Mnemonic::Fldlg2;
        case ZYDIS_MNEMONIC_FLDLN2:
            return Mnemonic::Fldln2;
        case ZYDIS_MNEMONIC_FLDPI:
            return Mnemonic::Fldpi;
        case ZYDIS_MNEMONIC_FLDZ:
            return Mnemonic::Fldz;
        case ZYDIS_MNEMONIC_FMUL:
            return Mnemonic::Fmul;
        case ZYDIS_MNEMONIC_FMULP:
            return Mnemonic::Fmulp;
        case ZYDIS_MNEMONIC_FNCLEX:
            return Mnemonic::Fnclex;
        case ZYDIS_MNEMONIC_FNINIT:
            return Mnemonic::Fninit;
        case ZYDIS_MNEMONIC_FNOP:
            return Mnemonic::Fnop;
        case ZYDIS_MNEMONIC_FNSAVE:
            return Mnemonic::Fnsave;
        case ZYDIS_MNEMONIC_FNSTCW:
            return Mnemonic::Fnstcw;
        case ZYDIS_MNEMONIC_FNSTENV:
            return Mnemonic::Fnstenv;
        case ZYDIS_MNEMONIC_FNSTSW:
            return Mnemonic::Fnstsw;
        case ZYDIS_MNEMONIC_FPATAN:
            return Mnemonic::Fpatan;
        case ZYDIS_MNEMONIC_FPREM:
            return Mnemonic::Fprem;
        case ZYDIS_MNEMONIC_FPREM1:
            return Mnemonic::Fprem1;
        case ZYDIS_MNEMONIC_FPTAN:
            return Mnemonic::Fptan;
        case ZYDIS_MNEMONIC_FRNDINT:
            return Mnemonic::Frndint;
        case ZYDIS_MNEMONIC_FRSTOR:
            return Mnemonic::Frstor;
        case ZYDIS_MNEMONIC_FSCALE:
            return Mnemonic::Fscale;
        case ZYDIS_MNEMONIC_FSETPM287_NOP:
            return Mnemonic::Fsetpm287_nop;
        case ZYDIS_MNEMONIC_FSIN:
            return Mnemonic::Fsin;
        case ZYDIS_MNEMONIC_FSINCOS:
            return Mnemonic::Fsincos;
        case ZYDIS_MNEMONIC_FSQRT:
            return Mnemonic::Fsqrt;
        case ZYDIS_MNEMONIC_FST:
            return Mnemonic::Fst;
        case ZYDIS_MNEMONIC_FSTP:
            return Mnemonic::Fstp;
        case ZYDIS_MNEMONIC_FSTPNCE:
            return Mnemonic::Fstpnce;
        case ZYDIS_MNEMONIC_FSUB:
            return Mnemonic::Fsub;
        case ZYDIS_MNEMONIC_FSUBP:
            return Mnemonic::Fsubp;
        case ZYDIS_MNEMONIC_FSUBR:
            return Mnemonic::Fsubr;
        case ZYDIS_MNEMONIC_FSUBRP:
            return Mnemonic::Fsubrp;
        case ZYDIS_MNEMONIC_FTST:
            return Mnemonic::Ftst;
        case ZYDIS_MNEMONIC_FUCOM:
            return Mnemonic::Fucom;
        case ZYDIS_MNEMONIC_FUCOMI:
            return Mnemonic::Fucomi;
        case ZYDIS_MNEMONIC_FUCOMIP:
            return Mnemonic::Fucomip;
        case ZYDIS_MNEMONIC_FUCOMP:
            return Mnemonic::Fucomp;
        case ZYDIS_MNEMONIC_FUCOMPP:
            return Mnemonic::Fucompp;
        case ZYDIS_MNEMONIC_FWAIT:
            return Mnemonic::Fwait;
        case ZYDIS_MNEMONIC_FXAM:
            return Mnemonic::Fxam;
        case ZYDIS_MNEMONIC_FXCH:
            return Mnemonic::Fxch;
        case ZYDIS_MNEMONIC_FXRSTOR:
            return Mnemonic::Fxrstor;
        case ZYDIS_MNEMONIC_FXRSTOR64:
            return Mnemonic::Fxrstor64;
        case ZYDIS_MNEMONIC_FXSAVE:
            return Mnemonic::Fxsave;
        case ZYDIS_MNEMONIC_FXSAVE64:
            return Mnemonic::Fxsave64;
        case ZYDIS_MNEMONIC_FXTRACT:
            return Mnemonic::Fxtract;
        case ZYDIS_MNEMONIC_FYL2X:
            return Mnemonic::Fyl2x;
        case ZYDIS_MNEMONIC_FYL2XP1:
            return Mnemonic::Fyl2xp1;
        case ZYDIS_MNEMONIC_GETSEC:
            return Mnemonic::Getsec;
        case ZYDIS_MNEMONIC_GF2P8AFFINEINVQB:
            return Mnemonic::Gf2p8affineinvqb;
        case ZYDIS_MNEMONIC_GF2P8AFFINEQB:
            return Mnemonic::Gf2p8affineqb;
        case ZYDIS_MNEMONIC_GF2P8MULB:
            return Mnemonic::Gf2p8mulb;
        case ZYDIS_MNEMONIC_HADDPD:
            return Mnemonic::Haddpd;
        case ZYDIS_MNEMONIC_HADDPS:
            return Mnemonic::Haddps;
        case ZYDIS_MNEMONIC_HLT:
            return Mnemonic::Hlt;
        case ZYDIS_MNEMONIC_HRESET:
            return Mnemonic::Hreset;
        case ZYDIS_MNEMONIC_HSUBPD:
            return Mnemonic::Hsubpd;
        case ZYDIS_MNEMONIC_HSUBPS:
            return Mnemonic::Hsubps;
        case ZYDIS_MNEMONIC_IDIV:
            return Mnemonic::Idiv;
        case ZYDIS_MNEMONIC_IMUL:
            return Mnemonic::Imul;
        case ZYDIS_MNEMONIC_IN:
            return Mnemonic::In;
        case ZYDIS_MNEMONIC_INC:
            return Mnemonic::Inc;
        case ZYDIS_MNEMONIC_INCSSPD:
            return Mnemonic::Incsspd;
        case ZYDIS_MNEMONIC_INCSSPQ:
            return Mnemonic::Incsspq;
        case ZYDIS_MNEMONIC_INSB:
            return Mnemonic::Insb;
        case ZYDIS_MNEMONIC_INSD:
            return Mnemonic::Insd;
        case ZYDIS_MNEMONIC_INSERTPS:
            return Mnemonic::Insertps;
        case ZYDIS_MNEMONIC_INSERTQ:
            return Mnemonic::Insertq;
        case ZYDIS_MNEMONIC_INSW:
            return Mnemonic::Insw;
        case ZYDIS_MNEMONIC_INT:
            return Mnemonic::Int;
        case ZYDIS_MNEMONIC_INT1:
            return Mnemonic::Int1;
        case ZYDIS_MNEMONIC_INT3:
            return Mnemonic::Int3;
        case ZYDIS_MNEMONIC_INTO:
            return Mnemonic::Into;
        case ZYDIS_MNEMONIC_INVD:
            return Mnemonic::Invd;
        case ZYDIS_MNEMONIC_INVEPT:
            return Mnemonic::Invept;
        case ZYDIS_MNEMONIC_INVLPG:
            return Mnemonic::Invlpg;
        case ZYDIS_MNEMONIC_INVLPGA:
            return Mnemonic::Invlpga;
        case ZYDIS_MNEMONIC_INVLPGB:
            return Mnemonic::Invlpgb;
        case ZYDIS_MNEMONIC_INVPCID:
            return Mnemonic::Invpcid;
        case ZYDIS_MNEMONIC_INVVPID:
            return Mnemonic::Invvpid;
        case ZYDIS_MNEMONIC_IRET:
            return Mnemonic::Iret;
        case ZYDIS_MNEMONIC_IRETD:
            return Mnemonic::Iretd;
        case ZYDIS_MNEMONIC_IRETQ:
            return Mnemonic::Iretq;
        case ZYDIS_MNEMONIC_JB:
            return Mnemonic::Jb;
        case ZYDIS_MNEMONIC_JBE:
            return Mnemonic::Jbe;
        case ZYDIS_MNEMONIC_JCXZ:
            return Mnemonic::Jcxz;
        case ZYDIS_MNEMONIC_JECXZ:
            return Mnemonic::Jecxz;
        case ZYDIS_MNEMONIC_JKNZD:
            return Mnemonic::Jknzd;
        case ZYDIS_MNEMONIC_JKZD:
            return Mnemonic::Jkzd;
        case ZYDIS_MNEMONIC_JL:
            return Mnemonic::Jl;
        case ZYDIS_MNEMONIC_JLE:
            return Mnemonic::Jle;
        case ZYDIS_MNEMONIC_JMP:
            return Mnemonic::Jmp;
        case ZYDIS_MNEMONIC_JNB:
            return Mnemonic::Jnb;
        case ZYDIS_MNEMONIC_JNBE:
            return Mnemonic::Jnbe;
        case ZYDIS_MNEMONIC_JNL:
            return Mnemonic::Jnl;
        case ZYDIS_MNEMONIC_JNLE:
            return Mnemonic::Jnle;
        case ZYDIS_MNEMONIC_JNO:
            return Mnemonic::Jno;
        case ZYDIS_MNEMONIC_JNP:
            return Mnemonic::Jnp;
        case ZYDIS_MNEMONIC_JNS:
            return Mnemonic::Jns;
        case ZYDIS_MNEMONIC_JNZ:
            return Mnemonic::Jnz;
        case ZYDIS_MNEMONIC_JO:
            return Mnemonic::Jo;
        case ZYDIS_MNEMONIC_JP:
            return Mnemonic::Jp;
        case ZYDIS_MNEMONIC_JRCXZ:
            return Mnemonic::Jrcxz;
        case ZYDIS_MNEMONIC_JS:
            return Mnemonic::Js;
        case ZYDIS_MNEMONIC_JZ:
            return Mnemonic::Jz;
        case ZYDIS_MNEMONIC_KADDB:
            return Mnemonic::Kaddb;
        case ZYDIS_MNEMONIC_KADDD:
            return Mnemonic::Kaddd;
        case ZYDIS_MNEMONIC_KADDQ:
            return Mnemonic::Kaddq;
        case ZYDIS_MNEMONIC_KADDW:
            return Mnemonic::Kaddw;
        case ZYDIS_MNEMONIC_KAND:
            return Mnemonic::Kand;
        case ZYDIS_MNEMONIC_KANDB:
            return Mnemonic::Kandb;
        case ZYDIS_MNEMONIC_KANDD:
            return Mnemonic::Kandd;
        case ZYDIS_MNEMONIC_KANDN:
            return Mnemonic::Kandn;
        case ZYDIS_MNEMONIC_KANDNB:
            return Mnemonic::Kandnb;
        case ZYDIS_MNEMONIC_KANDND:
            return Mnemonic::Kandnd;
        case ZYDIS_MNEMONIC_KANDNQ:
            return Mnemonic::Kandnq;
        case ZYDIS_MNEMONIC_KANDNR:
            return Mnemonic::Kandnr;
        case ZYDIS_MNEMONIC_KANDNW:
            return Mnemonic::Kandnw;
        case ZYDIS_MNEMONIC_KANDQ:
            return Mnemonic::Kandq;
        case ZYDIS_MNEMONIC_KANDW:
            return Mnemonic::Kandw;
        case ZYDIS_MNEMONIC_KCONCATH:
            return Mnemonic::Kconcath;
        case ZYDIS_MNEMONIC_KCONCATL:
            return Mnemonic::Kconcatl;
        case ZYDIS_MNEMONIC_KEXTRACT:
            return Mnemonic::Kextract;
        case ZYDIS_MNEMONIC_KMERGE2L1H:
            return Mnemonic::Kmerge2l1h;
        case ZYDIS_MNEMONIC_KMERGE2L1L:
            return Mnemonic::Kmerge2l1l;
        case ZYDIS_MNEMONIC_KMOV:
            return Mnemonic::Kmov;
        case ZYDIS_MNEMONIC_KMOVB:
            return Mnemonic::Kmovb;
        case ZYDIS_MNEMONIC_KMOVD:
            return Mnemonic::Kmovd;
        case ZYDIS_MNEMONIC_KMOVQ:
            return Mnemonic::Kmovq;
        case ZYDIS_MNEMONIC_KMOVW:
            return Mnemonic::Kmovw;
        case ZYDIS_MNEMONIC_KNOT:
            return Mnemonic::Knot;
        case ZYDIS_MNEMONIC_KNOTB:
            return Mnemonic::Knotb;
        case ZYDIS_MNEMONIC_KNOTD:
            return Mnemonic::Knotd;
        case ZYDIS_MNEMONIC_KNOTQ:
            return Mnemonic::Knotq;
        case ZYDIS_MNEMONIC_KNOTW:
            return Mnemonic::Knotw;
        case ZYDIS_MNEMONIC_KOR:
            return Mnemonic::Kor;
        case ZYDIS_MNEMONIC_KORB:
            return Mnemonic::Korb;
        case ZYDIS_MNEMONIC_KORD:
            return Mnemonic::Kord;
        case ZYDIS_MNEMONIC_KORQ:
            return Mnemonic::Korq;
        case ZYDIS_MNEMONIC_KORTEST:
            return Mnemonic::Kortest;
        case ZYDIS_MNEMONIC_KORTESTB:
            return Mnemonic::Kortestb;
        case ZYDIS_MNEMONIC_KORTESTD:
            return Mnemonic::Kortestd;
        case ZYDIS_MNEMONIC_KORTESTQ:
            return Mnemonic::Kortestq;
        case ZYDIS_MNEMONIC_KORTESTW:
            return Mnemonic::Kortestw;
        case ZYDIS_MNEMONIC_KORW:
            return Mnemonic::Korw;
        case ZYDIS_MNEMONIC_KSHIFTLB:
            return Mnemonic::Kshiftlb;
        case ZYDIS_MNEMONIC_KSHIFTLD:
            return Mnemonic::Kshiftld;
        case ZYDIS_MNEMONIC_KSHIFTLQ:
            return Mnemonic::Kshiftlq;
        case ZYDIS_MNEMONIC_KSHIFTLW:
            return Mnemonic::Kshiftlw;
        case ZYDIS_MNEMONIC_KSHIFTRB:
            return Mnemonic::Kshiftrb;
        case ZYDIS_MNEMONIC_KSHIFTRD:
            return Mnemonic::Kshiftrd;
        case ZYDIS_MNEMONIC_KSHIFTRQ:
            return Mnemonic::Kshiftrq;
        case ZYDIS_MNEMONIC_KSHIFTRW:
            return Mnemonic::Kshiftrw;
        case ZYDIS_MNEMONIC_KTESTB:
            return Mnemonic::Ktestb;
        case ZYDIS_MNEMONIC_KTESTD:
            return Mnemonic::Ktestd;
        case ZYDIS_MNEMONIC_KTESTQ:
            return Mnemonic::Ktestq;
        case ZYDIS_MNEMONIC_KTESTW:
            return Mnemonic::Ktestw;
        case ZYDIS_MNEMONIC_KUNPCKBW:
            return Mnemonic::Kunpckbw;
        case ZYDIS_MNEMONIC_KUNPCKDQ:
            return Mnemonic::Kunpckdq;
        case ZYDIS_MNEMONIC_KUNPCKWD:
            return Mnemonic::Kunpckwd;
        case ZYDIS_MNEMONIC_KXNOR:
            return Mnemonic::Kxnor;
        case ZYDIS_MNEMONIC_KXNORB:
            return Mnemonic::Kxnorb;
        case ZYDIS_MNEMONIC_KXNORD:
            return Mnemonic::Kxnord;
        case ZYDIS_MNEMONIC_KXNORQ:
            return Mnemonic::Kxnorq;
        case ZYDIS_MNEMONIC_KXNORW:
            return Mnemonic::Kxnorw;
        case ZYDIS_MNEMONIC_KXOR:
            return Mnemonic::Kxor;
        case ZYDIS_MNEMONIC_KXORB:
            return Mnemonic::Kxorb;
        case ZYDIS_MNEMONIC_KXORD:
            return Mnemonic::Kxord;
        case ZYDIS_MNEMONIC_KXORQ:
            return Mnemonic::Kxorq;
        case ZYDIS_MNEMONIC_KXORW:
            return Mnemonic::Kxorw;
        case ZYDIS_MNEMONIC_LAHF:
            return Mnemonic::Lahf;
        case ZYDIS_MNEMONIC_LAR:
            return Mnemonic::Lar;
        case ZYDIS_MNEMONIC_LDDQU:
            return Mnemonic::Lddqu;
        case ZYDIS_MNEMONIC_LDMXCSR:
            return Mnemonic::Ldmxcsr;
        case ZYDIS_MNEMONIC_LDS:
            return Mnemonic::Lds;
        case ZYDIS_MNEMONIC_LDTILECFG:
            return Mnemonic::Ldtilecfg;
        case ZYDIS_MNEMONIC_LEA:
            return Mnemonic::Lea;
        case ZYDIS_MNEMONIC_LEAVE:
            return Mnemonic::Leave;
        case ZYDIS_MNEMONIC_LES:
            return Mnemonic::Les;
        case ZYDIS_MNEMONIC_LFENCE:
            return Mnemonic::Lfence;
        case ZYDIS_MNEMONIC_LFS:
            return Mnemonic::Lfs;
        case ZYDIS_MNEMONIC_LGDT:
            return Mnemonic::Lgdt;
        case ZYDIS_MNEMONIC_LGS:
            return Mnemonic::Lgs;
        case ZYDIS_MNEMONIC_LIDT:
            return Mnemonic::Lidt;
        case ZYDIS_MNEMONIC_LLDT:
            return Mnemonic::Lldt;
        case ZYDIS_MNEMONIC_LLWPCB:
            return Mnemonic::Llwpcb;
        case ZYDIS_MNEMONIC_LMSW:
            return Mnemonic::Lmsw;
        case ZYDIS_MNEMONIC_LOADIWKEY:
            return Mnemonic::Loadiwkey;
        case ZYDIS_MNEMONIC_LODSB:
            return Mnemonic::Lodsb;
        case ZYDIS_MNEMONIC_LODSD:
            return Mnemonic::Lodsd;
        case ZYDIS_MNEMONIC_LODSQ:
            return Mnemonic::Lodsq;
        case ZYDIS_MNEMONIC_LODSW:
            return Mnemonic::Lodsw;
        case ZYDIS_MNEMONIC_LOOP:
            return Mnemonic::Loop;
        case ZYDIS_MNEMONIC_LOOPE:
            return Mnemonic::Loope;
        case ZYDIS_MNEMONIC_LOOPNE:
            return Mnemonic::Loopne;
        case ZYDIS_MNEMONIC_LSL:
            return Mnemonic::Lsl;
        case ZYDIS_MNEMONIC_LSS:
            return Mnemonic::Lss;
        case ZYDIS_MNEMONIC_LTR:
            return Mnemonic::Ltr;
        case ZYDIS_MNEMONIC_LWPINS:
            return Mnemonic::Lwpins;
        case ZYDIS_MNEMONIC_LWPVAL:
            return Mnemonic::Lwpval;
        case ZYDIS_MNEMONIC_LZCNT:
            return Mnemonic::Lzcnt;
        case ZYDIS_MNEMONIC_MASKMOVDQU:
            return Mnemonic::Maskmovdqu;
        case ZYDIS_MNEMONIC_MASKMOVQ:
            return Mnemonic::Maskmovq;
        case ZYDIS_MNEMONIC_MAXPD:
            return Mnemonic::Maxpd;
        case ZYDIS_MNEMONIC_MAXPS:
            return Mnemonic::Maxps;
        case ZYDIS_MNEMONIC_MAXSD:
            return Mnemonic::Maxsd;
        case ZYDIS_MNEMONIC_MAXSS:
            return Mnemonic::Maxss;
        case ZYDIS_MNEMONIC_MCOMMIT:
            return Mnemonic::Mcommit;
        case ZYDIS_MNEMONIC_MFENCE:
            return Mnemonic::Mfence;
        case ZYDIS_MNEMONIC_MINPD:
            return Mnemonic::Minpd;
        case ZYDIS_MNEMONIC_MINPS:
            return Mnemonic::Minps;
        case ZYDIS_MNEMONIC_MINSD:
            return Mnemonic::Minsd;
        case ZYDIS_MNEMONIC_MINSS:
            return Mnemonic::Minss;
        case ZYDIS_MNEMONIC_MONITOR:
            return Mnemonic::Monitor;
        case ZYDIS_MNEMONIC_MONITORX:
            return Mnemonic::Monitorx;
        case ZYDIS_MNEMONIC_MONTMUL:
            return Mnemonic::Montmul;
        case ZYDIS_MNEMONIC_MOV:
            return Mnemonic::Mov;
        case ZYDIS_MNEMONIC_MOVAPD:
            return Mnemonic::Movapd;
        case ZYDIS_MNEMONIC_MOVAPS:
            return Mnemonic::Movaps;
        case ZYDIS_MNEMONIC_MOVBE:
            return Mnemonic::Movbe;
        case ZYDIS_MNEMONIC_MOVD:
            return Mnemonic::Movd;
        case ZYDIS_MNEMONIC_MOVDDUP:
            return Mnemonic::Movddup;
        case ZYDIS_MNEMONIC_MOVDIR64B:
            return Mnemonic::Movdir64b;
        case ZYDIS_MNEMONIC_MOVDIRI:
            return Mnemonic::Movdiri;
        case ZYDIS_MNEMONIC_MOVDQ2Q:
            return Mnemonic::Movdq2q;
        case ZYDIS_MNEMONIC_MOVDQA:
            return Mnemonic::Movdqa;
        case ZYDIS_MNEMONIC_MOVDQU:
            return Mnemonic::Movdqu;
        case ZYDIS_MNEMONIC_MOVHLPS:
            return Mnemonic::Movhlps;
        case ZYDIS_MNEMONIC_MOVHPD:
            return Mnemonic::Movhpd;
        case ZYDIS_MNEMONIC_MOVHPS:
            return Mnemonic::Movhps;
        case ZYDIS_MNEMONIC_MOVLHPS:
            return Mnemonic::Movlhps;
        case ZYDIS_MNEMONIC_MOVLPD:
            return Mnemonic::Movlpd;
        case ZYDIS_MNEMONIC_MOVLPS:
            return Mnemonic::Movlps;
        case ZYDIS_MNEMONIC_MOVMSKPD:
            return Mnemonic::Movmskpd;
        case ZYDIS_MNEMONIC_MOVMSKPS:
            return Mnemonic::Movmskps;
        case ZYDIS_MNEMONIC_MOVNTDQ:
            return Mnemonic::Movntdq;
        case ZYDIS_MNEMONIC_MOVNTDQA:
            return Mnemonic::Movntdqa;
        case ZYDIS_MNEMONIC_MOVNTI:
            return Mnemonic::Movnti;
        case ZYDIS_MNEMONIC_MOVNTPD:
            return Mnemonic::Movntpd;
        case ZYDIS_MNEMONIC_MOVNTPS:
            return Mnemonic::Movntps;
        case ZYDIS_MNEMONIC_MOVNTQ:
            return Mnemonic::Movntq;
        case ZYDIS_MNEMONIC_MOVNTSD:
            return Mnemonic::Movntsd;
        case ZYDIS_MNEMONIC_MOVNTSS:
            return Mnemonic::Movntss;
        case ZYDIS_MNEMONIC_MOVQ:
            return Mnemonic::Movq;
        case ZYDIS_MNEMONIC_MOVQ2DQ:
            return Mnemonic::Movq2dq;
        case ZYDIS_MNEMONIC_MOVSB:
            return Mnemonic::Movsb;
        case ZYDIS_MNEMONIC_MOVSD:
            return Mnemonic::Movsd;
        case ZYDIS_MNEMONIC_MOVSHDUP:
            return Mnemonic::Movshdup;
        case ZYDIS_MNEMONIC_MOVSLDUP:
            return Mnemonic::Movsldup;
        case ZYDIS_MNEMONIC_MOVSQ:
            return Mnemonic::Movsq;
        case ZYDIS_MNEMONIC_MOVSS:
            return Mnemonic::Movss;
        case ZYDIS_MNEMONIC_MOVSW:
            return Mnemonic::Movsw;
        case ZYDIS_MNEMONIC_MOVSX:
            return Mnemonic::Movsx;
        case ZYDIS_MNEMONIC_MOVSXD:
            return Mnemonic::Movsxd;
        case ZYDIS_MNEMONIC_MOVUPD:
            return Mnemonic::Movupd;
        case ZYDIS_MNEMONIC_MOVUPS:
            return Mnemonic::Movups;
        case ZYDIS_MNEMONIC_MOVZX:
            return Mnemonic::Movzx;
        case ZYDIS_MNEMONIC_MPSADBW:
            return Mnemonic::Mpsadbw;
        case ZYDIS_MNEMONIC_MUL:
            return Mnemonic::Mul;
        case ZYDIS_MNEMONIC_MULPD:
            return Mnemonic::Mulpd;
        case ZYDIS_MNEMONIC_MULPS:
            return Mnemonic::Mulps;
        case ZYDIS_MNEMONIC_MULSD:
            return Mnemonic::Mulsd;
        case ZYDIS_MNEMONIC_MULSS:
            return Mnemonic::Mulss;
        case ZYDIS_MNEMONIC_MULX:
            return Mnemonic::Mulx;
        case ZYDIS_MNEMONIC_MWAIT:
            return Mnemonic::Mwait;
        case ZYDIS_MNEMONIC_MWAITX:
            return Mnemonic::Mwaitx;
        case ZYDIS_MNEMONIC_NEG:
            return Mnemonic::Neg;
        case ZYDIS_MNEMONIC_NOP:
            return Mnemonic::Nop;
        case ZYDIS_MNEMONIC_NOT:
            return Mnemonic::Not;
        case ZYDIS_MNEMONIC_OR:
            return Mnemonic::Or;
        case ZYDIS_MNEMONIC_ORPD:
            return Mnemonic::Orpd;
        case ZYDIS_MNEMONIC_ORPS:
            return Mnemonic::Orps;
        case ZYDIS_MNEMONIC_OUT:
            return Mnemonic::Out;
        case ZYDIS_MNEMONIC_OUTSB:
            return Mnemonic::Outsb;
        case ZYDIS_MNEMONIC_OUTSD:
            return Mnemonic::Outsd;
        case ZYDIS_MNEMONIC_OUTSW:
            return Mnemonic::Outsw;
        case ZYDIS_MNEMONIC_PABSB:
            return Mnemonic::Pabsb;
        case ZYDIS_MNEMONIC_PABSD:
            return Mnemonic::Pabsd;
        case ZYDIS_MNEMONIC_PABSW:
            return Mnemonic::Pabsw;
        case ZYDIS_MNEMONIC_PACKSSDW:
            return Mnemonic::Packssdw;
        case ZYDIS_MNEMONIC_PACKSSWB:
            return Mnemonic::Packsswb;
        case ZYDIS_MNEMONIC_PACKUSDW:
            return Mnemonic::Packusdw;
        case ZYDIS_MNEMONIC_PACKUSWB:
            return Mnemonic::Packuswb;
        case ZYDIS_MNEMONIC_PADDB:
            return Mnemonic::Paddb;
        case ZYDIS_MNEMONIC_PADDD:
            return Mnemonic::Paddd;
        case ZYDIS_MNEMONIC_PADDQ:
            return Mnemonic::Paddq;
        case ZYDIS_MNEMONIC_PADDSB:
            return Mnemonic::Paddsb;
        case ZYDIS_MNEMONIC_PADDSW:
            return Mnemonic::Paddsw;
        case ZYDIS_MNEMONIC_PADDUSB:
            return Mnemonic::Paddusb;
        case ZYDIS_MNEMONIC_PADDUSW:
            return Mnemonic::Paddusw;
        case ZYDIS_MNEMONIC_PADDW:
            return Mnemonic::Paddw;
        case ZYDIS_MNEMONIC_PALIGNR:
            return Mnemonic::Palignr;
        case ZYDIS_MNEMONIC_PAND:
            return Mnemonic::Pand;
        case ZYDIS_MNEMONIC_PANDN:
            return Mnemonic::Pandn;
        case ZYDIS_MNEMONIC_PAUSE:
            return Mnemonic::Pause;
        case ZYDIS_MNEMONIC_PAVGB:
            return Mnemonic::Pavgb;
        case ZYDIS_MNEMONIC_PAVGUSB:
            return Mnemonic::Pavgusb;
        case ZYDIS_MNEMONIC_PAVGW:
            return Mnemonic::Pavgw;
        case ZYDIS_MNEMONIC_PBLENDVB:
            return Mnemonic::Pblendvb;
        case ZYDIS_MNEMONIC_PBLENDW:
            return Mnemonic::Pblendw;
        case ZYDIS_MNEMONIC_PCLMULQDQ:
            return Mnemonic::Pclmulqdq;
        case ZYDIS_MNEMONIC_PCMPEQB:
            return Mnemonic::Pcmpeqb;
        case ZYDIS_MNEMONIC_PCMPEQD:
            return Mnemonic::Pcmpeqd;
        case ZYDIS_MNEMONIC_PCMPEQQ:
            return Mnemonic::Pcmpeqq;
        case ZYDIS_MNEMONIC_PCMPEQW:
            return Mnemonic::Pcmpeqw;
        case ZYDIS_MNEMONIC_PCMPESTRI:
            return Mnemonic::Pcmpestri;
        case ZYDIS_MNEMONIC_PCMPESTRM:
            return Mnemonic::Pcmpestrm;
        case ZYDIS_MNEMONIC_PCMPGTB:
            return Mnemonic::Pcmpgtb;
        case ZYDIS_MNEMONIC_PCMPGTD:
            return Mnemonic::Pcmpgtd;
        case ZYDIS_MNEMONIC_PCMPGTQ:
            return Mnemonic::Pcmpgtq;
        case ZYDIS_MNEMONIC_PCMPGTW:
            return Mnemonic::Pcmpgtw;
        case ZYDIS_MNEMONIC_PCMPISTRI:
            return Mnemonic::Pcmpistri;
        case ZYDIS_MNEMONIC_PCMPISTRM:
            return Mnemonic::Pcmpistrm;
        case ZYDIS_MNEMONIC_PCONFIG:
            return Mnemonic::Pconfig;
        case ZYDIS_MNEMONIC_PDEP:
            return Mnemonic::Pdep;
        case ZYDIS_MNEMONIC_PEXT:
            return Mnemonic::Pext;
        case ZYDIS_MNEMONIC_PEXTRB:
            return Mnemonic::Pextrb;
        case ZYDIS_MNEMONIC_PEXTRD:
            return Mnemonic::Pextrd;
        case ZYDIS_MNEMONIC_PEXTRQ:
            return Mnemonic::Pextrq;
        case ZYDIS_MNEMONIC_PEXTRW:
            return Mnemonic::Pextrw;
        case ZYDIS_MNEMONIC_PF2ID:
            return Mnemonic::Pf2id;
        case ZYDIS_MNEMONIC_PF2IW:
            return Mnemonic::Pf2iw;
        case ZYDIS_MNEMONIC_PFACC:
            return Mnemonic::Pfacc;
        case ZYDIS_MNEMONIC_PFADD:
            return Mnemonic::Pfadd;
        case ZYDIS_MNEMONIC_PFCMPEQ:
            return Mnemonic::Pfcmpeq;
        case ZYDIS_MNEMONIC_PFCMPGE:
            return Mnemonic::Pfcmpge;
        case ZYDIS_MNEMONIC_PFCMPGT:
            return Mnemonic::Pfcmpgt;
        case ZYDIS_MNEMONIC_PFCPIT1:
            return Mnemonic::Pfcpit1;
        case ZYDIS_MNEMONIC_PFMAX:
            return Mnemonic::Pfmax;
        case ZYDIS_MNEMONIC_PFMIN:
            return Mnemonic::Pfmin;
        case ZYDIS_MNEMONIC_PFMUL:
            return Mnemonic::Pfmul;
        case ZYDIS_MNEMONIC_PFNACC:
            return Mnemonic::Pfnacc;
        case ZYDIS_MNEMONIC_PFPNACC:
            return Mnemonic::Pfpnacc;
        case ZYDIS_MNEMONIC_PFRCP:
            return Mnemonic::Pfrcp;
        case ZYDIS_MNEMONIC_PFRCPIT2:
            return Mnemonic::Pfrcpit2;
        case ZYDIS_MNEMONIC_PFRSQIT1:
            return Mnemonic::Pfrsqit1;
        case ZYDIS_MNEMONIC_PFSQRT:
            return Mnemonic::Pfsqrt;
        case ZYDIS_MNEMONIC_PFSUB:
            return Mnemonic::Pfsub;
        case ZYDIS_MNEMONIC_PFSUBR:
            return Mnemonic::Pfsubr;
        case ZYDIS_MNEMONIC_PHADDD:
            return Mnemonic::Phaddd;
        case ZYDIS_MNEMONIC_PHADDSW:
            return Mnemonic::Phaddsw;
        case ZYDIS_MNEMONIC_PHADDW:
            return Mnemonic::Phaddw;
        case ZYDIS_MNEMONIC_PHMINPOSUW:
            return Mnemonic::Phminposuw;
        case ZYDIS_MNEMONIC_PHSUBD:
            return Mnemonic::Phsubd;
        case ZYDIS_MNEMONIC_PHSUBSW:
            return Mnemonic::Phsubsw;
        case ZYDIS_MNEMONIC_PHSUBW:
            return Mnemonic::Phsubw;
        case ZYDIS_MNEMONIC_PI2FD:
            return Mnemonic::Pi2fd;
        case ZYDIS_MNEMONIC_PI2FW:
            return Mnemonic::Pi2fw;
        case ZYDIS_MNEMONIC_PINSRB:
            return Mnemonic::Pinsrb;
        case ZYDIS_MNEMONIC_PINSRD:
            return Mnemonic::Pinsrd;
        case ZYDIS_MNEMONIC_PINSRQ:
            return Mnemonic::Pinsrq;
        case ZYDIS_MNEMONIC_PINSRW:
            return Mnemonic::Pinsrw;
        case ZYDIS_MNEMONIC_PMADDUBSW:
            return Mnemonic::Pmaddubsw;
        case ZYDIS_MNEMONIC_PMADDWD:
            return Mnemonic::Pmaddwd;
        case ZYDIS_MNEMONIC_PMAXSB:
            return Mnemonic::Pmaxsb;
        case ZYDIS_MNEMONIC_PMAXSD:
            return Mnemonic::Pmaxsd;
        case ZYDIS_MNEMONIC_PMAXSW:
            return Mnemonic::Pmaxsw;
        case ZYDIS_MNEMONIC_PMAXUB:
            return Mnemonic::Pmaxub;
        case ZYDIS_MNEMONIC_PMAXUD:
            return Mnemonic::Pmaxud;
        case ZYDIS_MNEMONIC_PMAXUW:
            return Mnemonic::Pmaxuw;
        case ZYDIS_MNEMONIC_PMINSB:
            return Mnemonic::Pminsb;
        case ZYDIS_MNEMONIC_PMINSD:
            return Mnemonic::Pminsd;
        case ZYDIS_MNEMONIC_PMINSW:
            return Mnemonic::Pminsw;
        case ZYDIS_MNEMONIC_PMINUB:
            return Mnemonic::Pminub;
        case ZYDIS_MNEMONIC_PMINUD:
            return Mnemonic::Pminud;
        case ZYDIS_MNEMONIC_PMINUW:
            return Mnemonic::Pminuw;
        case ZYDIS_MNEMONIC_PMOVMSKB:
            return Mnemonic::Pmovmskb;
        case ZYDIS_MNEMONIC_PMOVSXBD:
            return Mnemonic::Pmovsxbd;
        case ZYDIS_MNEMONIC_PMOVSXBQ:
            return Mnemonic::Pmovsxbq;
        case ZYDIS_MNEMONIC_PMOVSXBW:
            return Mnemonic::Pmovsxbw;
        case ZYDIS_MNEMONIC_PMOVSXDQ:
            return Mnemonic::Pmovsxdq;
        case ZYDIS_MNEMONIC_PMOVSXWD:
            return Mnemonic::Pmovsxwd;
        case ZYDIS_MNEMONIC_PMOVSXWQ:
            return Mnemonic::Pmovsxwq;
        case ZYDIS_MNEMONIC_PMOVZXBD:
            return Mnemonic::Pmovzxbd;
        case ZYDIS_MNEMONIC_PMOVZXBQ:
            return Mnemonic::Pmovzxbq;
        case ZYDIS_MNEMONIC_PMOVZXBW:
            return Mnemonic::Pmovzxbw;
        case ZYDIS_MNEMONIC_PMOVZXDQ:
            return Mnemonic::Pmovzxdq;
        case ZYDIS_MNEMONIC_PMOVZXWD:
            return Mnemonic::Pmovzxwd;
        case ZYDIS_MNEMONIC_PMOVZXWQ:
            return Mnemonic::Pmovzxwq;
        case ZYDIS_MNEMONIC_PMULDQ:
            return Mnemonic::Pmuldq;
        case ZYDIS_MNEMONIC_PMULHRSW:
            return Mnemonic::Pmulhrsw;
        case ZYDIS_MNEMONIC_PMULHRW:
            return Mnemonic::Pmulhrw;
        case ZYDIS_MNEMONIC_PMULHUW:
            return Mnemonic::Pmulhuw;
        case ZYDIS_MNEMONIC_PMULHW:
            return Mnemonic::Pmulhw;
        case ZYDIS_MNEMONIC_PMULLD:
            return Mnemonic::Pmulld;
        case ZYDIS_MNEMONIC_PMULLW:
            return Mnemonic::Pmullw;
        case ZYDIS_MNEMONIC_PMULUDQ:
            return Mnemonic::Pmuludq;
        case ZYDIS_MNEMONIC_POP:
            return Mnemonic::Pop;
        case ZYDIS_MNEMONIC_POPA:
            return Mnemonic::Popa;
        case ZYDIS_MNEMONIC_POPAD:
            return Mnemonic::Popad;
        case ZYDIS_MNEMONIC_POPCNT:
            return Mnemonic::Popcnt;
        case ZYDIS_MNEMONIC_POPF:
            return Mnemonic::Popf;
        case ZYDIS_MNEMONIC_POPFD:
            return Mnemonic::Popfd;
        case ZYDIS_MNEMONIC_POPFQ:
            return Mnemonic::Popfq;
        case ZYDIS_MNEMONIC_POR:
            return Mnemonic::Por;
        case ZYDIS_MNEMONIC_PREFETCH:
            return Mnemonic::Prefetch;
        case ZYDIS_MNEMONIC_PREFETCHNTA:
            return Mnemonic::Prefetchnta;
        case ZYDIS_MNEMONIC_PREFETCHT0:
            return Mnemonic::Prefetcht0;
        case ZYDIS_MNEMONIC_PREFETCHT1:
            return Mnemonic::Prefetcht1;
        case ZYDIS_MNEMONIC_PREFETCHT2:
            return Mnemonic::Prefetcht2;
        case ZYDIS_MNEMONIC_PREFETCHW:
            return Mnemonic::Prefetchw;
        case ZYDIS_MNEMONIC_PREFETCHWT1:
            return Mnemonic::Prefetchwt1;
        case ZYDIS_MNEMONIC_PSADBW:
            return Mnemonic::Psadbw;
        case ZYDIS_MNEMONIC_PSHUFB:
            return Mnemonic::Pshufb;
        case ZYDIS_MNEMONIC_PSHUFD:
            return Mnemonic::Pshufd;
        case ZYDIS_MNEMONIC_PSHUFHW:
            return Mnemonic::Pshufhw;
        case ZYDIS_MNEMONIC_PSHUFLW:
            return Mnemonic::Pshuflw;
        case ZYDIS_MNEMONIC_PSHUFW:
            return Mnemonic::Pshufw;
        case ZYDIS_MNEMONIC_PSIGNB:
            return Mnemonic::Psignb;
        case ZYDIS_MNEMONIC_PSIGND:
            return Mnemonic::Psignd;
        case ZYDIS_MNEMONIC_PSIGNW:
            return Mnemonic::Psignw;
        case ZYDIS_MNEMONIC_PSLLD:
            return Mnemonic::Pslld;
        case ZYDIS_MNEMONIC_PSLLDQ:
            return Mnemonic::Pslldq;
        case ZYDIS_MNEMONIC_PSLLQ:
            return Mnemonic::Psllq;
        case ZYDIS_MNEMONIC_PSLLW:
            return Mnemonic::Psllw;
        case ZYDIS_MNEMONIC_PSMASH:
            return Mnemonic::Psmash;
        case ZYDIS_MNEMONIC_PSRAD:
            return Mnemonic::Psrad;
        case ZYDIS_MNEMONIC_PSRAW:
            return Mnemonic::Psraw;
        case ZYDIS_MNEMONIC_PSRLD:
            return Mnemonic::Psrld;
        case ZYDIS_MNEMONIC_PSRLDQ:
            return Mnemonic::Psrldq;
        case ZYDIS_MNEMONIC_PSRLQ:
            return Mnemonic::Psrlq;
        case ZYDIS_MNEMONIC_PSRLW:
            return Mnemonic::Psrlw;
        case ZYDIS_MNEMONIC_PSUBB:
            return Mnemonic::Psubb;
        case ZYDIS_MNEMONIC_PSUBD:
            return Mnemonic::Psubd;
        case ZYDIS_MNEMONIC_PSUBQ:
            return Mnemonic::Psubq;
        case ZYDIS_MNEMONIC_PSUBSB:
            return Mnemonic::Psubsb;
        case ZYDIS_MNEMONIC_PSUBSW:
            return Mnemonic::Psubsw;
        case ZYDIS_MNEMONIC_PSUBUSB:
            return Mnemonic::Psubusb;
        case ZYDIS_MNEMONIC_PSUBUSW:
            return Mnemonic::Psubusw;
        case ZYDIS_MNEMONIC_PSUBW:
            return Mnemonic::Psubw;
        case ZYDIS_MNEMONIC_PSWAPD:
            return Mnemonic::Pswapd;
        case ZYDIS_MNEMONIC_PTEST:
            return Mnemonic::Ptest;
        case ZYDIS_MNEMONIC_PTWRITE:
            return Mnemonic::Ptwrite;
        case ZYDIS_MNEMONIC_PUNPCKHBW:
            return Mnemonic::Punpckhbw;
        case ZYDIS_MNEMONIC_PUNPCKHDQ:
            return Mnemonic::Punpckhdq;
        case ZYDIS_MNEMONIC_PUNPCKHQDQ:
            return Mnemonic::Punpckhqdq;
        case ZYDIS_MNEMONIC_PUNPCKHWD:
            return Mnemonic::Punpckhwd;
        case ZYDIS_MNEMONIC_PUNPCKLBW:
            return Mnemonic::Punpcklbw;
        case ZYDIS_MNEMONIC_PUNPCKLDQ:
            return Mnemonic::Punpckldq;
        case ZYDIS_MNEMONIC_PUNPCKLQDQ:
            return Mnemonic::Punpcklqdq;
        case ZYDIS_MNEMONIC_PUNPCKLWD:
            return Mnemonic::Punpcklwd;
        case ZYDIS_MNEMONIC_PUSH:
            return Mnemonic::Push;
        case ZYDIS_MNEMONIC_PUSHA:
            return Mnemonic::Pusha;
        case ZYDIS_MNEMONIC_PUSHAD:
            return Mnemonic::Pushad;
        case ZYDIS_MNEMONIC_PUSHF:
            return Mnemonic::Pushf;
        case ZYDIS_MNEMONIC_PUSHFD:
            return Mnemonic::Pushfd;
        case ZYDIS_MNEMONIC_PUSHFQ:
            return Mnemonic::Pushfq;
        case ZYDIS_MNEMONIC_PVALIDATE:
            return Mnemonic::Pvalidate;
        case ZYDIS_MNEMONIC_PXOR:
            return Mnemonic::Pxor;
        case ZYDIS_MNEMONIC_RCL:
            return Mnemonic::Rcl;
        case ZYDIS_MNEMONIC_RCPPS:
            return Mnemonic::Rcpps;
        case ZYDIS_MNEMONIC_RCPSS:
            return Mnemonic::Rcpss;
        case ZYDIS_MNEMONIC_RCR:
            return Mnemonic::Rcr;
        case ZYDIS_MNEMONIC_RDFSBASE:
            return Mnemonic::Rdfsbase;
        case ZYDIS_MNEMONIC_RDGSBASE:
            return Mnemonic::Rdgsbase;
        case ZYDIS_MNEMONIC_RDMSR:
            return Mnemonic::Rdmsr;
        case ZYDIS_MNEMONIC_RDPID:
            return Mnemonic::Rdpid;
        case ZYDIS_MNEMONIC_RDPKRU:
            return Mnemonic::Rdpkru;
        case ZYDIS_MNEMONIC_RDPMC:
            return Mnemonic::Rdpmc;
        case ZYDIS_MNEMONIC_RDPRU:
            return Mnemonic::Rdpru;
        case ZYDIS_MNEMONIC_RDRAND:
            return Mnemonic::Rdrand;
        case ZYDIS_MNEMONIC_RDSEED:
            return Mnemonic::Rdseed;
        case ZYDIS_MNEMONIC_RDSSPD:
            return Mnemonic::Rdsspd;
        case ZYDIS_MNEMONIC_RDSSPQ:
            return Mnemonic::Rdsspq;
        case ZYDIS_MNEMONIC_RDTSC:
            return Mnemonic::Rdtsc;
        case ZYDIS_MNEMONIC_RDTSCP:
            return Mnemonic::Rdtscp;
        case ZYDIS_MNEMONIC_RET:
            return Mnemonic::Ret;
        case ZYDIS_MNEMONIC_RMPADJUST:
            return Mnemonic::Rmpadjust;
        case ZYDIS_MNEMONIC_RMPUPDATE:
            return Mnemonic::Rmpupdate;
        case ZYDIS_MNEMONIC_ROL:
            return Mnemonic::Rol;
        case ZYDIS_MNEMONIC_ROR:
            return Mnemonic::Ror;
        case ZYDIS_MNEMONIC_RORX:
            return Mnemonic::Rorx;
        case ZYDIS_MNEMONIC_ROUNDPD:
            return Mnemonic::Roundpd;
        case ZYDIS_MNEMONIC_ROUNDPS:
            return Mnemonic::Roundps;
        case ZYDIS_MNEMONIC_ROUNDSD:
            return Mnemonic::Roundsd;
        case ZYDIS_MNEMONIC_ROUNDSS:
            return Mnemonic::Roundss;
        case ZYDIS_MNEMONIC_RSM:
            return Mnemonic::Rsm;
        case ZYDIS_MNEMONIC_RSQRTPS:
            return Mnemonic::Rsqrtps;
        case ZYDIS_MNEMONIC_RSQRTSS:
            return Mnemonic::Rsqrtss;
        case ZYDIS_MNEMONIC_RSTORSSP:
            return Mnemonic::Rstorssp;
        case ZYDIS_MNEMONIC_SAHF:
            return Mnemonic::Sahf;
        case ZYDIS_MNEMONIC_SALC:
            return Mnemonic::Salc;
        case ZYDIS_MNEMONIC_SAR:
            return Mnemonic::Sar;
        case ZYDIS_MNEMONIC_SARX:
            return Mnemonic::Sarx;
        case ZYDIS_MNEMONIC_SAVEPREVSSP:
            return Mnemonic::Saveprevssp;
        case ZYDIS_MNEMONIC_SBB:
            return Mnemonic::Sbb;
        case ZYDIS_MNEMONIC_SCASB:
            return Mnemonic::Scasb;
        case ZYDIS_MNEMONIC_SCASD:
            return Mnemonic::Scasd;
        case ZYDIS_MNEMONIC_SCASQ:
            return Mnemonic::Scasq;
        case ZYDIS_MNEMONIC_SCASW:
            return Mnemonic::Scasw;
        case ZYDIS_MNEMONIC_SEAMCALL:
            return Mnemonic::Seamcall;
        case ZYDIS_MNEMONIC_SEAMOPS:
            return Mnemonic::Seamops;
        case ZYDIS_MNEMONIC_SEAMRET:
            return Mnemonic::Seamret;
        case ZYDIS_MNEMONIC_SENDUIPI:
            return Mnemonic::Senduipi;
        case ZYDIS_MNEMONIC_SERIALIZE:
            return Mnemonic::Serialize;
        case ZYDIS_MNEMONIC_SETB:
            return Mnemonic::Setb;
        case ZYDIS_MNEMONIC_SETBE:
            return Mnemonic::Setbe;
        case ZYDIS_MNEMONIC_SETL:
            return Mnemonic::Setl;
        case ZYDIS_MNEMONIC_SETLE:
            return Mnemonic::Setle;
        case ZYDIS_MNEMONIC_SETNB:
            return Mnemonic::Setnb;
        case ZYDIS_MNEMONIC_SETNBE:
            return Mnemonic::Setnbe;
        case ZYDIS_MNEMONIC_SETNL:
            return Mnemonic::Setnl;
        case ZYDIS_MNEMONIC_SETNLE:
            return Mnemonic::Setnle;
        case ZYDIS_MNEMONIC_SETNO:
            return Mnemonic::Setno;
        case ZYDIS_MNEMONIC_SETNP:
            return Mnemonic::Setnp;
        case ZYDIS_MNEMONIC_SETNS:
            return Mnemonic::Setns;
        case ZYDIS_MNEMONIC_SETNZ:
            return Mnemonic::Setnz;
        case ZYDIS_MNEMONIC_SETO:
            return Mnemonic::Seto;
        case ZYDIS_MNEMONIC_SETP:
            return Mnemonic::Setp;
        case ZYDIS_MNEMONIC_SETS:
            return Mnemonic::Sets;
        case ZYDIS_MNEMONIC_SETSSBSY:
            return Mnemonic::Setssbsy;
        case ZYDIS_MNEMONIC_SETZ:
            return Mnemonic::Setz;
        case ZYDIS_MNEMONIC_SFENCE:
            return Mnemonic::Sfence;
        case ZYDIS_MNEMONIC_SGDT:
            return Mnemonic::Sgdt;
        case ZYDIS_MNEMONIC_SHA1MSG1:
            return Mnemonic::Sha1msg1;
        case ZYDIS_MNEMONIC_SHA1MSG2:
            return Mnemonic::Sha1msg2;
        case ZYDIS_MNEMONIC_SHA1NEXTE:
            return Mnemonic::Sha1nexte;
        case ZYDIS_MNEMONIC_SHA1RNDS4:
            return Mnemonic::Sha1rnds4;
        case ZYDIS_MNEMONIC_SHA256MSG1:
            return Mnemonic::Sha256msg1;
        case ZYDIS_MNEMONIC_SHA256MSG2:
            return Mnemonic::Sha256msg2;
        case ZYDIS_MNEMONIC_SHA256RNDS2:
            return Mnemonic::Sha256rnds2;
        case ZYDIS_MNEMONIC_SHL:
            return Mnemonic::Shl;
        case ZYDIS_MNEMONIC_SHLD:
            return Mnemonic::Shld;
        case ZYDIS_MNEMONIC_SHLX:
            return Mnemonic::Shlx;
        case ZYDIS_MNEMONIC_SHR:
            return Mnemonic::Shr;
        case ZYDIS_MNEMONIC_SHRD:
            return Mnemonic::Shrd;
        case ZYDIS_MNEMONIC_SHRX:
            return Mnemonic::Shrx;
        case ZYDIS_MNEMONIC_SHUFPD:
            return Mnemonic::Shufpd;
        case ZYDIS_MNEMONIC_SHUFPS:
            return Mnemonic::Shufps;
        case ZYDIS_MNEMONIC_SIDT:
            return Mnemonic::Sidt;
        case ZYDIS_MNEMONIC_SKINIT:
            return Mnemonic::Skinit;
        case ZYDIS_MNEMONIC_SLDT:
            return Mnemonic::Sldt;
        case ZYDIS_MNEMONIC_SLWPCB:
            return Mnemonic::Slwpcb;
        case ZYDIS_MNEMONIC_SMSW:
            return Mnemonic::Smsw;
        case ZYDIS_MNEMONIC_SPFLT:
            return Mnemonic::Spflt;
        case ZYDIS_MNEMONIC_SQRTPD:
            return Mnemonic::Sqrtpd;
        case ZYDIS_MNEMONIC_SQRTPS:
            return Mnemonic::Sqrtps;
        case ZYDIS_MNEMONIC_SQRTSD:
            return Mnemonic::Sqrtsd;
        case ZYDIS_MNEMONIC_SQRTSS:
            return Mnemonic::Sqrtss;
        case ZYDIS_MNEMONIC_STAC:
            return Mnemonic::Stac;
        case ZYDIS_MNEMONIC_STC:
            return Mnemonic::Stc;
        case ZYDIS_MNEMONIC_STD:
            return Mnemonic::Std;
        case ZYDIS_MNEMONIC_STGI:
            return Mnemonic::Stgi;
        case ZYDIS_MNEMONIC_STI:
            return Mnemonic::Sti;
        case ZYDIS_MNEMONIC_STMXCSR:
            return Mnemonic::Stmxcsr;
        case ZYDIS_MNEMONIC_STOSB:
            return Mnemonic::Stosb;
        case ZYDIS_MNEMONIC_STOSD:
            return Mnemonic::Stosd;
        case ZYDIS_MNEMONIC_STOSQ:
            return Mnemonic::Stosq;
        case ZYDIS_MNEMONIC_STOSW:
            return Mnemonic::Stosw;
        case ZYDIS_MNEMONIC_STR:
            return Mnemonic::Str;
        case ZYDIS_MNEMONIC_STTILECFG:
            return Mnemonic::Sttilecfg;
        case ZYDIS_MNEMONIC_STUI:
            return Mnemonic::Stui;
        case ZYDIS_MNEMONIC_SUB:
            return Mnemonic::Sub;
        case ZYDIS_MNEMONIC_SUBPD:
            return Mnemonic::Subpd;
        case ZYDIS_MNEMONIC_SUBPS:
            return Mnemonic::Subps;
        case ZYDIS_MNEMONIC_SUBSD:
            return Mnemonic::Subsd;
        case ZYDIS_MNEMONIC_SUBSS:
            return Mnemonic::Subss;
        case ZYDIS_MNEMONIC_SWAPGS:
            return Mnemonic::Swapgs;
        case ZYDIS_MNEMONIC_SYSCALL:
            return Mnemonic::Syscall;
        case ZYDIS_MNEMONIC_SYSENTER:
            return Mnemonic::Sysenter;
        case ZYDIS_MNEMONIC_SYSEXIT:
            return Mnemonic::Sysexit;
        case ZYDIS_MNEMONIC_SYSRET:
            return Mnemonic::Sysret;
        case ZYDIS_MNEMONIC_T1MSKC:
            return Mnemonic::T1mskc;
        case ZYDIS_MNEMONIC_TDCALL:
            return Mnemonic::Tdcall;
        case ZYDIS_MNEMONIC_TDPBF16PS:
            return Mnemonic::Tdpbf16ps;
        case ZYDIS_MNEMONIC_TDPBSSD:
            return Mnemonic::Tdpbssd;
        case ZYDIS_MNEMONIC_TDPBSUD:
            return Mnemonic::Tdpbsud;
        case ZYDIS_MNEMONIC_TDPBUSD:
            return Mnemonic::Tdpbusd;
        case ZYDIS_MNEMONIC_TDPBUUD:
            return Mnemonic::Tdpbuud;
        case ZYDIS_MNEMONIC_TEST:
            return Mnemonic::Test;
        case ZYDIS_MNEMONIC_TESTUI:
            return Mnemonic::Testui;
        case ZYDIS_MNEMONIC_TILELOADD:
            return Mnemonic::Tileloadd;
        case ZYDIS_MNEMONIC_TILELOADDT1:
            return Mnemonic::Tileloaddt1;
        case ZYDIS_MNEMONIC_TILERELEASE:
            return Mnemonic::Tilerelease;
        case ZYDIS_MNEMONIC_TILESTORED:
            return Mnemonic::Tilestored;
        case ZYDIS_MNEMONIC_TILEZERO:
            return Mnemonic::Tilezero;
        case ZYDIS_MNEMONIC_TLBSYNC:
            return Mnemonic::Tlbsync;
        case ZYDIS_MNEMONIC_TPAUSE:
            return Mnemonic::Tpause;
        case ZYDIS_MNEMONIC_TZCNT:
            return Mnemonic::Tzcnt;
        case ZYDIS_MNEMONIC_TZCNTI:
            return Mnemonic::Tzcnti;
        case ZYDIS_MNEMONIC_TZMSK:
            return Mnemonic::Tzmsk;
        case ZYDIS_MNEMONIC_UCOMISD:
            return Mnemonic::Ucomisd;
        case ZYDIS_MNEMONIC_UCOMISS:
            return Mnemonic::Ucomiss;
        case ZYDIS_MNEMONIC_UD0:
            return Mnemonic::Ud0;
        case ZYDIS_MNEMONIC_UD1:
            return Mnemonic::Ud1;
        case ZYDIS_MNEMONIC_UD2:
            return Mnemonic::Ud2;
        case ZYDIS_MNEMONIC_UIRET:
            return Mnemonic::Uiret;
        case ZYDIS_MNEMONIC_UMONITOR:
            return Mnemonic::Umonitor;
        case ZYDIS_MNEMONIC_UMWAIT:
            return Mnemonic::Umwait;
        case ZYDIS_MNEMONIC_UNPCKHPD:
            return Mnemonic::Unpckhpd;
        case ZYDIS_MNEMONIC_UNPCKHPS:
            return Mnemonic::Unpckhps;
        case ZYDIS_MNEMONIC_UNPCKLPD:
            return Mnemonic::Unpcklpd;
        case ZYDIS_MNEMONIC_UNPCKLPS:
            return Mnemonic::Unpcklps;
        case ZYDIS_MNEMONIC_V4FMADDPS:
            return Mnemonic::V4fmaddps;
        case ZYDIS_MNEMONIC_V4FMADDSS:
            return Mnemonic::V4fmaddss;
        case ZYDIS_MNEMONIC_V4FNMADDPS:
            return Mnemonic::V4fnmaddps;
        case ZYDIS_MNEMONIC_V4FNMADDSS:
            return Mnemonic::V4fnmaddss;
        case ZYDIS_MNEMONIC_VADDNPD:
            return Mnemonic::Vaddnpd;
        case ZYDIS_MNEMONIC_VADDNPS:
            return Mnemonic::Vaddnps;
        case ZYDIS_MNEMONIC_VADDPD:
            return Mnemonic::Vaddpd;
        case ZYDIS_MNEMONIC_VADDPH:
            return Mnemonic::Vaddph;
        case ZYDIS_MNEMONIC_VADDPS:
            return Mnemonic::Vaddps;
        case ZYDIS_MNEMONIC_VADDSD:
            return Mnemonic::Vaddsd;
        case ZYDIS_MNEMONIC_VADDSETSPS:
            return Mnemonic::Vaddsetsps;
        case ZYDIS_MNEMONIC_VADDSH:
            return Mnemonic::Vaddsh;
        case ZYDIS_MNEMONIC_VADDSS:
            return Mnemonic::Vaddss;
        case ZYDIS_MNEMONIC_VADDSUBPD:
            return Mnemonic::Vaddsubpd;
        case ZYDIS_MNEMONIC_VADDSUBPS:
            return Mnemonic::Vaddsubps;
        case ZYDIS_MNEMONIC_VAESDEC:
            return Mnemonic::Vaesdec;
        case ZYDIS_MNEMONIC_VAESDECLAST:
            return Mnemonic::Vaesdeclast;
        case ZYDIS_MNEMONIC_VAESENC:
            return Mnemonic::Vaesenc;
        case ZYDIS_MNEMONIC_VAESENCLAST:
            return Mnemonic::Vaesenclast;
        case ZYDIS_MNEMONIC_VAESIMC:
            return Mnemonic::Vaesimc;
        case ZYDIS_MNEMONIC_VAESKEYGENASSIST:
            return Mnemonic::Vaeskeygenassist;
        case ZYDIS_MNEMONIC_VALIGND:
            return Mnemonic::Valignd;
        case ZYDIS_MNEMONIC_VALIGNQ:
            return Mnemonic::Valignq;
        case ZYDIS_MNEMONIC_VANDNPD:
            return Mnemonic::Vandnpd;
        case ZYDIS_MNEMONIC_VANDNPS:
            return Mnemonic::Vandnps;
        case ZYDIS_MNEMONIC_VANDPD:
            return Mnemonic::Vandpd;
        case ZYDIS_MNEMONIC_VANDPS:
            return Mnemonic::Vandps;
        case ZYDIS_MNEMONIC_VBLENDMPD:
            return Mnemonic::Vblendmpd;
        case ZYDIS_MNEMONIC_VBLENDMPS:
            return Mnemonic::Vblendmps;
        case ZYDIS_MNEMONIC_VBLENDPD:
            return Mnemonic::Vblendpd;
        case ZYDIS_MNEMONIC_VBLENDPS:
            return Mnemonic::Vblendps;
        case ZYDIS_MNEMONIC_VBLENDVPD:
            return Mnemonic::Vblendvpd;
        case ZYDIS_MNEMONIC_VBLENDVPS:
            return Mnemonic::Vblendvps;
        case ZYDIS_MNEMONIC_VBROADCASTF128:
            return Mnemonic::Vbroadcastf128;
        case ZYDIS_MNEMONIC_VBROADCASTF32X2:
            return Mnemonic::Vbroadcastf32x2;
        case ZYDIS_MNEMONIC_VBROADCASTF32X4:
            return Mnemonic::Vbroadcastf32x4;
        case ZYDIS_MNEMONIC_VBROADCASTF32X8:
            return Mnemonic::Vbroadcastf32x8;
        case ZYDIS_MNEMONIC_VBROADCASTF64X2:
            return Mnemonic::Vbroadcastf64x2;
        case ZYDIS_MNEMONIC_VBROADCASTF64X4:
            return Mnemonic::Vbroadcastf64x4;
        case ZYDIS_MNEMONIC_VBROADCASTI128:
            return Mnemonic::Vbroadcasti128;
        case ZYDIS_MNEMONIC_VBROADCASTI32X2:
            return Mnemonic::Vbroadcasti32x2;
        case ZYDIS_MNEMONIC_VBROADCASTI32X4:
            return Mnemonic::Vbroadcasti32x4;
        case ZYDIS_MNEMONIC_VBROADCASTI32X8:
            return Mnemonic::Vbroadcasti32x8;
        case ZYDIS_MNEMONIC_VBROADCASTI64X2:
            return Mnemonic::Vbroadcasti64x2;
        case ZYDIS_MNEMONIC_VBROADCASTI64X4:
            return Mnemonic::Vbroadcasti64x4;
        case ZYDIS_MNEMONIC_VBROADCASTSD:
            return Mnemonic::Vbroadcastsd;
        case ZYDIS_MNEMONIC_VBROADCASTSS:
            return Mnemonic::Vbroadcastss;
        case ZYDIS_MNEMONIC_VCMPPD:
            return Mnemonic::Vcmppd;
        case ZYDIS_MNEMONIC_VCMPPH:
            return Mnemonic::Vcmpph;
        case ZYDIS_MNEMONIC_VCMPPS:
            return Mnemonic::Vcmpps;
        case ZYDIS_MNEMONIC_VCMPSD:
            return Mnemonic::Vcmpsd;
        case ZYDIS_MNEMONIC_VCMPSH:
            return Mnemonic::Vcmpsh;
        case ZYDIS_MNEMONIC_VCMPSS:
            return Mnemonic::Vcmpss;
        case ZYDIS_MNEMONIC_VCOMISD:
            return Mnemonic::Vcomisd;
        case ZYDIS_MNEMONIC_VCOMISH:
            return Mnemonic::Vcomish;
        case ZYDIS_MNEMONIC_VCOMISS:
            return Mnemonic::Vcomiss;
        case ZYDIS_MNEMONIC_VCOMPRESSPD:
            return Mnemonic::Vcompresspd;
        case ZYDIS_MNEMONIC_VCOMPRESSPS:
            return Mnemonic::Vcompressps;
        case ZYDIS_MNEMONIC_VCVTDQ2PD:
            return Mnemonic::Vcvtdq2pd;
        case ZYDIS_MNEMONIC_VCVTDQ2PH:
            return Mnemonic::Vcvtdq2ph;
        case ZYDIS_MNEMONIC_VCVTDQ2PS:
            return Mnemonic::Vcvtdq2ps;
        case ZYDIS_MNEMONIC_VCVTFXPNTDQ2PS:
            return Mnemonic::Vcvtfxpntdq2ps;
        case ZYDIS_MNEMONIC_VCVTFXPNTPD2DQ:
            return Mnemonic::Vcvtfxpntpd2dq;
        case ZYDIS_MNEMONIC_VCVTFXPNTPD2UDQ:
            return Mnemonic::Vcvtfxpntpd2udq;
        case ZYDIS_MNEMONIC_VCVTFXPNTPS2DQ:
            return Mnemonic::Vcvtfxpntps2dq;
        case ZYDIS_MNEMONIC_VCVTFXPNTPS2UDQ:
            return Mnemonic::Vcvtfxpntps2udq;
        case ZYDIS_MNEMONIC_VCVTFXPNTUDQ2PS:
            return Mnemonic::Vcvtfxpntudq2ps;
        case ZYDIS_MNEMONIC_VCVTNE2PS2BF16:
            return Mnemonic::Vcvtne2ps2bf16;
        case ZYDIS_MNEMONIC_VCVTNEPS2BF16:
            return Mnemonic::Vcvtneps2bf16;
        case ZYDIS_MNEMONIC_VCVTPD2DQ:
            return Mnemonic::Vcvtpd2dq;
        case ZYDIS_MNEMONIC_VCVTPD2PH:
            return Mnemonic::Vcvtpd2ph;
        case ZYDIS_MNEMONIC_VCVTPD2PS:
            return Mnemonic::Vcvtpd2ps;
        case ZYDIS_MNEMONIC_VCVTPD2QQ:
            return Mnemonic::Vcvtpd2qq;
        case ZYDIS_MNEMONIC_VCVTPD2UDQ:
            return Mnemonic::Vcvtpd2udq;
        case ZYDIS_MNEMONIC_VCVTPD2UQQ:
            return Mnemonic::Vcvtpd2uqq;
        case ZYDIS_MNEMONIC_VCVTPH2DQ:
            return Mnemonic::Vcvtph2dq;
        case ZYDIS_MNEMONIC_VCVTPH2PD:
            return Mnemonic::Vcvtph2pd;
        case ZYDIS_MNEMONIC_VCVTPH2PS:
            return Mnemonic::Vcvtph2ps;
        case ZYDIS_MNEMONIC_VCVTPH2PSX:
            return Mnemonic::Vcvtph2psx;
        case ZYDIS_MNEMONIC_VCVTPH2QQ:
            return Mnemonic::Vcvtph2qq;
        case ZYDIS_MNEMONIC_VCVTPH2UDQ:
            return Mnemonic::Vcvtph2udq;
        case ZYDIS_MNEMONIC_VCVTPH2UQQ:
            return Mnemonic::Vcvtph2uqq;
        case ZYDIS_MNEMONIC_VCVTPH2UW:
            return Mnemonic::Vcvtph2uw;
        case ZYDIS_MNEMONIC_VCVTPH2W:
            return Mnemonic::Vcvtph2w;
        case ZYDIS_MNEMONIC_VCVTPS2DQ:
            return Mnemonic::Vcvtps2dq;
        case ZYDIS_MNEMONIC_VCVTPS2PD:
            return Mnemonic::Vcvtps2pd;
        case ZYDIS_MNEMONIC_VCVTPS2PH:
            return Mnemonic::Vcvtps2ph;
        case ZYDIS_MNEMONIC_VCVTPS2PHX:
            return Mnemonic::Vcvtps2phx;
        case ZYDIS_MNEMONIC_VCVTPS2QQ:
            return Mnemonic::Vcvtps2qq;
        case ZYDIS_MNEMONIC_VCVTPS2UDQ:
            return Mnemonic::Vcvtps2udq;
        case ZYDIS_MNEMONIC_VCVTPS2UQQ:
            return Mnemonic::Vcvtps2uqq;
        case ZYDIS_MNEMONIC_VCVTQQ2PD:
            return Mnemonic::Vcvtqq2pd;
        case ZYDIS_MNEMONIC_VCVTQQ2PH:
            return Mnemonic::Vcvtqq2ph;
        case ZYDIS_MNEMONIC_VCVTQQ2PS:
            return Mnemonic::Vcvtqq2ps;
        case ZYDIS_MNEMONIC_VCVTSD2SH:
            return Mnemonic::Vcvtsd2sh;
        case ZYDIS_MNEMONIC_VCVTSD2SI:
            return Mnemonic::Vcvtsd2si;
        case ZYDIS_MNEMONIC_VCVTSD2SS:
            return Mnemonic::Vcvtsd2ss;
        case ZYDIS_MNEMONIC_VCVTSD2USI:
            return Mnemonic::Vcvtsd2usi;
        case ZYDIS_MNEMONIC_VCVTSH2SD:
            return Mnemonic::Vcvtsh2sd;
        case ZYDIS_MNEMONIC_VCVTSH2SI:
            return Mnemonic::Vcvtsh2si;
        case ZYDIS_MNEMONIC_VCVTSH2SS:
            return Mnemonic::Vcvtsh2ss;
        case ZYDIS_MNEMONIC_VCVTSH2USI:
            return Mnemonic::Vcvtsh2usi;
        case ZYDIS_MNEMONIC_VCVTSI2SD:
            return Mnemonic::Vcvtsi2sd;
        case ZYDIS_MNEMONIC_VCVTSI2SH:
            return Mnemonic::Vcvtsi2sh;
        case ZYDIS_MNEMONIC_VCVTSI2SS:
            return Mnemonic::Vcvtsi2ss;
        case ZYDIS_MNEMONIC_VCVTSS2SD:
            return Mnemonic::Vcvtss2sd;
        case ZYDIS_MNEMONIC_VCVTSS2SH:
            return Mnemonic::Vcvtss2sh;
        case ZYDIS_MNEMONIC_VCVTSS2SI:
            return Mnemonic::Vcvtss2si;
        case ZYDIS_MNEMONIC_VCVTSS2USI:
            return Mnemonic::Vcvtss2usi;
        case ZYDIS_MNEMONIC_VCVTTPD2DQ:
            return Mnemonic::Vcvttpd2dq;
        case ZYDIS_MNEMONIC_VCVTTPD2QQ:
            return Mnemonic::Vcvttpd2qq;
        case ZYDIS_MNEMONIC_VCVTTPD2UDQ:
            return Mnemonic::Vcvttpd2udq;
        case ZYDIS_MNEMONIC_VCVTTPD2UQQ:
            return Mnemonic::Vcvttpd2uqq;
        case ZYDIS_MNEMONIC_VCVTTPH2DQ:
            return Mnemonic::Vcvttph2dq;
        case ZYDIS_MNEMONIC_VCVTTPH2QQ:
            return Mnemonic::Vcvttph2qq;
        case ZYDIS_MNEMONIC_VCVTTPH2UDQ:
            return Mnemonic::Vcvttph2udq;
        case ZYDIS_MNEMONIC_VCVTTPH2UQQ:
            return Mnemonic::Vcvttph2uqq;
        case ZYDIS_MNEMONIC_VCVTTPH2UW:
            return Mnemonic::Vcvttph2uw;
        case ZYDIS_MNEMONIC_VCVTTPH2W:
            return Mnemonic::Vcvttph2w;
        case ZYDIS_MNEMONIC_VCVTTPS2DQ:
            return Mnemonic::Vcvttps2dq;
        case ZYDIS_MNEMONIC_VCVTTPS2QQ:
            return Mnemonic::Vcvttps2qq;
        case ZYDIS_MNEMONIC_VCVTTPS2UDQ:
            return Mnemonic::Vcvttps2udq;
        case ZYDIS_MNEMONIC_VCVTTPS2UQQ:
            return Mnemonic::Vcvttps2uqq;
        case ZYDIS_MNEMONIC_VCVTTSD2SI:
            return Mnemonic::Vcvttsd2si;
        case ZYDIS_MNEMONIC_VCVTTSD2USI:
            return Mnemonic::Vcvttsd2usi;
        case ZYDIS_MNEMONIC_VCVTTSH2SI:
            return Mnemonic::Vcvttsh2si;
        case ZYDIS_MNEMONIC_VCVTTSH2USI:
            return Mnemonic::Vcvttsh2usi;
        case ZYDIS_MNEMONIC_VCVTTSS2SI:
            return Mnemonic::Vcvttss2si;
        case ZYDIS_MNEMONIC_VCVTTSS2USI:
            return Mnemonic::Vcvttss2usi;
        case ZYDIS_MNEMONIC_VCVTUDQ2PD:
            return Mnemonic::Vcvtudq2pd;
        case ZYDIS_MNEMONIC_VCVTUDQ2PH:
            return Mnemonic::Vcvtudq2ph;
        case ZYDIS_MNEMONIC_VCVTUDQ2PS:
            return Mnemonic::Vcvtudq2ps;
        case ZYDIS_MNEMONIC_VCVTUQQ2PD:
            return Mnemonic::Vcvtuqq2pd;
        case ZYDIS_MNEMONIC_VCVTUQQ2PH:
            return Mnemonic::Vcvtuqq2ph;
        case ZYDIS_MNEMONIC_VCVTUQQ2PS:
            return Mnemonic::Vcvtuqq2ps;
        case ZYDIS_MNEMONIC_VCVTUSI2SD:
            return Mnemonic::Vcvtusi2sd;
        case ZYDIS_MNEMONIC_VCVTUSI2SH:
            return Mnemonic::Vcvtusi2sh;
        case ZYDIS_MNEMONIC_VCVTUSI2SS:
            return Mnemonic::Vcvtusi2ss;
        case ZYDIS_MNEMONIC_VCVTUW2PH:
            return Mnemonic::Vcvtuw2ph;
        case ZYDIS_MNEMONIC_VCVTW2PH:
            return Mnemonic::Vcvtw2ph;
        case ZYDIS_MNEMONIC_VDBPSADBW:
            return Mnemonic::Vdbpsadbw;
        case ZYDIS_MNEMONIC_VDIVPD:
            return Mnemonic::Vdivpd;
        case ZYDIS_MNEMONIC_VDIVPH:
            return Mnemonic::Vdivph;
        case ZYDIS_MNEMONIC_VDIVPS:
            return Mnemonic::Vdivps;
        case ZYDIS_MNEMONIC_VDIVSD:
            return Mnemonic::Vdivsd;
        case ZYDIS_MNEMONIC_VDIVSH:
            return Mnemonic::Vdivsh;
        case ZYDIS_MNEMONIC_VDIVSS:
            return Mnemonic::Vdivss;
        case ZYDIS_MNEMONIC_VDPBF16PS:
            return Mnemonic::Vdpbf16ps;
        case ZYDIS_MNEMONIC_VDPPD:
            return Mnemonic::Vdppd;
        case ZYDIS_MNEMONIC_VDPPS:
            return Mnemonic::Vdpps;
        case ZYDIS_MNEMONIC_VERR:
            return Mnemonic::Verr;
        case ZYDIS_MNEMONIC_VERW:
            return Mnemonic::Verw;
        case ZYDIS_MNEMONIC_VEXP223PS:
            return Mnemonic::Vexp223ps;
        case ZYDIS_MNEMONIC_VEXP2PD:
            return Mnemonic::Vexp2pd;
        case ZYDIS_MNEMONIC_VEXP2PS:
            return Mnemonic::Vexp2ps;
        case ZYDIS_MNEMONIC_VEXPANDPD:
            return Mnemonic::Vexpandpd;
        case ZYDIS_MNEMONIC_VEXPANDPS:
            return Mnemonic::Vexpandps;
        case ZYDIS_MNEMONIC_VEXTRACTF128:
            return Mnemonic::Vextractf128;
        case ZYDIS_MNEMONIC_VEXTRACTF32X4:
            return Mnemonic::Vextractf32x4;
        case ZYDIS_MNEMONIC_VEXTRACTF32X8:
            return Mnemonic::Vextractf32x8;
        case ZYDIS_MNEMONIC_VEXTRACTF64X2:
            return Mnemonic::Vextractf64x2;
        case ZYDIS_MNEMONIC_VEXTRACTF64X4:
            return Mnemonic::Vextractf64x4;
        case ZYDIS_MNEMONIC_VEXTRACTI128:
            return Mnemonic::Vextracti128;
        case ZYDIS_MNEMONIC_VEXTRACTI32X4:
            return Mnemonic::Vextracti32x4;
        case ZYDIS_MNEMONIC_VEXTRACTI32X8:
            return Mnemonic::Vextracti32x8;
        case ZYDIS_MNEMONIC_VEXTRACTI64X2:
            return Mnemonic::Vextracti64x2;
        case ZYDIS_MNEMONIC_VEXTRACTI64X4:
            return Mnemonic::Vextracti64x4;
        case ZYDIS_MNEMONIC_VEXTRACTPS:
            return Mnemonic::Vextractps;
        case ZYDIS_MNEMONIC_VFCMADDCPH:
            return Mnemonic::Vfcmaddcph;
        case ZYDIS_MNEMONIC_VFCMADDCSH:
            return Mnemonic::Vfcmaddcsh;
        case ZYDIS_MNEMONIC_VFCMULCPH:
            return Mnemonic::Vfcmulcph;
        case ZYDIS_MNEMONIC_VFCMULCSH:
            return Mnemonic::Vfcmulcsh;
        case ZYDIS_MNEMONIC_VFIXUPIMMPD:
            return Mnemonic::Vfixupimmpd;
        case ZYDIS_MNEMONIC_VFIXUPIMMPS:
            return Mnemonic::Vfixupimmps;
        case ZYDIS_MNEMONIC_VFIXUPIMMSD:
            return Mnemonic::Vfixupimmsd;
        case ZYDIS_MNEMONIC_VFIXUPIMMSS:
            return Mnemonic::Vfixupimmss;
        case ZYDIS_MNEMONIC_VFIXUPNANPD:
            return Mnemonic::Vfixupnanpd;
        case ZYDIS_MNEMONIC_VFIXUPNANPS:
            return Mnemonic::Vfixupnanps;
        case ZYDIS_MNEMONIC_VFMADD132PD:
            return Mnemonic::Vfmadd132pd;
        case ZYDIS_MNEMONIC_VFMADD132PH:
            return Mnemonic::Vfmadd132ph;
        case ZYDIS_MNEMONIC_VFMADD132PS:
            return Mnemonic::Vfmadd132ps;
        case ZYDIS_MNEMONIC_VFMADD132SD:
            return Mnemonic::Vfmadd132sd;
        case ZYDIS_MNEMONIC_VFMADD132SH:
            return Mnemonic::Vfmadd132sh;
        case ZYDIS_MNEMONIC_VFMADD132SS:
            return Mnemonic::Vfmadd132ss;
        case ZYDIS_MNEMONIC_VFMADD213PD:
            return Mnemonic::Vfmadd213pd;
        case ZYDIS_MNEMONIC_VFMADD213PH:
            return Mnemonic::Vfmadd213ph;
        case ZYDIS_MNEMONIC_VFMADD213PS:
            return Mnemonic::Vfmadd213ps;
        case ZYDIS_MNEMONIC_VFMADD213SD:
            return Mnemonic::Vfmadd213sd;
        case ZYDIS_MNEMONIC_VFMADD213SH:
            return Mnemonic::Vfmadd213sh;
        case ZYDIS_MNEMONIC_VFMADD213SS:
            return Mnemonic::Vfmadd213ss;
        case ZYDIS_MNEMONIC_VFMADD231PD:
            return Mnemonic::Vfmadd231pd;
        case ZYDIS_MNEMONIC_VFMADD231PH:
            return Mnemonic::Vfmadd231ph;
        case ZYDIS_MNEMONIC_VFMADD231PS:
            return Mnemonic::Vfmadd231ps;
        case ZYDIS_MNEMONIC_VFMADD231SD:
            return Mnemonic::Vfmadd231sd;
        case ZYDIS_MNEMONIC_VFMADD231SH:
            return Mnemonic::Vfmadd231sh;
        case ZYDIS_MNEMONIC_VFMADD231SS:
            return Mnemonic::Vfmadd231ss;
        case ZYDIS_MNEMONIC_VFMADD233PS:
            return Mnemonic::Vfmadd233ps;
        case ZYDIS_MNEMONIC_VFMADDCPH:
            return Mnemonic::Vfmaddcph;
        case ZYDIS_MNEMONIC_VFMADDCSH:
            return Mnemonic::Vfmaddcsh;
        case ZYDIS_MNEMONIC_VFMADDPD:
            return Mnemonic::Vfmaddpd;
        case ZYDIS_MNEMONIC_VFMADDPS:
            return Mnemonic::Vfmaddps;
        case ZYDIS_MNEMONIC_VFMADDSD:
            return Mnemonic::Vfmaddsd;
        case ZYDIS_MNEMONIC_VFMADDSS:
            return Mnemonic::Vfmaddss;
        case ZYDIS_MNEMONIC_VFMADDSUB132PD:
            return Mnemonic::Vfmaddsub132pd;
        case ZYDIS_MNEMONIC_VFMADDSUB132PH:
            return Mnemonic::Vfmaddsub132ph;
        case ZYDIS_MNEMONIC_VFMADDSUB132PS:
            return Mnemonic::Vfmaddsub132ps;
        case ZYDIS_MNEMONIC_VFMADDSUB213PD:
            return Mnemonic::Vfmaddsub213pd;
        case ZYDIS_MNEMONIC_VFMADDSUB213PH:
            return Mnemonic::Vfmaddsub213ph;
        case ZYDIS_MNEMONIC_VFMADDSUB213PS:
            return Mnemonic::Vfmaddsub213ps;
        case ZYDIS_MNEMONIC_VFMADDSUB231PD:
            return Mnemonic::Vfmaddsub231pd;
        case ZYDIS_MNEMONIC_VFMADDSUB231PH:
            return Mnemonic::Vfmaddsub231ph;
        case ZYDIS_MNEMONIC_VFMADDSUB231PS:
            return Mnemonic::Vfmaddsub231ps;
        case ZYDIS_MNEMONIC_VFMADDSUBPD:
            return Mnemonic::Vfmaddsubpd;
        case ZYDIS_MNEMONIC_VFMADDSUBPS:
            return Mnemonic::Vfmaddsubps;
        case ZYDIS_MNEMONIC_VFMSUB132PD:
            return Mnemonic::Vfmsub132pd;
        case ZYDIS_MNEMONIC_VFMSUB132PH:
            return Mnemonic::Vfmsub132ph;
        case ZYDIS_MNEMONIC_VFMSUB132PS:
            return Mnemonic::Vfmsub132ps;
        case ZYDIS_MNEMONIC_VFMSUB132SD:
            return Mnemonic::Vfmsub132sd;
        case ZYDIS_MNEMONIC_VFMSUB132SH:
            return Mnemonic::Vfmsub132sh;
        case ZYDIS_MNEMONIC_VFMSUB132SS:
            return Mnemonic::Vfmsub132ss;
        case ZYDIS_MNEMONIC_VFMSUB213PD:
            return Mnemonic::Vfmsub213pd;
        case ZYDIS_MNEMONIC_VFMSUB213PH:
            return Mnemonic::Vfmsub213ph;
        case ZYDIS_MNEMONIC_VFMSUB213PS:
            return Mnemonic::Vfmsub213ps;
        case ZYDIS_MNEMONIC_VFMSUB213SD:
            return Mnemonic::Vfmsub213sd;
        case ZYDIS_MNEMONIC_VFMSUB213SH:
            return Mnemonic::Vfmsub213sh;
        case ZYDIS_MNEMONIC_VFMSUB213SS:
            return Mnemonic::Vfmsub213ss;
        case ZYDIS_MNEMONIC_VFMSUB231PD:
            return Mnemonic::Vfmsub231pd;
        case ZYDIS_MNEMONIC_VFMSUB231PH:
            return Mnemonic::Vfmsub231ph;
        case ZYDIS_MNEMONIC_VFMSUB231PS:
            return Mnemonic::Vfmsub231ps;
        case ZYDIS_MNEMONIC_VFMSUB231SD:
            return Mnemonic::Vfmsub231sd;
        case ZYDIS_MNEMONIC_VFMSUB231SH:
            return Mnemonic::Vfmsub231sh;
        case ZYDIS_MNEMONIC_VFMSUB231SS:
            return Mnemonic::Vfmsub231ss;
        case ZYDIS_MNEMONIC_VFMSUBADD132PD:
            return Mnemonic::Vfmsubadd132pd;
        case ZYDIS_MNEMONIC_VFMSUBADD132PH:
            return Mnemonic::Vfmsubadd132ph;
        case ZYDIS_MNEMONIC_VFMSUBADD132PS:
            return Mnemonic::Vfmsubadd132ps;
        case ZYDIS_MNEMONIC_VFMSUBADD213PD:
            return Mnemonic::Vfmsubadd213pd;
        case ZYDIS_MNEMONIC_VFMSUBADD213PH:
            return Mnemonic::Vfmsubadd213ph;
        case ZYDIS_MNEMONIC_VFMSUBADD213PS:
            return Mnemonic::Vfmsubadd213ps;
        case ZYDIS_MNEMONIC_VFMSUBADD231PD:
            return Mnemonic::Vfmsubadd231pd;
        case ZYDIS_MNEMONIC_VFMSUBADD231PH:
            return Mnemonic::Vfmsubadd231ph;
        case ZYDIS_MNEMONIC_VFMSUBADD231PS:
            return Mnemonic::Vfmsubadd231ps;
        case ZYDIS_MNEMONIC_VFMSUBADDPD:
            return Mnemonic::Vfmsubaddpd;
        case ZYDIS_MNEMONIC_VFMSUBADDPS:
            return Mnemonic::Vfmsubaddps;
        case ZYDIS_MNEMONIC_VFMSUBPD:
            return Mnemonic::Vfmsubpd;
        case ZYDIS_MNEMONIC_VFMSUBPS:
            return Mnemonic::Vfmsubps;
        case ZYDIS_MNEMONIC_VFMSUBSD:
            return Mnemonic::Vfmsubsd;
        case ZYDIS_MNEMONIC_VFMSUBSS:
            return Mnemonic::Vfmsubss;
        case ZYDIS_MNEMONIC_VFMULCPH:
            return Mnemonic::Vfmulcph;
        case ZYDIS_MNEMONIC_VFMULCSH:
            return Mnemonic::Vfmulcsh;
        case ZYDIS_MNEMONIC_VFNMADD132PD:
            return Mnemonic::Vfnmadd132pd;
        case ZYDIS_MNEMONIC_VFNMADD132PH:
            return Mnemonic::Vfnmadd132ph;
        case ZYDIS_MNEMONIC_VFNMADD132PS:
            return Mnemonic::Vfnmadd132ps;
        case ZYDIS_MNEMONIC_VFNMADD132SD:
            return Mnemonic::Vfnmadd132sd;
        case ZYDIS_MNEMONIC_VFNMADD132SH:
            return Mnemonic::Vfnmadd132sh;
        case ZYDIS_MNEMONIC_VFNMADD132SS:
            return Mnemonic::Vfnmadd132ss;
        case ZYDIS_MNEMONIC_VFNMADD213PD:
            return Mnemonic::Vfnmadd213pd;
        case ZYDIS_MNEMONIC_VFNMADD213PH:
            return Mnemonic::Vfnmadd213ph;
        case ZYDIS_MNEMONIC_VFNMADD213PS:
            return Mnemonic::Vfnmadd213ps;
        case ZYDIS_MNEMONIC_VFNMADD213SD:
            return Mnemonic::Vfnmadd213sd;
        case ZYDIS_MNEMONIC_VFNMADD213SH:
            return Mnemonic::Vfnmadd213sh;
        case ZYDIS_MNEMONIC_VFNMADD213SS:
            return Mnemonic::Vfnmadd213ss;
        case ZYDIS_MNEMONIC_VFNMADD231PD:
            return Mnemonic::Vfnmadd231pd;
        case ZYDIS_MNEMONIC_VFNMADD231PH:
            return Mnemonic::Vfnmadd231ph;
        case ZYDIS_MNEMONIC_VFNMADD231PS:
            return Mnemonic::Vfnmadd231ps;
        case ZYDIS_MNEMONIC_VFNMADD231SD:
            return Mnemonic::Vfnmadd231sd;
        case ZYDIS_MNEMONIC_VFNMADD231SH:
            return Mnemonic::Vfnmadd231sh;
        case ZYDIS_MNEMONIC_VFNMADD231SS:
            return Mnemonic::Vfnmadd231ss;
        case ZYDIS_MNEMONIC_VFNMADDPD:
            return Mnemonic::Vfnmaddpd;
        case ZYDIS_MNEMONIC_VFNMADDPS:
            return Mnemonic::Vfnmaddps;
        case ZYDIS_MNEMONIC_VFNMADDSD:
            return Mnemonic::Vfnmaddsd;
        case ZYDIS_MNEMONIC_VFNMADDSS:
            return Mnemonic::Vfnmaddss;
        case ZYDIS_MNEMONIC_VFNMSUB132PD:
            return Mnemonic::Vfnmsub132pd;
        case ZYDIS_MNEMONIC_VFNMSUB132PH:
            return Mnemonic::Vfnmsub132ph;
        case ZYDIS_MNEMONIC_VFNMSUB132PS:
            return Mnemonic::Vfnmsub132ps;
        case ZYDIS_MNEMONIC_VFNMSUB132SD:
            return Mnemonic::Vfnmsub132sd;
        case ZYDIS_MNEMONIC_VFNMSUB132SH:
            return Mnemonic::Vfnmsub132sh;
        case ZYDIS_MNEMONIC_VFNMSUB132SS:
            return Mnemonic::Vfnmsub132ss;
        case ZYDIS_MNEMONIC_VFNMSUB213PD:
            return Mnemonic::Vfnmsub213pd;
        case ZYDIS_MNEMONIC_VFNMSUB213PH:
            return Mnemonic::Vfnmsub213ph;
        case ZYDIS_MNEMONIC_VFNMSUB213PS:
            return Mnemonic::Vfnmsub213ps;
        case ZYDIS_MNEMONIC_VFNMSUB213SD:
            return Mnemonic::Vfnmsub213sd;
        case ZYDIS_MNEMONIC_VFNMSUB213SH:
            return Mnemonic::Vfnmsub213sh;
        case ZYDIS_MNEMONIC_VFNMSUB213SS:
            return Mnemonic::Vfnmsub213ss;
        case ZYDIS_MNEMONIC_VFNMSUB231PD:
            return Mnemonic::Vfnmsub231pd;
        case ZYDIS_MNEMONIC_VFNMSUB231PH:
            return Mnemonic::Vfnmsub231ph;
        case ZYDIS_MNEMONIC_VFNMSUB231PS:
            return Mnemonic::Vfnmsub231ps;
        case ZYDIS_MNEMONIC_VFNMSUB231SD:
            return Mnemonic::Vfnmsub231sd;
        case ZYDIS_MNEMONIC_VFNMSUB231SH:
            return Mnemonic::Vfnmsub231sh;
        case ZYDIS_MNEMONIC_VFNMSUB231SS:
            return Mnemonic::Vfnmsub231ss;
        case ZYDIS_MNEMONIC_VFNMSUBPD:
            return Mnemonic::Vfnmsubpd;
        case ZYDIS_MNEMONIC_VFNMSUBPS:
            return Mnemonic::Vfnmsubps;
        case ZYDIS_MNEMONIC_VFNMSUBSD:
            return Mnemonic::Vfnmsubsd;
        case ZYDIS_MNEMONIC_VFNMSUBSS:
            return Mnemonic::Vfnmsubss;
        case ZYDIS_MNEMONIC_VFPCLASSPD:
            return Mnemonic::Vfpclasspd;
        case ZYDIS_MNEMONIC_VFPCLASSPH:
            return Mnemonic::Vfpclassph;
        case ZYDIS_MNEMONIC_VFPCLASSPS:
            return Mnemonic::Vfpclassps;
        case ZYDIS_MNEMONIC_VFPCLASSSD:
            return Mnemonic::Vfpclasssd;
        case ZYDIS_MNEMONIC_VFPCLASSSH:
            return Mnemonic::Vfpclasssh;
        case ZYDIS_MNEMONIC_VFPCLASSSS:
            return Mnemonic::Vfpclassss;
        case ZYDIS_MNEMONIC_VFRCZPD:
            return Mnemonic::Vfrczpd;
        case ZYDIS_MNEMONIC_VFRCZPS:
            return Mnemonic::Vfrczps;
        case ZYDIS_MNEMONIC_VFRCZSD:
            return Mnemonic::Vfrczsd;
        case ZYDIS_MNEMONIC_VFRCZSS:
            return Mnemonic::Vfrczss;
        case ZYDIS_MNEMONIC_VGATHERDPD:
            return Mnemonic::Vgatherdpd;
        case ZYDIS_MNEMONIC_VGATHERDPS:
            return Mnemonic::Vgatherdps;
        case ZYDIS_MNEMONIC_VGATHERPF0DPD:
            return Mnemonic::Vgatherpf0dpd;
        case ZYDIS_MNEMONIC_VGATHERPF0DPS:
            return Mnemonic::Vgatherpf0dps;
        case ZYDIS_MNEMONIC_VGATHERPF0HINTDPD:
            return Mnemonic::Vgatherpf0hintdpd;
        case ZYDIS_MNEMONIC_VGATHERPF0HINTDPS:
            return Mnemonic::Vgatherpf0hintdps;
        case ZYDIS_MNEMONIC_VGATHERPF0QPD:
            return Mnemonic::Vgatherpf0qpd;
        case ZYDIS_MNEMONIC_VGATHERPF0QPS:
            return Mnemonic::Vgatherpf0qps;
        case ZYDIS_MNEMONIC_VGATHERPF1DPD:
            return Mnemonic::Vgatherpf1dpd;
        case ZYDIS_MNEMONIC_VGATHERPF1DPS:
            return Mnemonic::Vgatherpf1dps;
        case ZYDIS_MNEMONIC_VGATHERPF1QPD:
            return Mnemonic::Vgatherpf1qpd;
        case ZYDIS_MNEMONIC_VGATHERPF1QPS:
            return Mnemonic::Vgatherpf1qps;
        case ZYDIS_MNEMONIC_VGATHERQPD:
            return Mnemonic::Vgatherqpd;
        case ZYDIS_MNEMONIC_VGATHERQPS:
            return Mnemonic::Vgatherqps;
        case ZYDIS_MNEMONIC_VGETEXPPD:
            return Mnemonic::Vgetexppd;
        case ZYDIS_MNEMONIC_VGETEXPPH:
            return Mnemonic::Vgetexpph;
        case ZYDIS_MNEMONIC_VGETEXPPS:
            return Mnemonic::Vgetexpps;
        case ZYDIS_MNEMONIC_VGETEXPSD:
            return Mnemonic::Vgetexpsd;
        case ZYDIS_MNEMONIC_VGETEXPSH:
            return Mnemonic::Vgetexpsh;
        case ZYDIS_MNEMONIC_VGETEXPSS:
            return Mnemonic::Vgetexpss;
        case ZYDIS_MNEMONIC_VGETMANTPD:
            return Mnemonic::Vgetmantpd;
        case ZYDIS_MNEMONIC_VGETMANTPH:
            return Mnemonic::Vgetmantph;
        case ZYDIS_MNEMONIC_VGETMANTPS:
            return Mnemonic::Vgetmantps;
        case ZYDIS_MNEMONIC_VGETMANTSD:
            return Mnemonic::Vgetmantsd;
        case ZYDIS_MNEMONIC_VGETMANTSH:
            return Mnemonic::Vgetmantsh;
        case ZYDIS_MNEMONIC_VGETMANTSS:
            return Mnemonic::Vgetmantss;
        case ZYDIS_MNEMONIC_VGF2P8AFFINEINVQB:
            return Mnemonic::Vgf2p8affineinvqb;
        case ZYDIS_MNEMONIC_VGF2P8AFFINEQB:
            return Mnemonic::Vgf2p8affineqb;
        case ZYDIS_MNEMONIC_VGF2P8MULB:
            return Mnemonic::Vgf2p8mulb;
        case ZYDIS_MNEMONIC_VGMAXABSPS:
            return Mnemonic::Vgmaxabsps;
        case ZYDIS_MNEMONIC_VGMAXPD:
            return Mnemonic::Vgmaxpd;
        case ZYDIS_MNEMONIC_VGMAXPS:
            return Mnemonic::Vgmaxps;
        case ZYDIS_MNEMONIC_VGMINPD:
            return Mnemonic::Vgminpd;
        case ZYDIS_MNEMONIC_VGMINPS:
            return Mnemonic::Vgminps;
        case ZYDIS_MNEMONIC_VHADDPD:
            return Mnemonic::Vhaddpd;
        case ZYDIS_MNEMONIC_VHADDPS:
            return Mnemonic::Vhaddps;
        case ZYDIS_MNEMONIC_VHSUBPD:
            return Mnemonic::Vhsubpd;
        case ZYDIS_MNEMONIC_VHSUBPS:
            return Mnemonic::Vhsubps;
        case ZYDIS_MNEMONIC_VINSERTF128:
            return Mnemonic::Vinsertf128;
        case ZYDIS_MNEMONIC_VINSERTF32X4:
            return Mnemonic::Vinsertf32x4;
        case ZYDIS_MNEMONIC_VINSERTF32X8:
            return Mnemonic::Vinsertf32x8;
        case ZYDIS_MNEMONIC_VINSERTF64X2:
            return Mnemonic::Vinsertf64x2;
        case ZYDIS_MNEMONIC_VINSERTF64X4:
            return Mnemonic::Vinsertf64x4;
        case ZYDIS_MNEMONIC_VINSERTI128:
            return Mnemonic::Vinserti128;
        case ZYDIS_MNEMONIC_VINSERTI32X4:
            return Mnemonic::Vinserti32x4;
        case ZYDIS_MNEMONIC_VINSERTI32X8:
            return Mnemonic::Vinserti32x8;
        case ZYDIS_MNEMONIC_VINSERTI64X2:
            return Mnemonic::Vinserti64x2;
        case ZYDIS_MNEMONIC_VINSERTI64X4:
            return Mnemonic::Vinserti64x4;
        case ZYDIS_MNEMONIC_VINSERTPS:
            return Mnemonic::Vinsertps;
        case ZYDIS_MNEMONIC_VLDDQU:
            return Mnemonic::Vlddqu;
        case ZYDIS_MNEMONIC_VLDMXCSR:
            return Mnemonic::Vldmxcsr;
        case ZYDIS_MNEMONIC_VLOADUNPACKHD:
            return Mnemonic::Vloadunpackhd;
        case ZYDIS_MNEMONIC_VLOADUNPACKHPD:
            return Mnemonic::Vloadunpackhpd;
        case ZYDIS_MNEMONIC_VLOADUNPACKHPS:
            return Mnemonic::Vloadunpackhps;
        case ZYDIS_MNEMONIC_VLOADUNPACKHQ:
            return Mnemonic::Vloadunpackhq;
        case ZYDIS_MNEMONIC_VLOADUNPACKLD:
            return Mnemonic::Vloadunpackld;
        case ZYDIS_MNEMONIC_VLOADUNPACKLPD:
            return Mnemonic::Vloadunpacklpd;
        case ZYDIS_MNEMONIC_VLOADUNPACKLPS:
            return Mnemonic::Vloadunpacklps;
        case ZYDIS_MNEMONIC_VLOADUNPACKLQ:
            return Mnemonic::Vloadunpacklq;
        case ZYDIS_MNEMONIC_VLOG2PS:
            return Mnemonic::Vlog2ps;
        case ZYDIS_MNEMONIC_VMASKMOVDQU:
            return Mnemonic::Vmaskmovdqu;
        case ZYDIS_MNEMONIC_VMASKMOVPD:
            return Mnemonic::Vmaskmovpd;
        case ZYDIS_MNEMONIC_VMASKMOVPS:
            return Mnemonic::Vmaskmovps;
        case ZYDIS_MNEMONIC_VMAXPD:
            return Mnemonic::Vmaxpd;
        case ZYDIS_MNEMONIC_VMAXPH:
            return Mnemonic::Vmaxph;
        case ZYDIS_MNEMONIC_VMAXPS:
            return Mnemonic::Vmaxps;
        case ZYDIS_MNEMONIC_VMAXSD:
            return Mnemonic::Vmaxsd;
        case ZYDIS_MNEMONIC_VMAXSH:
            return Mnemonic::Vmaxsh;
        case ZYDIS_MNEMONIC_VMAXSS:
            return Mnemonic::Vmaxss;
        case ZYDIS_MNEMONIC_VMCALL:
            return Mnemonic::Vmcall;
        case ZYDIS_MNEMONIC_VMCLEAR:
            return Mnemonic::Vmclear;
        case ZYDIS_MNEMONIC_VMFUNC:
            return Mnemonic::Vmfunc;
        case ZYDIS_MNEMONIC_VMINPD:
            return Mnemonic::Vminpd;
        case ZYDIS_MNEMONIC_VMINPH:
            return Mnemonic::Vminph;
        case ZYDIS_MNEMONIC_VMINPS:
            return Mnemonic::Vminps;
        case ZYDIS_MNEMONIC_VMINSD:
            return Mnemonic::Vminsd;
        case ZYDIS_MNEMONIC_VMINSH:
            return Mnemonic::Vminsh;
        case ZYDIS_MNEMONIC_VMINSS:
            return Mnemonic::Vminss;
        case ZYDIS_MNEMONIC_VMLAUNCH:
            return Mnemonic::Vmlaunch;
        case ZYDIS_MNEMONIC_VMLOAD:
            return Mnemonic::Vmload;
        case ZYDIS_MNEMONIC_VMMCALL:
            return Mnemonic::Vmmcall;
        case ZYDIS_MNEMONIC_VMOVAPD:
            return Mnemonic::Vmovapd;
        case ZYDIS_MNEMONIC_VMOVAPS:
            return Mnemonic::Vmovaps;
        case ZYDIS_MNEMONIC_VMOVD:
            return Mnemonic::Vmovd;
        case ZYDIS_MNEMONIC_VMOVDDUP:
            return Mnemonic::Vmovddup;
        case ZYDIS_MNEMONIC_VMOVDQA:
            return Mnemonic::Vmovdqa;
        case ZYDIS_MNEMONIC_VMOVDQA32:
            return Mnemonic::Vmovdqa32;
        case ZYDIS_MNEMONIC_VMOVDQA64:
            return Mnemonic::Vmovdqa64;
        case ZYDIS_MNEMONIC_VMOVDQU:
            return Mnemonic::Vmovdqu;
        case ZYDIS_MNEMONIC_VMOVDQU16:
            return Mnemonic::Vmovdqu16;
        case ZYDIS_MNEMONIC_VMOVDQU32:
            return Mnemonic::Vmovdqu32;
        case ZYDIS_MNEMONIC_VMOVDQU64:
            return Mnemonic::Vmovdqu64;
        case ZYDIS_MNEMONIC_VMOVDQU8:
            return Mnemonic::Vmovdqu8;
        case ZYDIS_MNEMONIC_VMOVHLPS:
            return Mnemonic::Vmovhlps;
        case ZYDIS_MNEMONIC_VMOVHPD:
            return Mnemonic::Vmovhpd;
        case ZYDIS_MNEMONIC_VMOVHPS:
            return Mnemonic::Vmovhps;
        case ZYDIS_MNEMONIC_VMOVLHPS:
            return Mnemonic::Vmovlhps;
        case ZYDIS_MNEMONIC_VMOVLPD:
            return Mnemonic::Vmovlpd;
        case ZYDIS_MNEMONIC_VMOVLPS:
            return Mnemonic::Vmovlps;
        case ZYDIS_MNEMONIC_VMOVMSKPD:
            return Mnemonic::Vmovmskpd;
        case ZYDIS_MNEMONIC_VMOVMSKPS:
            return Mnemonic::Vmovmskps;
        case ZYDIS_MNEMONIC_VMOVNRAPD:
            return Mnemonic::Vmovnrapd;
        case ZYDIS_MNEMONIC_VMOVNRAPS:
            return Mnemonic::Vmovnraps;
        case ZYDIS_MNEMONIC_VMOVNRNGOAPD:
            return Mnemonic::Vmovnrngoapd;
        case ZYDIS_MNEMONIC_VMOVNRNGOAPS:
            return Mnemonic::Vmovnrngoaps;
        case ZYDIS_MNEMONIC_VMOVNTDQ:
            return Mnemonic::Vmovntdq;
        case ZYDIS_MNEMONIC_VMOVNTDQA:
            return Mnemonic::Vmovntdqa;
        case ZYDIS_MNEMONIC_VMOVNTPD:
            return Mnemonic::Vmovntpd;
        case ZYDIS_MNEMONIC_VMOVNTPS:
            return Mnemonic::Vmovntps;
        case ZYDIS_MNEMONIC_VMOVQ:
            return Mnemonic::Vmovq;
        case ZYDIS_MNEMONIC_VMOVSD:
            return Mnemonic::Vmovsd;
        case ZYDIS_MNEMONIC_VMOVSH:
            return Mnemonic::Vmovsh;
        case ZYDIS_MNEMONIC_VMOVSHDUP:
            return Mnemonic::Vmovshdup;
        case ZYDIS_MNEMONIC_VMOVSLDUP:
            return Mnemonic::Vmovsldup;
        case ZYDIS_MNEMONIC_VMOVSS:
            return Mnemonic::Vmovss;
        case ZYDIS_MNEMONIC_VMOVUPD:
            return Mnemonic::Vmovupd;
        case ZYDIS_MNEMONIC_VMOVUPS:
            return Mnemonic::Vmovups;
        case ZYDIS_MNEMONIC_VMOVW:
            return Mnemonic::Vmovw;
        case ZYDIS_MNEMONIC_VMPSADBW:
            return Mnemonic::Vmpsadbw;
        case ZYDIS_MNEMONIC_VMPTRLD:
            return Mnemonic::Vmptrld;
        case ZYDIS_MNEMONIC_VMPTRST:
            return Mnemonic::Vmptrst;
        case ZYDIS_MNEMONIC_VMREAD:
            return Mnemonic::Vmread;
        case ZYDIS_MNEMONIC_VMRESUME:
            return Mnemonic::Vmresume;
        case ZYDIS_MNEMONIC_VMRUN:
            return Mnemonic::Vmrun;
        case ZYDIS_MNEMONIC_VMSAVE:
            return Mnemonic::Vmsave;
        case ZYDIS_MNEMONIC_VMULPD:
            return Mnemonic::Vmulpd;
        case ZYDIS_MNEMONIC_VMULPH:
            return Mnemonic::Vmulph;
        case ZYDIS_MNEMONIC_VMULPS:
            return Mnemonic::Vmulps;
        case ZYDIS_MNEMONIC_VMULSD:
            return Mnemonic::Vmulsd;
        case ZYDIS_MNEMONIC_VMULSH:
            return Mnemonic::Vmulsh;
        case ZYDIS_MNEMONIC_VMULSS:
            return Mnemonic::Vmulss;
        case ZYDIS_MNEMONIC_VMWRITE:
            return Mnemonic::Vmwrite;
        case ZYDIS_MNEMONIC_VMXOFF:
            return Mnemonic::Vmxoff;
        case ZYDIS_MNEMONIC_VMXON:
            return Mnemonic::Vmxon;
        case ZYDIS_MNEMONIC_VORPD:
            return Mnemonic::Vorpd;
        case ZYDIS_MNEMONIC_VORPS:
            return Mnemonic::Vorps;
        case ZYDIS_MNEMONIC_VP2INTERSECTD:
            return Mnemonic::Vp2intersectd;
        case ZYDIS_MNEMONIC_VP2INTERSECTQ:
            return Mnemonic::Vp2intersectq;
        case ZYDIS_MNEMONIC_VP4DPWSSD:
            return Mnemonic::Vp4dpwssd;
        case ZYDIS_MNEMONIC_VP4DPWSSDS:
            return Mnemonic::Vp4dpwssds;
        case ZYDIS_MNEMONIC_VPABSB:
            return Mnemonic::Vpabsb;
        case ZYDIS_MNEMONIC_VPABSD:
            return Mnemonic::Vpabsd;
        case ZYDIS_MNEMONIC_VPABSQ:
            return Mnemonic::Vpabsq;
        case ZYDIS_MNEMONIC_VPABSW:
            return Mnemonic::Vpabsw;
        case ZYDIS_MNEMONIC_VPACKSSDW:
            return Mnemonic::Vpackssdw;
        case ZYDIS_MNEMONIC_VPACKSSWB:
            return Mnemonic::Vpacksswb;
        case ZYDIS_MNEMONIC_VPACKSTOREHD:
            return Mnemonic::Vpackstorehd;
        case ZYDIS_MNEMONIC_VPACKSTOREHPD:
            return Mnemonic::Vpackstorehpd;
        case ZYDIS_MNEMONIC_VPACKSTOREHPS:
            return Mnemonic::Vpackstorehps;
        case ZYDIS_MNEMONIC_VPACKSTOREHQ:
            return Mnemonic::Vpackstorehq;
        case ZYDIS_MNEMONIC_VPACKSTORELD:
            return Mnemonic::Vpackstoreld;
        case ZYDIS_MNEMONIC_VPACKSTORELPD:
            return Mnemonic::Vpackstorelpd;
        case ZYDIS_MNEMONIC_VPACKSTORELPS:
            return Mnemonic::Vpackstorelps;
        case ZYDIS_MNEMONIC_VPACKSTORELQ:
            return Mnemonic::Vpackstorelq;
        case ZYDIS_MNEMONIC_VPACKUSDW:
            return Mnemonic::Vpackusdw;
        case ZYDIS_MNEMONIC_VPACKUSWB:
            return Mnemonic::Vpackuswb;
        case ZYDIS_MNEMONIC_VPADCD:
            return Mnemonic::Vpadcd;
        case ZYDIS_MNEMONIC_VPADDB:
            return Mnemonic::Vpaddb;
        case ZYDIS_MNEMONIC_VPADDD:
            return Mnemonic::Vpaddd;
        case ZYDIS_MNEMONIC_VPADDQ:
            return Mnemonic::Vpaddq;
        case ZYDIS_MNEMONIC_VPADDSB:
            return Mnemonic::Vpaddsb;
        case ZYDIS_MNEMONIC_VPADDSETCD:
            return Mnemonic::Vpaddsetcd;
        case ZYDIS_MNEMONIC_VPADDSETSD:
            return Mnemonic::Vpaddsetsd;
        case ZYDIS_MNEMONIC_VPADDSW:
            return Mnemonic::Vpaddsw;
        case ZYDIS_MNEMONIC_VPADDUSB:
            return Mnemonic::Vpaddusb;
        case ZYDIS_MNEMONIC_VPADDUSW:
            return Mnemonic::Vpaddusw;
        case ZYDIS_MNEMONIC_VPADDW:
            return Mnemonic::Vpaddw;
        case ZYDIS_MNEMONIC_VPALIGNR:
            return Mnemonic::Vpalignr;
        case ZYDIS_MNEMONIC_VPAND:
            return Mnemonic::Vpand;
        case ZYDIS_MNEMONIC_VPANDD:
            return Mnemonic::Vpandd;
        case ZYDIS_MNEMONIC_VPANDN:
            return Mnemonic::Vpandn;
        case ZYDIS_MNEMONIC_VPANDND:
            return Mnemonic::Vpandnd;
        case ZYDIS_MNEMONIC_VPANDNQ:
            return Mnemonic::Vpandnq;
        case ZYDIS_MNEMONIC_VPANDQ:
            return Mnemonic::Vpandq;
        case ZYDIS_MNEMONIC_VPAVGB:
            return Mnemonic::Vpavgb;
        case ZYDIS_MNEMONIC_VPAVGW:
            return Mnemonic::Vpavgw;
        case ZYDIS_MNEMONIC_VPBLENDD:
            return Mnemonic::Vpblendd;
        case ZYDIS_MNEMONIC_VPBLENDMB:
            return Mnemonic::Vpblendmb;
        case ZYDIS_MNEMONIC_VPBLENDMD:
            return Mnemonic::Vpblendmd;
        case ZYDIS_MNEMONIC_VPBLENDMQ:
            return Mnemonic::Vpblendmq;
        case ZYDIS_MNEMONIC_VPBLENDMW:
            return Mnemonic::Vpblendmw;
        case ZYDIS_MNEMONIC_VPBLENDVB:
            return Mnemonic::Vpblendvb;
        case ZYDIS_MNEMONIC_VPBLENDW:
            return Mnemonic::Vpblendw;
        case ZYDIS_MNEMONIC_VPBROADCASTB:
            return Mnemonic::Vpbroadcastb;
        case ZYDIS_MNEMONIC_VPBROADCASTD:
            return Mnemonic::Vpbroadcastd;
        case ZYDIS_MNEMONIC_VPBROADCASTMB2Q:
            return Mnemonic::Vpbroadcastmb2q;
        case ZYDIS_MNEMONIC_VPBROADCASTMW2D:
            return Mnemonic::Vpbroadcastmw2d;
        case ZYDIS_MNEMONIC_VPBROADCASTQ:
            return Mnemonic::Vpbroadcastq;
        case ZYDIS_MNEMONIC_VPBROADCASTW:
            return Mnemonic::Vpbroadcastw;
        case ZYDIS_MNEMONIC_VPCLMULQDQ:
            return Mnemonic::Vpclmulqdq;
        case ZYDIS_MNEMONIC_VPCMOV:
            return Mnemonic::Vpcmov;
        case ZYDIS_MNEMONIC_VPCMPB:
            return Mnemonic::Vpcmpb;
        case ZYDIS_MNEMONIC_VPCMPD:
            return Mnemonic::Vpcmpd;
        case ZYDIS_MNEMONIC_VPCMPEQB:
            return Mnemonic::Vpcmpeqb;
        case ZYDIS_MNEMONIC_VPCMPEQD:
            return Mnemonic::Vpcmpeqd;
        case ZYDIS_MNEMONIC_VPCMPEQQ:
            return Mnemonic::Vpcmpeqq;
        case ZYDIS_MNEMONIC_VPCMPEQW:
            return Mnemonic::Vpcmpeqw;
        case ZYDIS_MNEMONIC_VPCMPESTRI:
            return Mnemonic::Vpcmpestri;
        case ZYDIS_MNEMONIC_VPCMPESTRM:
            return Mnemonic::Vpcmpestrm;
        case ZYDIS_MNEMONIC_VPCMPGTB:
            return Mnemonic::Vpcmpgtb;
        case ZYDIS_MNEMONIC_VPCMPGTD:
            return Mnemonic::Vpcmpgtd;
        case ZYDIS_MNEMONIC_VPCMPGTQ:
            return Mnemonic::Vpcmpgtq;
        case ZYDIS_MNEMONIC_VPCMPGTW:
            return Mnemonic::Vpcmpgtw;
        case ZYDIS_MNEMONIC_VPCMPISTRI:
            return Mnemonic::Vpcmpistri;
        case ZYDIS_MNEMONIC_VPCMPISTRM:
            return Mnemonic::Vpcmpistrm;
        case ZYDIS_MNEMONIC_VPCMPLTD:
            return Mnemonic::Vpcmpltd;
        case ZYDIS_MNEMONIC_VPCMPQ:
            return Mnemonic::Vpcmpq;
        case ZYDIS_MNEMONIC_VPCMPUB:
            return Mnemonic::Vpcmpub;
        case ZYDIS_MNEMONIC_VPCMPUD:
            return Mnemonic::Vpcmpud;
        case ZYDIS_MNEMONIC_VPCMPUQ:
            return Mnemonic::Vpcmpuq;
        case ZYDIS_MNEMONIC_VPCMPUW:
            return Mnemonic::Vpcmpuw;
        case ZYDIS_MNEMONIC_VPCMPW:
            return Mnemonic::Vpcmpw;
        case ZYDIS_MNEMONIC_VPCOMB:
            return Mnemonic::Vpcomb;
        case ZYDIS_MNEMONIC_VPCOMD:
            return Mnemonic::Vpcomd;
        case ZYDIS_MNEMONIC_VPCOMPRESSB:
            return Mnemonic::Vpcompressb;
        case ZYDIS_MNEMONIC_VPCOMPRESSD:
            return Mnemonic::Vpcompressd;
        case ZYDIS_MNEMONIC_VPCOMPRESSQ:
            return Mnemonic::Vpcompressq;
        case ZYDIS_MNEMONIC_VPCOMPRESSW:
            return Mnemonic::Vpcompressw;
        case ZYDIS_MNEMONIC_VPCOMQ:
            return Mnemonic::Vpcomq;
        case ZYDIS_MNEMONIC_VPCOMUB:
            return Mnemonic::Vpcomub;
        case ZYDIS_MNEMONIC_VPCOMUD:
            return Mnemonic::Vpcomud;
        case ZYDIS_MNEMONIC_VPCOMUQ:
            return Mnemonic::Vpcomuq;
        case ZYDIS_MNEMONIC_VPCOMUW:
            return Mnemonic::Vpcomuw;
        case ZYDIS_MNEMONIC_VPCOMW:
            return Mnemonic::Vpcomw;
        case ZYDIS_MNEMONIC_VPCONFLICTD:
            return Mnemonic::Vpconflictd;
        case ZYDIS_MNEMONIC_VPCONFLICTQ:
            return Mnemonic::Vpconflictq;
        case ZYDIS_MNEMONIC_VPDPBUSD:
            return Mnemonic::Vpdpbusd;
        case ZYDIS_MNEMONIC_VPDPBUSDS:
            return Mnemonic::Vpdpbusds;
        case ZYDIS_MNEMONIC_VPDPWSSD:
            return Mnemonic::Vpdpwssd;
        case ZYDIS_MNEMONIC_VPDPWSSDS:
            return Mnemonic::Vpdpwssds;
        case ZYDIS_MNEMONIC_VPERM2F128:
            return Mnemonic::Vperm2f128;
        case ZYDIS_MNEMONIC_VPERM2I128:
            return Mnemonic::Vperm2i128;
        case ZYDIS_MNEMONIC_VPERMB:
            return Mnemonic::Vpermb;
        case ZYDIS_MNEMONIC_VPERMD:
            return Mnemonic::Vpermd;
        case ZYDIS_MNEMONIC_VPERMF32X4:
            return Mnemonic::Vpermf32x4;
        case ZYDIS_MNEMONIC_VPERMI2B:
            return Mnemonic::Vpermi2b;
        case ZYDIS_MNEMONIC_VPERMI2D:
            return Mnemonic::Vpermi2d;
        case ZYDIS_MNEMONIC_VPERMI2PD:
            return Mnemonic::Vpermi2pd;
        case ZYDIS_MNEMONIC_VPERMI2PS:
            return Mnemonic::Vpermi2ps;
        case ZYDIS_MNEMONIC_VPERMI2Q:
            return Mnemonic::Vpermi2q;
        case ZYDIS_MNEMONIC_VPERMI2W:
            return Mnemonic::Vpermi2w;
        case ZYDIS_MNEMONIC_VPERMIL2PD:
            return Mnemonic::Vpermil2pd;
        case ZYDIS_MNEMONIC_VPERMIL2PS:
            return Mnemonic::Vpermil2ps;
        case ZYDIS_MNEMONIC_VPERMILPD:
            return Mnemonic::Vpermilpd;
        case ZYDIS_MNEMONIC_VPERMILPS:
            return Mnemonic::Vpermilps;
        case ZYDIS_MNEMONIC_VPERMPD:
            return Mnemonic::Vpermpd;
        case ZYDIS_MNEMONIC_VPERMPS:
            return Mnemonic::Vpermps;
        case ZYDIS_MNEMONIC_VPERMQ:
            return Mnemonic::Vpermq;
        case ZYDIS_MNEMONIC_VPERMT2B:
            return Mnemonic::Vpermt2b;
        case ZYDIS_MNEMONIC_VPERMT2D:
            return Mnemonic::Vpermt2d;
        case ZYDIS_MNEMONIC_VPERMT2PD:
            return Mnemonic::Vpermt2pd;
        case ZYDIS_MNEMONIC_VPERMT2PS:
            return Mnemonic::Vpermt2ps;
        case ZYDIS_MNEMONIC_VPERMT2Q:
            return Mnemonic::Vpermt2q;
        case ZYDIS_MNEMONIC_VPERMT2W:
            return Mnemonic::Vpermt2w;
        case ZYDIS_MNEMONIC_VPERMW:
            return Mnemonic::Vpermw;
        case ZYDIS_MNEMONIC_VPEXPANDB:
            return Mnemonic::Vpexpandb;
        case ZYDIS_MNEMONIC_VPEXPANDD:
            return Mnemonic::Vpexpandd;
        case ZYDIS_MNEMONIC_VPEXPANDQ:
            return Mnemonic::Vpexpandq;
        case ZYDIS_MNEMONIC_VPEXPANDW:
            return Mnemonic::Vpexpandw;
        case ZYDIS_MNEMONIC_VPEXTRB:
            return Mnemonic::Vpextrb;
        case ZYDIS_MNEMONIC_VPEXTRD:
            return Mnemonic::Vpextrd;
        case ZYDIS_MNEMONIC_VPEXTRQ:
            return Mnemonic::Vpextrq;
        case ZYDIS_MNEMONIC_VPEXTRW:
            return Mnemonic::Vpextrw;
        case ZYDIS_MNEMONIC_VPGATHERDD:
            return Mnemonic::Vpgatherdd;
        case ZYDIS_MNEMONIC_VPGATHERDQ:
            return Mnemonic::Vpgatherdq;
        case ZYDIS_MNEMONIC_VPGATHERQD:
            return Mnemonic::Vpgatherqd;
        case ZYDIS_MNEMONIC_VPGATHERQQ:
            return Mnemonic::Vpgatherqq;
        case ZYDIS_MNEMONIC_VPHADDBD:
            return Mnemonic::Vphaddbd;
        case ZYDIS_MNEMONIC_VPHADDBQ:
            return Mnemonic::Vphaddbq;
        case ZYDIS_MNEMONIC_VPHADDBW:
            return Mnemonic::Vphaddbw;
        case ZYDIS_MNEMONIC_VPHADDD:
            return Mnemonic::Vphaddd;
        case ZYDIS_MNEMONIC_VPHADDDQ:
            return Mnemonic::Vphadddq;
        case ZYDIS_MNEMONIC_VPHADDSW:
            return Mnemonic::Vphaddsw;
        case ZYDIS_MNEMONIC_VPHADDUBD:
            return Mnemonic::Vphaddubd;
        case ZYDIS_MNEMONIC_VPHADDUBQ:
            return Mnemonic::Vphaddubq;
        case ZYDIS_MNEMONIC_VPHADDUBW:
            return Mnemonic::Vphaddubw;
        case ZYDIS_MNEMONIC_VPHADDUDQ:
            return Mnemonic::Vphaddudq;
        case ZYDIS_MNEMONIC_VPHADDUWD:
            return Mnemonic::Vphadduwd;
        case ZYDIS_MNEMONIC_VPHADDUWQ:
            return Mnemonic::Vphadduwq;
        case ZYDIS_MNEMONIC_VPHADDW:
            return Mnemonic::Vphaddw;
        case ZYDIS_MNEMONIC_VPHADDWD:
            return Mnemonic::Vphaddwd;
        case ZYDIS_MNEMONIC_VPHADDWQ:
            return Mnemonic::Vphaddwq;
        case ZYDIS_MNEMONIC_VPHMINPOSUW:
            return Mnemonic::Vphminposuw;
        case ZYDIS_MNEMONIC_VPHSUBBW:
            return Mnemonic::Vphsubbw;
        case ZYDIS_MNEMONIC_VPHSUBD:
            return Mnemonic::Vphsubd;
        case ZYDIS_MNEMONIC_VPHSUBDQ:
            return Mnemonic::Vphsubdq;
        case ZYDIS_MNEMONIC_VPHSUBSW:
            return Mnemonic::Vphsubsw;
        case ZYDIS_MNEMONIC_VPHSUBW:
            return Mnemonic::Vphsubw;
        case ZYDIS_MNEMONIC_VPHSUBWD:
            return Mnemonic::Vphsubwd;
        case ZYDIS_MNEMONIC_VPINSRB:
            return Mnemonic::Vpinsrb;
        case ZYDIS_MNEMONIC_VPINSRD:
            return Mnemonic::Vpinsrd;
        case ZYDIS_MNEMONIC_VPINSRQ:
            return Mnemonic::Vpinsrq;
        case ZYDIS_MNEMONIC_VPINSRW:
            return Mnemonic::Vpinsrw;
        case ZYDIS_MNEMONIC_VPLZCNTD:
            return Mnemonic::Vplzcntd;
        case ZYDIS_MNEMONIC_VPLZCNTQ:
            return Mnemonic::Vplzcntq;
        case ZYDIS_MNEMONIC_VPMACSDD:
            return Mnemonic::Vpmacsdd;
        case ZYDIS_MNEMONIC_VPMACSDQH:
            return Mnemonic::Vpmacsdqh;
        case ZYDIS_MNEMONIC_VPMACSDQL:
            return Mnemonic::Vpmacsdql;
        case ZYDIS_MNEMONIC_VPMACSSDD:
            return Mnemonic::Vpmacssdd;
        case ZYDIS_MNEMONIC_VPMACSSDQH:
            return Mnemonic::Vpmacssdqh;
        case ZYDIS_MNEMONIC_VPMACSSDQL:
            return Mnemonic::Vpmacssdql;
        case ZYDIS_MNEMONIC_VPMACSSWD:
            return Mnemonic::Vpmacsswd;
        case ZYDIS_MNEMONIC_VPMACSSWW:
            return Mnemonic::Vpmacssww;
        case ZYDIS_MNEMONIC_VPMACSWD:
            return Mnemonic::Vpmacswd;
        case ZYDIS_MNEMONIC_VPMACSWW:
            return Mnemonic::Vpmacsww;
        case ZYDIS_MNEMONIC_VPMADCSSWD:
            return Mnemonic::Vpmadcsswd;
        case ZYDIS_MNEMONIC_VPMADCSWD:
            return Mnemonic::Vpmadcswd;
        case ZYDIS_MNEMONIC_VPMADD231D:
            return Mnemonic::Vpmadd231d;
        case ZYDIS_MNEMONIC_VPMADD233D:
            return Mnemonic::Vpmadd233d;
        case ZYDIS_MNEMONIC_VPMADD52HUQ:
            return Mnemonic::Vpmadd52huq;
        case ZYDIS_MNEMONIC_VPMADD52LUQ:
            return Mnemonic::Vpmadd52luq;
        case ZYDIS_MNEMONIC_VPMADDUBSW:
            return Mnemonic::Vpmaddubsw;
        case ZYDIS_MNEMONIC_VPMADDWD:
            return Mnemonic::Vpmaddwd;
        case ZYDIS_MNEMONIC_VPMASKMOVD:
            return Mnemonic::Vpmaskmovd;
        case ZYDIS_MNEMONIC_VPMASKMOVQ:
            return Mnemonic::Vpmaskmovq;
        case ZYDIS_MNEMONIC_VPMAXSB:
            return Mnemonic::Vpmaxsb;
        case ZYDIS_MNEMONIC_VPMAXSD:
            return Mnemonic::Vpmaxsd;
        case ZYDIS_MNEMONIC_VPMAXSQ:
            return Mnemonic::Vpmaxsq;
        case ZYDIS_MNEMONIC_VPMAXSW:
            return Mnemonic::Vpmaxsw;
        case ZYDIS_MNEMONIC_VPMAXUB:
            return Mnemonic::Vpmaxub;
        case ZYDIS_MNEMONIC_VPMAXUD:
            return Mnemonic::Vpmaxud;
        case ZYDIS_MNEMONIC_VPMAXUQ:
            return Mnemonic::Vpmaxuq;
        case ZYDIS_MNEMONIC_VPMAXUW:
            return Mnemonic::Vpmaxuw;
        case ZYDIS_MNEMONIC_VPMINSB:
            return Mnemonic::Vpminsb;
        case ZYDIS_MNEMONIC_VPMINSD:
            return Mnemonic::Vpminsd;
        case ZYDIS_MNEMONIC_VPMINSQ:
            return Mnemonic::Vpminsq;
        case ZYDIS_MNEMONIC_VPMINSW:
            return Mnemonic::Vpminsw;
        case ZYDIS_MNEMONIC_VPMINUB:
            return Mnemonic::Vpminub;
        case ZYDIS_MNEMONIC_VPMINUD:
            return Mnemonic::Vpminud;
        case ZYDIS_MNEMONIC_VPMINUQ:
            return Mnemonic::Vpminuq;
        case ZYDIS_MNEMONIC_VPMINUW:
            return Mnemonic::Vpminuw;
        case ZYDIS_MNEMONIC_VPMOVB2M:
            return Mnemonic::Vpmovb2m;
        case ZYDIS_MNEMONIC_VPMOVD2M:
            return Mnemonic::Vpmovd2m;
        case ZYDIS_MNEMONIC_VPMOVDB:
            return Mnemonic::Vpmovdb;
        case ZYDIS_MNEMONIC_VPMOVDW:
            return Mnemonic::Vpmovdw;
        case ZYDIS_MNEMONIC_VPMOVM2B:
            return Mnemonic::Vpmovm2b;
        case ZYDIS_MNEMONIC_VPMOVM2D:
            return Mnemonic::Vpmovm2d;
        case ZYDIS_MNEMONIC_VPMOVM2Q:
            return Mnemonic::Vpmovm2q;
        case ZYDIS_MNEMONIC_VPMOVM2W:
            return Mnemonic::Vpmovm2w;
        case ZYDIS_MNEMONIC_VPMOVMSKB:
            return Mnemonic::Vpmovmskb;
        case ZYDIS_MNEMONIC_VPMOVQ2M:
            return Mnemonic::Vpmovq2m;
        case ZYDIS_MNEMONIC_VPMOVQB:
            return Mnemonic::Vpmovqb;
        case ZYDIS_MNEMONIC_VPMOVQD:
            return Mnemonic::Vpmovqd;
        case ZYDIS_MNEMONIC_VPMOVQW:
            return Mnemonic::Vpmovqw;
        case ZYDIS_MNEMONIC_VPMOVSDB:
            return Mnemonic::Vpmovsdb;
        case ZYDIS_MNEMONIC_VPMOVSDW:
            return Mnemonic::Vpmovsdw;
        case ZYDIS_MNEMONIC_VPMOVSQB:
            return Mnemonic::Vpmovsqb;
        case ZYDIS_MNEMONIC_VPMOVSQD:
            return Mnemonic::Vpmovsqd;
        case ZYDIS_MNEMONIC_VPMOVSQW:
            return Mnemonic::Vpmovsqw;
        case ZYDIS_MNEMONIC_VPMOVSWB:
            return Mnemonic::Vpmovswb;
        case ZYDIS_MNEMONIC_VPMOVSXBD:
            return Mnemonic::Vpmovsxbd;
        case ZYDIS_MNEMONIC_VPMOVSXBQ:
            return Mnemonic::Vpmovsxbq;
        case ZYDIS_MNEMONIC_VPMOVSXBW:
            return Mnemonic::Vpmovsxbw;
        case ZYDIS_MNEMONIC_VPMOVSXDQ:
            return Mnemonic::Vpmovsxdq;
        case ZYDIS_MNEMONIC_VPMOVSXWD:
            return Mnemonic::Vpmovsxwd;
        case ZYDIS_MNEMONIC_VPMOVSXWQ:
            return Mnemonic::Vpmovsxwq;
        case ZYDIS_MNEMONIC_VPMOVUSDB:
            return Mnemonic::Vpmovusdb;
        case ZYDIS_MNEMONIC_VPMOVUSDW:
            return Mnemonic::Vpmovusdw;
        case ZYDIS_MNEMONIC_VPMOVUSQB:
            return Mnemonic::Vpmovusqb;
        case ZYDIS_MNEMONIC_VPMOVUSQD:
            return Mnemonic::Vpmovusqd;
        case ZYDIS_MNEMONIC_VPMOVUSQW:
            return Mnemonic::Vpmovusqw;
        case ZYDIS_MNEMONIC_VPMOVUSWB:
            return Mnemonic::Vpmovuswb;
        case ZYDIS_MNEMONIC_VPMOVW2M:
            return Mnemonic::Vpmovw2m;
        case ZYDIS_MNEMONIC_VPMOVWB:
            return Mnemonic::Vpmovwb;
        case ZYDIS_MNEMONIC_VPMOVZXBD:
            return Mnemonic::Vpmovzxbd;
        case ZYDIS_MNEMONIC_VPMOVZXBQ:
            return Mnemonic::Vpmovzxbq;
        case ZYDIS_MNEMONIC_VPMOVZXBW:
            return Mnemonic::Vpmovzxbw;
        case ZYDIS_MNEMONIC_VPMOVZXDQ:
            return Mnemonic::Vpmovzxdq;
        case ZYDIS_MNEMONIC_VPMOVZXWD:
            return Mnemonic::Vpmovzxwd;
        case ZYDIS_MNEMONIC_VPMOVZXWQ:
            return Mnemonic::Vpmovzxwq;
        case ZYDIS_MNEMONIC_VPMULDQ:
            return Mnemonic::Vpmuldq;
        case ZYDIS_MNEMONIC_VPMULHD:
            return Mnemonic::Vpmulhd;
        case ZYDIS_MNEMONIC_VPMULHRSW:
            return Mnemonic::Vpmulhrsw;
        case ZYDIS_MNEMONIC_VPMULHUD:
            return Mnemonic::Vpmulhud;
        case ZYDIS_MNEMONIC_VPMULHUW:
            return Mnemonic::Vpmulhuw;
        case ZYDIS_MNEMONIC_VPMULHW:
            return Mnemonic::Vpmulhw;
        case ZYDIS_MNEMONIC_VPMULLD:
            return Mnemonic::Vpmulld;
        case ZYDIS_MNEMONIC_VPMULLQ:
            return Mnemonic::Vpmullq;
        case ZYDIS_MNEMONIC_VPMULLW:
            return Mnemonic::Vpmullw;
        case ZYDIS_MNEMONIC_VPMULTISHIFTQB:
            return Mnemonic::Vpmultishiftqb;
        case ZYDIS_MNEMONIC_VPMULUDQ:
            return Mnemonic::Vpmuludq;
        case ZYDIS_MNEMONIC_VPOPCNTB:
            return Mnemonic::Vpopcntb;
        case ZYDIS_MNEMONIC_VPOPCNTD:
            return Mnemonic::Vpopcntd;
        case ZYDIS_MNEMONIC_VPOPCNTQ:
            return Mnemonic::Vpopcntq;
        case ZYDIS_MNEMONIC_VPOPCNTW:
            return Mnemonic::Vpopcntw;
        case ZYDIS_MNEMONIC_VPOR:
            return Mnemonic::Vpor;
        case ZYDIS_MNEMONIC_VPORD:
            return Mnemonic::Vpord;
        case ZYDIS_MNEMONIC_VPORQ:
            return Mnemonic::Vporq;
        case ZYDIS_MNEMONIC_VPPERM:
            return Mnemonic::Vpperm;
        case ZYDIS_MNEMONIC_VPREFETCH0:
            return Mnemonic::Vprefetch0;
        case ZYDIS_MNEMONIC_VPREFETCH1:
            return Mnemonic::Vprefetch1;
        case ZYDIS_MNEMONIC_VPREFETCH2:
            return Mnemonic::Vprefetch2;
        case ZYDIS_MNEMONIC_VPREFETCHE0:
            return Mnemonic::Vprefetche0;
        case ZYDIS_MNEMONIC_VPREFETCHE1:
            return Mnemonic::Vprefetche1;
        case ZYDIS_MNEMONIC_VPREFETCHE2:
            return Mnemonic::Vprefetche2;
        case ZYDIS_MNEMONIC_VPREFETCHENTA:
            return Mnemonic::Vprefetchenta;
        case ZYDIS_MNEMONIC_VPREFETCHNTA:
            return Mnemonic::Vprefetchnta;
        case ZYDIS_MNEMONIC_VPROLD:
            return Mnemonic::Vprold;
        case ZYDIS_MNEMONIC_VPROLQ:
            return Mnemonic::Vprolq;
        case ZYDIS_MNEMONIC_VPROLVD:
            return Mnemonic::Vprolvd;
        case ZYDIS_MNEMONIC_VPROLVQ:
            return Mnemonic::Vprolvq;
        case ZYDIS_MNEMONIC_VPRORD:
            return Mnemonic::Vprord;
        case ZYDIS_MNEMONIC_VPRORQ:
            return Mnemonic::Vprorq;
        case ZYDIS_MNEMONIC_VPRORVD:
            return Mnemonic::Vprorvd;
        case ZYDIS_MNEMONIC_VPRORVQ:
            return Mnemonic::Vprorvq;
        case ZYDIS_MNEMONIC_VPROTB:
            return Mnemonic::Vprotb;
        case ZYDIS_MNEMONIC_VPROTD:
            return Mnemonic::Vprotd;
        case ZYDIS_MNEMONIC_VPROTQ:
            return Mnemonic::Vprotq;
        case ZYDIS_MNEMONIC_VPROTW:
            return Mnemonic::Vprotw;
        case ZYDIS_MNEMONIC_VPSADBW:
            return Mnemonic::Vpsadbw;
        case ZYDIS_MNEMONIC_VPSBBD:
            return Mnemonic::Vpsbbd;
        case ZYDIS_MNEMONIC_VPSBBRD:
            return Mnemonic::Vpsbbrd;
        case ZYDIS_MNEMONIC_VPSCATTERDD:
            return Mnemonic::Vpscatterdd;
        case ZYDIS_MNEMONIC_VPSCATTERDQ:
            return Mnemonic::Vpscatterdq;
        case ZYDIS_MNEMONIC_VPSCATTERQD:
            return Mnemonic::Vpscatterqd;
        case ZYDIS_MNEMONIC_VPSCATTERQQ:
            return Mnemonic::Vpscatterqq;
        case ZYDIS_MNEMONIC_VPSHAB:
            return Mnemonic::Vpshab;
        case ZYDIS_MNEMONIC_VPSHAD:
            return Mnemonic::Vpshad;
        case ZYDIS_MNEMONIC_VPSHAQ:
            return Mnemonic::Vpshaq;
        case ZYDIS_MNEMONIC_VPSHAW:
            return Mnemonic::Vpshaw;
        case ZYDIS_MNEMONIC_VPSHLB:
            return Mnemonic::Vpshlb;
        case ZYDIS_MNEMONIC_VPSHLD:
            return Mnemonic::Vpshld;
        case ZYDIS_MNEMONIC_VPSHLDD:
            return Mnemonic::Vpshldd;
        case ZYDIS_MNEMONIC_VPSHLDQ:
            return Mnemonic::Vpshldq;
        case ZYDIS_MNEMONIC_VPSHLDVD:
            return Mnemonic::Vpshldvd;
        case ZYDIS_MNEMONIC_VPSHLDVQ:
            return Mnemonic::Vpshldvq;
        case ZYDIS_MNEMONIC_VPSHLDVW:
            return Mnemonic::Vpshldvw;
        case ZYDIS_MNEMONIC_VPSHLDW:
            return Mnemonic::Vpshldw;
        case ZYDIS_MNEMONIC_VPSHLQ:
            return Mnemonic::Vpshlq;
        case ZYDIS_MNEMONIC_VPSHLW:
            return Mnemonic::Vpshlw;
        case ZYDIS_MNEMONIC_VPSHRDD:
            return Mnemonic::Vpshrdd;
        case ZYDIS_MNEMONIC_VPSHRDQ:
            return Mnemonic::Vpshrdq;
        case ZYDIS_MNEMONIC_VPSHRDVD:
            return Mnemonic::Vpshrdvd;
        case ZYDIS_MNEMONIC_VPSHRDVQ:
            return Mnemonic::Vpshrdvq;
        case ZYDIS_MNEMONIC_VPSHRDVW:
            return Mnemonic::Vpshrdvw;
        case ZYDIS_MNEMONIC_VPSHRDW:
            return Mnemonic::Vpshrdw;
        case ZYDIS_MNEMONIC_VPSHUFB:
            return Mnemonic::Vpshufb;
        case ZYDIS_MNEMONIC_VPSHUFBITQMB:
            return Mnemonic::Vpshufbitqmb;
        case ZYDIS_MNEMONIC_VPSHUFD:
            return Mnemonic::Vpshufd;
        case ZYDIS_MNEMONIC_VPSHUFHW:
            return Mnemonic::Vpshufhw;
        case ZYDIS_MNEMONIC_VPSHUFLW:
            return Mnemonic::Vpshuflw;
        case ZYDIS_MNEMONIC_VPSIGNB:
            return Mnemonic::Vpsignb;
        case ZYDIS_MNEMONIC_VPSIGND:
            return Mnemonic::Vpsignd;
        case ZYDIS_MNEMONIC_VPSIGNW:
            return Mnemonic::Vpsignw;
        case ZYDIS_MNEMONIC_VPSLLD:
            return Mnemonic::Vpslld;
        case ZYDIS_MNEMONIC_VPSLLDQ:
            return Mnemonic::Vpslldq;
        case ZYDIS_MNEMONIC_VPSLLQ:
            return Mnemonic::Vpsllq;
        case ZYDIS_MNEMONIC_VPSLLVD:
            return Mnemonic::Vpsllvd;
        case ZYDIS_MNEMONIC_VPSLLVQ:
            return Mnemonic::Vpsllvq;
        case ZYDIS_MNEMONIC_VPSLLVW:
            return Mnemonic::Vpsllvw;
        case ZYDIS_MNEMONIC_VPSLLW:
            return Mnemonic::Vpsllw;
        case ZYDIS_MNEMONIC_VPSRAD:
            return Mnemonic::Vpsrad;
        case ZYDIS_MNEMONIC_VPSRAQ:
            return Mnemonic::Vpsraq;
        case ZYDIS_MNEMONIC_VPSRAVD:
            return Mnemonic::Vpsravd;
        case ZYDIS_MNEMONIC_VPSRAVQ:
            return Mnemonic::Vpsravq;
        case ZYDIS_MNEMONIC_VPSRAVW:
            return Mnemonic::Vpsravw;
        case ZYDIS_MNEMONIC_VPSRAW:
            return Mnemonic::Vpsraw;
        case ZYDIS_MNEMONIC_VPSRLD:
            return Mnemonic::Vpsrld;
        case ZYDIS_MNEMONIC_VPSRLDQ:
            return Mnemonic::Vpsrldq;
        case ZYDIS_MNEMONIC_VPSRLQ:
            return Mnemonic::Vpsrlq;
        case ZYDIS_MNEMONIC_VPSRLVD:
            return Mnemonic::Vpsrlvd;
        case ZYDIS_MNEMONIC_VPSRLVQ:
            return Mnemonic::Vpsrlvq;
        case ZYDIS_MNEMONIC_VPSRLVW:
            return Mnemonic::Vpsrlvw;
        case ZYDIS_MNEMONIC_VPSRLW:
            return Mnemonic::Vpsrlw;
        case ZYDIS_MNEMONIC_VPSUBB:
            return Mnemonic::Vpsubb;
        case ZYDIS_MNEMONIC_VPSUBD:
            return Mnemonic::Vpsubd;
        case ZYDIS_MNEMONIC_VPSUBQ:
            return Mnemonic::Vpsubq;
        case ZYDIS_MNEMONIC_VPSUBRD:
            return Mnemonic::Vpsubrd;
        case ZYDIS_MNEMONIC_VPSUBRSETBD:
            return Mnemonic::Vpsubrsetbd;
        case ZYDIS_MNEMONIC_VPSUBSB:
            return Mnemonic::Vpsubsb;
        case ZYDIS_MNEMONIC_VPSUBSETBD:
            return Mnemonic::Vpsubsetbd;
        case ZYDIS_MNEMONIC_VPSUBSW:
            return Mnemonic::Vpsubsw;
        case ZYDIS_MNEMONIC_VPSUBUSB:
            return Mnemonic::Vpsubusb;
        case ZYDIS_MNEMONIC_VPSUBUSW:
            return Mnemonic::Vpsubusw;
        case ZYDIS_MNEMONIC_VPSUBW:
            return Mnemonic::Vpsubw;
        case ZYDIS_MNEMONIC_VPTERNLOGD:
            return Mnemonic::Vpternlogd;
        case ZYDIS_MNEMONIC_VPTERNLOGQ:
            return Mnemonic::Vpternlogq;
        case ZYDIS_MNEMONIC_VPTEST:
            return Mnemonic::Vptest;
        case ZYDIS_MNEMONIC_VPTESTMB:
            return Mnemonic::Vptestmb;
        case ZYDIS_MNEMONIC_VPTESTMD:
            return Mnemonic::Vptestmd;
        case ZYDIS_MNEMONIC_VPTESTMQ:
            return Mnemonic::Vptestmq;
        case ZYDIS_MNEMONIC_VPTESTMW:
            return Mnemonic::Vptestmw;
        case ZYDIS_MNEMONIC_VPTESTNMB:
            return Mnemonic::Vptestnmb;
        case ZYDIS_MNEMONIC_VPTESTNMD:
            return Mnemonic::Vptestnmd;
        case ZYDIS_MNEMONIC_VPTESTNMQ:
            return Mnemonic::Vptestnmq;
        case ZYDIS_MNEMONIC_VPTESTNMW:
            return Mnemonic::Vptestnmw;
        case ZYDIS_MNEMONIC_VPUNPCKHBW:
            return Mnemonic::Vpunpckhbw;
        case ZYDIS_MNEMONIC_VPUNPCKHDQ:
            return Mnemonic::Vpunpckhdq;
        case ZYDIS_MNEMONIC_VPUNPCKHQDQ:
            return Mnemonic::Vpunpckhqdq;
        case ZYDIS_MNEMONIC_VPUNPCKHWD:
            return Mnemonic::Vpunpckhwd;
        case ZYDIS_MNEMONIC_VPUNPCKLBW:
            return Mnemonic::Vpunpcklbw;
        case ZYDIS_MNEMONIC_VPUNPCKLDQ:
            return Mnemonic::Vpunpckldq;
        case ZYDIS_MNEMONIC_VPUNPCKLQDQ:
            return Mnemonic::Vpunpcklqdq;
        case ZYDIS_MNEMONIC_VPUNPCKLWD:
            return Mnemonic::Vpunpcklwd;
        case ZYDIS_MNEMONIC_VPXOR:
            return Mnemonic::Vpxor;
        case ZYDIS_MNEMONIC_VPXORD:
            return Mnemonic::Vpxord;
        case ZYDIS_MNEMONIC_VPXORQ:
            return Mnemonic::Vpxorq;
        case ZYDIS_MNEMONIC_VRANGEPD:
            return Mnemonic::Vrangepd;
        case ZYDIS_MNEMONIC_VRANGEPS:
            return Mnemonic::Vrangeps;
        case ZYDIS_MNEMONIC_VRANGESD:
            return Mnemonic::Vrangesd;
        case ZYDIS_MNEMONIC_VRANGESS:
            return Mnemonic::Vrangess;
        case ZYDIS_MNEMONIC_VRCP14PD:
            return Mnemonic::Vrcp14pd;
        case ZYDIS_MNEMONIC_VRCP14PS:
            return Mnemonic::Vrcp14ps;
        case ZYDIS_MNEMONIC_VRCP14SD:
            return Mnemonic::Vrcp14sd;
        case ZYDIS_MNEMONIC_VRCP14SS:
            return Mnemonic::Vrcp14ss;
        case ZYDIS_MNEMONIC_VRCP23PS:
            return Mnemonic::Vrcp23ps;
        case ZYDIS_MNEMONIC_VRCP28PD:
            return Mnemonic::Vrcp28pd;
        case ZYDIS_MNEMONIC_VRCP28PS:
            return Mnemonic::Vrcp28ps;
        case ZYDIS_MNEMONIC_VRCP28SD:
            return Mnemonic::Vrcp28sd;
        case ZYDIS_MNEMONIC_VRCP28SS:
            return Mnemonic::Vrcp28ss;
        case ZYDIS_MNEMONIC_VRCPPH:
            return Mnemonic::Vrcpph;
        case ZYDIS_MNEMONIC_VRCPPS:
            return Mnemonic::Vrcpps;
        case ZYDIS_MNEMONIC_VRCPSH:
            return Mnemonic::Vrcpsh;
        case ZYDIS_MNEMONIC_VRCPSS:
            return Mnemonic::Vrcpss;
        case ZYDIS_MNEMONIC_VREDUCEPD:
            return Mnemonic::Vreducepd;
        case ZYDIS_MNEMONIC_VREDUCEPH:
            return Mnemonic::Vreduceph;
        case ZYDIS_MNEMONIC_VREDUCEPS:
            return Mnemonic::Vreduceps;
        case ZYDIS_MNEMONIC_VREDUCESD:
            return Mnemonic::Vreducesd;
        case ZYDIS_MNEMONIC_VREDUCESH:
            return Mnemonic::Vreducesh;
        case ZYDIS_MNEMONIC_VREDUCESS:
            return Mnemonic::Vreducess;
        case ZYDIS_MNEMONIC_VRNDFXPNTPD:
            return Mnemonic::Vrndfxpntpd;
        case ZYDIS_MNEMONIC_VRNDFXPNTPS:
            return Mnemonic::Vrndfxpntps;
        case ZYDIS_MNEMONIC_VRNDSCALEPD:
            return Mnemonic::Vrndscalepd;
        case ZYDIS_MNEMONIC_VRNDSCALEPH:
            return Mnemonic::Vrndscaleph;
        case ZYDIS_MNEMONIC_VRNDSCALEPS:
            return Mnemonic::Vrndscaleps;
        case ZYDIS_MNEMONIC_VRNDSCALESD:
            return Mnemonic::Vrndscalesd;
        case ZYDIS_MNEMONIC_VRNDSCALESH:
            return Mnemonic::Vrndscalesh;
        case ZYDIS_MNEMONIC_VRNDSCALESS:
            return Mnemonic::Vrndscaless;
        case ZYDIS_MNEMONIC_VROUNDPD:
            return Mnemonic::Vroundpd;
        case ZYDIS_MNEMONIC_VROUNDPS:
            return Mnemonic::Vroundps;
        case ZYDIS_MNEMONIC_VROUNDSD:
            return Mnemonic::Vroundsd;
        case ZYDIS_MNEMONIC_VROUNDSS:
            return Mnemonic::Vroundss;
        case ZYDIS_MNEMONIC_VRSQRT14PD:
            return Mnemonic::Vrsqrt14pd;
        case ZYDIS_MNEMONIC_VRSQRT14PS:
            return Mnemonic::Vrsqrt14ps;
        case ZYDIS_MNEMONIC_VRSQRT14SD:
            return Mnemonic::Vrsqrt14sd;
        case ZYDIS_MNEMONIC_VRSQRT14SS:
            return Mnemonic::Vrsqrt14ss;
        case ZYDIS_MNEMONIC_VRSQRT23PS:
            return Mnemonic::Vrsqrt23ps;
        case ZYDIS_MNEMONIC_VRSQRT28PD:
            return Mnemonic::Vrsqrt28pd;
        case ZYDIS_MNEMONIC_VRSQRT28PS:
            return Mnemonic::Vrsqrt28ps;
        case ZYDIS_MNEMONIC_VRSQRT28SD:
            return Mnemonic::Vrsqrt28sd;
        case ZYDIS_MNEMONIC_VRSQRT28SS:
            return Mnemonic::Vrsqrt28ss;
        case ZYDIS_MNEMONIC_VRSQRTPH:
            return Mnemonic::Vrsqrtph;
        case ZYDIS_MNEMONIC_VRSQRTPS:
            return Mnemonic::Vrsqrtps;
        case ZYDIS_MNEMONIC_VRSQRTSH:
            return Mnemonic::Vrsqrtsh;
        case ZYDIS_MNEMONIC_VRSQRTSS:
            return Mnemonic::Vrsqrtss;
        case ZYDIS_MNEMONIC_VSCALEFPD:
            return Mnemonic::Vscalefpd;
        case ZYDIS_MNEMONIC_VSCALEFPH:
            return Mnemonic::Vscalefph;
        case ZYDIS_MNEMONIC_VSCALEFPS:
            return Mnemonic::Vscalefps;
        case ZYDIS_MNEMONIC_VSCALEFSD:
            return Mnemonic::Vscalefsd;
        case ZYDIS_MNEMONIC_VSCALEFSH:
            return Mnemonic::Vscalefsh;
        case ZYDIS_MNEMONIC_VSCALEFSS:
            return Mnemonic::Vscalefss;
        case ZYDIS_MNEMONIC_VSCALEPS:
            return Mnemonic::Vscaleps;
        case ZYDIS_MNEMONIC_VSCATTERDPD:
            return Mnemonic::Vscatterdpd;
        case ZYDIS_MNEMONIC_VSCATTERDPS:
            return Mnemonic::Vscatterdps;
        case ZYDIS_MNEMONIC_VSCATTERPF0DPD:
            return Mnemonic::Vscatterpf0dpd;
        case ZYDIS_MNEMONIC_VSCATTERPF0DPS:
            return Mnemonic::Vscatterpf0dps;
        case ZYDIS_MNEMONIC_VSCATTERPF0HINTDPD:
            return Mnemonic::Vscatterpf0hintdpd;
        case ZYDIS_MNEMONIC_VSCATTERPF0HINTDPS:
            return Mnemonic::Vscatterpf0hintdps;
        case ZYDIS_MNEMONIC_VSCATTERPF0QPD:
            return Mnemonic::Vscatterpf0qpd;
        case ZYDIS_MNEMONIC_VSCATTERPF0QPS:
            return Mnemonic::Vscatterpf0qps;
        case ZYDIS_MNEMONIC_VSCATTERPF1DPD:
            return Mnemonic::Vscatterpf1dpd;
        case ZYDIS_MNEMONIC_VSCATTERPF1DPS:
            return Mnemonic::Vscatterpf1dps;
        case ZYDIS_MNEMONIC_VSCATTERPF1QPD:
            return Mnemonic::Vscatterpf1qpd;
        case ZYDIS_MNEMONIC_VSCATTERPF1QPS:
            return Mnemonic::Vscatterpf1qps;
        case ZYDIS_MNEMONIC_VSCATTERQPD:
            return Mnemonic::Vscatterqpd;
        case ZYDIS_MNEMONIC_VSCATTERQPS:
            return Mnemonic::Vscatterqps;
        case ZYDIS_MNEMONIC_VSHUFF32X4:
            return Mnemonic::Vshuff32x4;
        case ZYDIS_MNEMONIC_VSHUFF64X2:
            return Mnemonic::Vshuff64x2;
        case ZYDIS_MNEMONIC_VSHUFI32X4:
            return Mnemonic::Vshufi32x4;
        case ZYDIS_MNEMONIC_VSHUFI64X2:
            return Mnemonic::Vshufi64x2;
        case ZYDIS_MNEMONIC_VSHUFPD:
            return Mnemonic::Vshufpd;
        case ZYDIS_MNEMONIC_VSHUFPS:
            return Mnemonic::Vshufps;
        case ZYDIS_MNEMONIC_VSQRTPD:
            return Mnemonic::Vsqrtpd;
        case ZYDIS_MNEMONIC_VSQRTPH:
            return Mnemonic::Vsqrtph;
        case ZYDIS_MNEMONIC_VSQRTPS:
            return Mnemonic::Vsqrtps;
        case ZYDIS_MNEMONIC_VSQRTSD:
            return Mnemonic::Vsqrtsd;
        case ZYDIS_MNEMONIC_VSQRTSH:
            return Mnemonic::Vsqrtsh;
        case ZYDIS_MNEMONIC_VSQRTSS:
            return Mnemonic::Vsqrtss;
        case ZYDIS_MNEMONIC_VSTMXCSR:
            return Mnemonic::Vstmxcsr;
        case ZYDIS_MNEMONIC_VSUBPD:
            return Mnemonic::Vsubpd;
        case ZYDIS_MNEMONIC_VSUBPH:
            return Mnemonic::Vsubph;
        case ZYDIS_MNEMONIC_VSUBPS:
            return Mnemonic::Vsubps;
        case ZYDIS_MNEMONIC_VSUBRPD:
            return Mnemonic::Vsubrpd;
        case ZYDIS_MNEMONIC_VSUBRPS:
            return Mnemonic::Vsubrps;
        case ZYDIS_MNEMONIC_VSUBSD:
            return Mnemonic::Vsubsd;
        case ZYDIS_MNEMONIC_VSUBSH:
            return Mnemonic::Vsubsh;
        case ZYDIS_MNEMONIC_VSUBSS:
            return Mnemonic::Vsubss;
        case ZYDIS_MNEMONIC_VTESTPD:
            return Mnemonic::Vtestpd;
        case ZYDIS_MNEMONIC_VTESTPS:
            return Mnemonic::Vtestps;
        case ZYDIS_MNEMONIC_VUCOMISD:
            return Mnemonic::Vucomisd;
        case ZYDIS_MNEMONIC_VUCOMISH:
            return Mnemonic::Vucomish;
        case ZYDIS_MNEMONIC_VUCOMISS:
            return Mnemonic::Vucomiss;
        case ZYDIS_MNEMONIC_VUNPCKHPD:
            return Mnemonic::Vunpckhpd;
        case ZYDIS_MNEMONIC_VUNPCKHPS:
            return Mnemonic::Vunpckhps;
        case ZYDIS_MNEMONIC_VUNPCKLPD:
            return Mnemonic::Vunpcklpd;
        case ZYDIS_MNEMONIC_VUNPCKLPS:
            return Mnemonic::Vunpcklps;
        case ZYDIS_MNEMONIC_VXORPD:
            return Mnemonic::Vxorpd;
        case ZYDIS_MNEMONIC_VXORPS:
            return Mnemonic::Vxorps;
        case ZYDIS_MNEMONIC_VZEROALL:
            return Mnemonic::Vzeroall;
        case ZYDIS_MNEMONIC_VZEROUPPER:
            return Mnemonic::Vzeroupper;
        case ZYDIS_MNEMONIC_WBINVD:
            return Mnemonic::Wbinvd;
        case ZYDIS_MNEMONIC_WRFSBASE:
            return Mnemonic::Wrfsbase;
        case ZYDIS_MNEMONIC_WRGSBASE:
            return Mnemonic::Wrgsbase;
        case ZYDIS_MNEMONIC_WRMSR:
            return Mnemonic::Wrmsr;
        case ZYDIS_MNEMONIC_WRPKRU:
            return Mnemonic::Wrpkru;
        case ZYDIS_MNEMONIC_WRSSD:
            return Mnemonic::Wrssd;
        case ZYDIS_MNEMONIC_WRSSQ:
            return Mnemonic::Wrssq;
        case ZYDIS_MNEMONIC_WRUSSD:
            return Mnemonic::Wrussd;
        case ZYDIS_MNEMONIC_WRUSSQ:
            return Mnemonic::Wrussq;
        case ZYDIS_MNEMONIC_XABORT:
            return Mnemonic::Xabort;
        case ZYDIS_MNEMONIC_XADD:
            return Mnemonic::Xadd;
        case ZYDIS_MNEMONIC_XBEGIN:
            return Mnemonic::Xbegin;
        case ZYDIS_MNEMONIC_XCHG:
            return Mnemonic::Xchg;
        case ZYDIS_MNEMONIC_XCRYPT_CBC:
            return Mnemonic::Xcrypt_cbc;
        case ZYDIS_MNEMONIC_XCRYPT_CFB:
            return Mnemonic::Xcrypt_cfb;
        case ZYDIS_MNEMONIC_XCRYPT_CTR:
            return Mnemonic::Xcrypt_ctr;
        case ZYDIS_MNEMONIC_XCRYPT_ECB:
            return Mnemonic::Xcrypt_ecb;
        case ZYDIS_MNEMONIC_XCRYPT_OFB:
            return Mnemonic::Xcrypt_ofb;
        case ZYDIS_MNEMONIC_XEND:
            return Mnemonic::Xend;
        case ZYDIS_MNEMONIC_XGETBV:
            return Mnemonic::Xgetbv;
        case ZYDIS_MNEMONIC_XLAT:
            return Mnemonic::Xlat;
        case ZYDIS_MNEMONIC_XOR:
            return Mnemonic::Xor;
        case ZYDIS_MNEMONIC_XORPD:
            return Mnemonic::Xorpd;
        case ZYDIS_MNEMONIC_XORPS:
            return Mnemonic::Xorps;
        case ZYDIS_MNEMONIC_XRESLDTRK:
            return Mnemonic::Xresldtrk;
        case ZYDIS_MNEMONIC_XRSTOR:
            return Mnemonic::Xrstor;
        case ZYDIS_MNEMONIC_XRSTOR64:
            return Mnemonic::Xrstor64;
        case ZYDIS_MNEMONIC_XRSTORS:
            return Mnemonic::Xrstors;
        case ZYDIS_MNEMONIC_XRSTORS64:
            return Mnemonic::Xrstors64;
        case ZYDIS_MNEMONIC_XSAVE:
            return Mnemonic::Xsave;
        case ZYDIS_MNEMONIC_XSAVE64:
            return Mnemonic::Xsave64;
        case ZYDIS_MNEMONIC_XSAVEC:
            return Mnemonic::Xsavec;
        case ZYDIS_MNEMONIC_XSAVEC64:
            return Mnemonic::Xsavec64;
        case ZYDIS_MNEMONIC_XSAVEOPT:
            return Mnemonic::Xsaveopt;
        case ZYDIS_MNEMONIC_XSAVEOPT64:
            return Mnemonic::Xsaveopt64;
        case ZYDIS_MNEMONIC_XSAVES:
            return Mnemonic::Xsaves;
        case ZYDIS_MNEMONIC_XSAVES64:
            return Mnemonic::Xsaves64;
        case ZYDIS_MNEMONIC_XSETBV:
            return Mnemonic::Xsetbv;
        case ZYDIS_MNEMONIC_XSHA1:
            return Mnemonic::Xsha1;
        case ZYDIS_MNEMONIC_XSHA256:
            return Mnemonic::Xsha256;
        case ZYDIS_MNEMONIC_XSTORE:
            return Mnemonic::Xstore;
        case ZYDIS_MNEMONIC_XSUSLDTRK:
            return Mnemonic::Xsusldtrk;
        case ZYDIS_MNEMONIC_XTEST:
            return Mnemonic::Xtest;
        }
        return Mnemonic::Invalid;
    }

} // namespace Dotx64Dbg
